@@ -74,8 +74,7 @@ public class ClassFileInput extends DataInputStream
     String name;
 
     ctype.thisClassIndex = readUnsignedShort();
-    clas = (CpoolClass) ctype.constants.getForced(ctype.thisClassIndex,
-						  ConstantPool.CLASS);
+    clas = getClassConstant(ctype.thisClassIndex);
     name = clas.name.string;
     ctype.this_name = name.replace('/', '.');
     ctype.setSignature("L"+name+";");
@@ -85,8 +84,7 @@ public class ClassFileInput extends DataInputStream
       ctype.setSuper((ClassType) null);
     else
       {
-	clas = (CpoolClass) ctype.constants.getForced(ctype.superClassIndex,
-						      ConstantPool.CLASS);
+	clas = getClassConstant(ctype.superClassIndex);
 	name = clas.name.string;
 	ctype.setSuper(name.replace('/', '.'));
       }
@@ -254,10 +252,19 @@ public class ClassFileInput extends DataInputStream
       }
     else if (name == "StackMapTable" && container instanceof CodeAttr)
       {
-        int count = readUnsignedShort();
-        byte[] data = new byte[length-2];
-        readFully(data, 0, length-2);
-        return new StackMapTableAttr(count, data, (CodeAttr) container);
+        byte[] data = new byte[length];
+        readFully(data, 0, length);
+        return new StackMapTableAttr(data, (CodeAttr) container);
+      }
+    else if ((name == "RuntimeVisibleAnnotations"
+              || name == "RuntimeInvisibleAnnotations")
+             && (container instanceof Field
+                 || container instanceof Method
+                 || container instanceof ClassType))
+      {
+        byte[] data = new byte[length];
+        readFully(data, 0, length);
+        return new RuntimeAnnotationsAttr(name, data, container);
       }
     else if (name == "ConstantValue" && container instanceof Field)
       {
@@ -338,4 +345,9 @@ public class ClassFileInput extends DataInputStream
 	readAttributes(meth);
       }
   }
+
+  CpoolClass getClassConstant (int index)
+  {
+    return (CpoolClass) ctype.constants.getForced(index, ConstantPool.CLASS);
+  }
 }
