@@ -54,6 +54,7 @@ public class ClassType extends ObjectType
     return (ClassType) Type.getType(name);
   }
 
+  /** @deprecated */
   public static ClassType make (String name, ClassType superClass)
   {
     ClassType type = make(name);
@@ -97,7 +98,7 @@ public class ClassType extends ObjectType
   }
 
   /** Return the modifiers (access flags) for this class. */
-  public final int getModifiers()
+  public final synchronized int getModifiers()
   {
     if (access_flags == 0
 	&& (flags & EXISTING_CLASS) != 0 && getReflectClass() != null)
@@ -113,7 +114,7 @@ public class ClassType extends ObjectType
   public final void setModifiers(int flags) { access_flags = flags; }
   public final void addModifiers(int flags) { access_flags |= flags; }
 
-  public String getSimpleName ()
+  public synchronized String getSimpleName ()
   {
     /* #ifdef JAVA5 */
     if ((flags & EXISTING_CLASS) != 0 && getReflectClass() != null)
@@ -201,7 +202,7 @@ public class ClassType extends ObjectType
     enclosingMember = member;
   }
 
-  void addEnclosingMember ()
+  synchronized void addEnclosingMember ()
   {
     if ((flags & (ADD_ENCLOSING_DONE|EXISTING_CLASS)) != EXISTING_CLASS)
       return;
@@ -236,7 +237,7 @@ public class ClassType extends ObjectType
     /* #endif */
   }
 
-  public void addMemberClasses ()
+  public synchronized void addMemberClasses ()
   {
     if ((flags & (ADD_MEMBERCLASSES_DONE|EXISTING_CLASS)) != EXISTING_CLASS)
       return;
@@ -402,7 +403,7 @@ public class ClassType extends ObjectType
     this.superClass = superClass;
   }
 
-  public ClassType getSuperclass ()
+  public synchronized ClassType getSuperclass ()
   {
     if (superClass == null
 	&& ! isInterface()
@@ -530,7 +531,7 @@ public class ClassType extends ObjectType
    *   Howeve, if mask is -1, ignore the access flags.
    * @return the matching field, or null if there is no such field.
    */
-  public Field getField(String name, int mask)
+  public synchronized Field getField(String name, int mask)
   {
     ClassType cl = this;
     for (;;)
@@ -596,7 +597,7 @@ public class ClassType extends ObjectType
    * Does not add private or package-private fields.
    * Does not check for duplicate (already-known) fields.
    * Is not thread-safe if another thread may access this ClassType. */
-  public void addFields()
+  public synchronized void addFields()
   {
     Class clas = getReflectClass();
     java.lang.reflect.Field[] fields;
@@ -660,7 +661,7 @@ public class ClassType extends ObjectType
     * If an existing method matches, return that.  Otherwise, create
     * a new one.
     * In contrast, the other addMethod methods always create new Methods. */
-  public Method addMethod (String name, int flags,
+  public synchronized Method addMethod (String name, int flags,
 			   Type[] arg_types, Type return_type)
   {
     Method method = getDeclaredMethod(name, arg_types);
@@ -850,6 +851,23 @@ public class ClassType extends ObjectType
     return count;
   }
 
+  static class AbstractMethodFilter implements gnu.bytecode.Filter
+  {
+    public static final AbstractMethodFilter instance
+      = new AbstractMethodFilter();
+
+    public boolean select(Object value)
+    {
+      gnu.bytecode.Method method = (gnu.bytecode.Method) value;
+      return method.isAbstract();
+    }
+  }
+
+  public Method[] getAbstractMethods ()
+  {
+    return getMethods(AbstractMethodFilter.instance, 2);
+  }
+
   /** Look for a matching method.
    * @param name method name
    * @param arg_types parameter types that must match.
@@ -890,7 +908,7 @@ public class ClassType extends ObjectType
   }
 
   /** Get a method with matching name and number of arguments. */
-  public Method getDeclaredMethod(String name, int argCount)
+  public synchronized Method getDeclaredMethod(String name, int argCount)
   {
     Method result = null;
     int needOuterLinkArg = "<init>".equals(name) && hasOuterLink() ? 1 : 0;
@@ -910,7 +928,7 @@ public class ClassType extends ObjectType
     return result;
   }
 
-  public Method getMethod(String name, Type[] arg_types)
+  public synchronized Method getMethod(String name, Type[] arg_types)
   {
     ClassType cl = this;
     for (;;)
@@ -947,7 +965,7 @@ public class ClassType extends ObjectType
    * Does not add constructors nor private or package-private methods.
    * Does not check for duplicate (already-known) methods.
    * @param clas should be the same as getReflectClass(). */
-  public void addMethods(Class clas)
+  public synchronized void addMethods(Class clas)
   {
     // Set this flag BEFORE the actual addition.
     // This prevents this method to be called indirectly for the same class
@@ -1274,11 +1292,13 @@ public class ClassType extends ObjectType
     if (cother.isSubclass(this))
       return 1;
     if (this == toStringType)
-      return cother == Type.pointer_type ? -1 : 1;
+      return cother == Type.javalangObjectType ? -1 : 1;
     if (cother == toStringType)
-      return this == Type.pointer_type ? 1 : -1;
-    if (this.isInterface() || cother.isInterface())
-      return -2;
+      return this == Type.javalangObjectType ? 1 : -1;
+    if (this.isInterface())
+      return cother == Type.javalangObjectType ? -1 : -2;
+    if (cother.isInterface())
+      return this == Type.javalangObjectType ? 1 : -2;
     return -3;
   }
 
@@ -1330,4 +1350,33 @@ public class ClassType extends ObjectType
     attributes = null;
     sourceDbgExt = null;
   }
+
+  /** Check to see if this is a Single Abstract Method (SAM) type.
+   * I.e. an interface or abstract class that has one and only one
+   * abstract method.  (One way that lambdas/closures are useful
+   * is that when given a lambda in a context that requires a SAM,
+   * create an implementing class using the lambda for the abstract method.)
+   * @return the single abstract Method, or null if this is not a SAM type.
+   */
+  public Method checkSingleAbstractMethod ()
+  {
+    Method[] methods = getAbstractMethods();
+    int nmethods = methods.length;
+    Method result = null;
+    for (int i = 0;  i < nmethods;  i++)
+      {
+        Method meth = methods[i];
+        String mname = meth.getName();
+        Type[] ptypes = meth.getParameterTypes();
+
+        Method mimpl = getMethod(mname, ptypes);
+        if (mimpl != null && ! mimpl.isAbstract())
+          continue;
+        if (result != null)
+          return null;
+        result = meth;
+      }
+    return result;
+  }
+
 }
