@@ -20,8 +20,9 @@ public class ClassTypeWriter extends PrintWriter
   public static final int PRINT_CONSTANT_POOL_INDEXES = 2;
   /** Flag to print classfile version  numbers. */
   public static final int PRINT_VERSION = 4;
+  public static final int PRINT_EXTRAS = 8;
   public static final int PRINT_VERBOSE =
-    PRINT_CONSTANT_POOL|PRINT_CONSTANT_POOL_INDEXES|PRINT_VERSION;
+    PRINT_CONSTANT_POOL|PRINT_CONSTANT_POOL_INDEXES|PRINT_EXTRAS|PRINT_VERSION;
 
   public ClassTypeWriter (ClassType ctype, Writer stream, int flags)
   {
@@ -74,6 +75,11 @@ public class ClassTypeWriter extends PrintWriter
     printAttributes();
   }
 
+  public void setClass (ClassType ctype)
+  {
+    this.ctype = ctype;
+  }
+
   public void print (ClassType ctype)
   {
     this.ctype = ctype;
@@ -170,7 +176,11 @@ public class ClassTypeWriter extends PrintWriter
     println();
     Method method = ctype.methods;
     for (; method != null; method = method.next)
-      {
+      printMethod(method);
+  }
+
+  public void printMethod (Method method)
+  {
 	println();
 	print("Method name:");
 	if (method.name_index != 0)
@@ -193,15 +203,20 @@ public class ClassTypeWriter extends PrintWriter
 	printSignature(method.return_type);
 	println();
 	printAttributes(method);
-      }
   }
 
-  final void printConstantTersely(int index, int expected_tag)
+  CpoolEntry getCpoolEntry (int index)
   {
     CpoolEntry[] pool = ctype.constants.pool;
-    CpoolEntry entry;
-    if (pool == null || index < 0 || index >= pool.length
-	|| (entry = pool[index]) == null)
+    if (pool == null || index < 0 || index >= pool.length)
+      return null;
+    else
+      return pool[index];
+  }
+
+  final void printConstantTersely(CpoolEntry entry, int expected_tag)
+  {
+    if (entry == null)
       print("<invalid constant index>");
     else if (entry.getTag() != expected_tag)
       {
@@ -213,6 +228,23 @@ public class ClassTypeWriter extends PrintWriter
       entry.print(this, 0);
   }
 
+  final void printConstantTersely(int index, int expected_tag)
+  {
+    printConstantTersely(getCpoolEntry(index), expected_tag);
+  }
+
+  final void printContantUtf8AsClass(int type_index)
+  {
+    CpoolEntry entry = getCpoolEntry(type_index);
+    if (entry != null && entry.getTag() == ConstantPool.UTF8)
+      {
+        String name = ((CpoolUtf8) entry).string;
+        Type.printSignature(name, 0, name.length(), this);
+      }
+    else
+      printConstantTersely(type_index, ConstantPool.UTF8);
+  }
+
   /** Print constant pool index for dis-assembler. */
   final void printConstantOperand(int index)
   {
@@ -423,4 +455,10 @@ public class ClassTypeWriter extends PrintWriter
     else
       printSignature(type.getSignature());
   }
+
+  public void printSpaces (int count)
+  {
+    while (--count >= 0)
+      print(' ');
+  }
 }
