@@ -64,23 +64,13 @@ public class PrimType extends Type {
     return ! (value instanceof Boolean) || ((Boolean) value).booleanValue();
   }
 
-  public void emitCoerceToObject (CodeAttr code)
+  public ClassType boxedType ()
   {
     char sig1 = getSignature().charAt(0);
-    ClassType clas;
-    Method method;
     String cname;
-    Type[] args;
     switch (sig1)
       {
-      case 'Z':
-	clas = ClassType.make("java.lang.Boolean");
-	code.emitIfIntNotZero();
-	code.emitGetStatic(clas.getDeclaredField("TRUE"));
-	code.emitElse();
-	code.emitGetStatic(clas.getDeclaredField("FALSE"));
-	code.emitFi();
-	return;
+      case 'Z':  cname = "java.lang.Boolean";   break;
       case 'C':  cname = "java.lang.Character"; break;
       case 'B':  cname = "java.lang.Byte";      break;
       case 'S':  cname = "java.lang.Short";     break;
@@ -90,8 +80,24 @@ public class PrimType extends Type {
       case 'D':  cname = "java.lang.Double";    break;
       default:   cname = null; // Should never happen.
       }
-    clas = ClassType.make(cname);
-    args = new Type[1];
+    return ClassType.make(cname);
+ }
+
+  public void emitCoerceToObject (CodeAttr code)
+  {
+    char sig1 = getSignature().charAt(0);
+    ClassType clas = boxedType();
+    if (sig1 == 'Z')
+      {
+	code.emitIfIntNotZero();
+	code.emitGetStatic(clas.getDeclaredField("TRUE"));
+	code.emitElse();
+	code.emitGetStatic(clas.getDeclaredField("FALSE"));
+	code.emitFi();
+	return;
+      }
+    Method method;
+    Type[] args = new Type[1];
     args[0] = this;
     if (code.getMethod().getDeclaringClass().classfileFormatVersion >= ClassType.JDK_1_5_VERSION)
         method = clas.getDeclaredMethod("valueOf", args);
@@ -209,10 +215,43 @@ public class PrimType extends Type {
       }
   }
 
+  /** An encoding of the subset/priority order of Number class.
+   * Each entry is a letter, then ':', then a class name, then ';'.
+   * Classes with letters later in alphabet are more general.
+   */
+  private static final String numberHierarchy =
+    "A:java.lang.Byte;" +
+    "B:java.lang.Short;" +
+    "C:java.lang.Integer;" +
+    "D:java.lang.Long;" +
+    "E:gnu.math.IntNum;E:java.gnu.math.BitInteger;" +
+    "G:gnu.math.RatNum;" +
+    "H:java.lang.Float;" +
+    "I:java.lang.Double;I:gnu.math.DFloNum;" +
+    "J:gnu.math.RealNum;" +
+    "K:gnu.math.Complex;" +
+    "L:gnu.math.Quantity;" +
+    "K:gnu.math.Numeric;" +
+    "N:java.lang.Number;";
+
+  /** Map a class name to a priority letter, using the numberHierarchy table. */
+  private static char findInHierarchy (String cname)
+  {
+    int pos = numberHierarchy.indexOf(cname) - 2;
+    return pos < 0 ? '\0' : numberHierarchy.charAt(pos);
+  }
+
   public int compare(Type other)
   {
     if (other instanceof PrimType)
-      return compare(this, (PrimType) other);
+      {
+        // Catches the case that other is a gnu.kawa.listpexpr.LangPrimType,
+        // in which case it's best that LangPrimType's compare handle it.
+        // For example intType.compare(Scheme.booleanType) should be -1.
+        if (other.getImplementationType() != other)
+          return swappedCompareResult(other.compare(this));
+        return compare(this, (PrimType) other);
+      }
     if (! (other instanceof ClassType))
       {
         if (other instanceof ArrayType)
@@ -224,42 +263,35 @@ public class PrimType extends Type {
     String otherName = other.getName();
     if (otherName == null)
        return -1;
+    char thisPriority = '\0';
     switch (sig1)
       {
       case 'V':
         return 1;
       case 'Z':
+        if (otherName.equals("java.lang.Boolean"))
+          return 0;
       case 'C':
+        if (otherName.equals("java.lang.Character"))
+          return 0;
         break;
-      case 'D':
-        if (otherName.equals("java.lang.Double")
-            || otherName.equals("gnu.math.DFloNum"))
-          return 0; // Or maybe 1?
-        if (otherName.equals("java.lang.Float"))
-          return 1;
-        break;
-      case 'F':
-        if (otherName.equals("java.lang.Double")
-            || otherName.equals("gnu.math.DFloNum"))
-          return -1;
-        if (otherName.equals("java.lang.Float"))
-          return 0; // Or maybe 1?
-        break;
-      case 'I':
-        if (otherName.equals("java.lang.Integer"))
-          return 0; // Or maybe 1?
-        if (otherName.equals("gnu.math.IntNum"))
-          return -1;
-        /* fall through */
-      default:
-        if (otherName.equals("java.lang.Double")
-            || otherName.equals("java.lang.Float")
-            || otherName.equals("gnu.math.DFloNum"))
-          return -1;
+      case 'B': thisPriority = 'A'; break;
+      case 'S': thisPriority = 'B'; break;
+      case 'I': thisPriority = 'C'; break;
+      case 'J': thisPriority = 'D'; break;
+      case 'F': thisPriority = 'H'; break;
+      case 'D': thisPriority = 'I'; break;
+      }
+    if (thisPriority != '\0')
+      {
+        char otherPriority = findInHierarchy(otherName);
+        if (otherPriority != '\0')
+          return otherPriority == thisPriority ? 0
+            : otherPriority < thisPriority ? 1 : -1;
       }
     if (otherName.equals("java.lang.Object")
 	|| other == toStringType)
       return -1;
-    return -2;
+    return -3;
   }
 }
