@@ -43,6 +43,12 @@ public abstract class Type
     return this;
   }
 
+  public boolean isExisting()
+  {
+    // Overridden in ObjectType.
+    return true;
+  }
+
   static ClassToTypeMap mapClassToType;
 
   /** Maps Java type name (e.g. "java.lang.String[]") to corresponding Type. */
@@ -98,7 +104,7 @@ public abstract class Type
   {
     ClassToTypeMap map = mapClassToType;
     if (map == null)
-      mapClassToType = map = new ClassToTypeMap(100);
+      mapClassToType = map = new ClassToTypeMap();
     type.reflectClass = clas;
     map.put(clas, type);
   }
@@ -143,7 +149,7 @@ public abstract class Type
     return type;
   }
 
-  public final String getSignature () { return signature; }
+  public String getSignature () { return signature; }
   protected void setSignature(String sig) { this.signature = sig; }
 
   Type (String nam, String sig) {
@@ -414,7 +420,7 @@ public abstract class Type
    *  0: all A values can be converted to B without a coercion failure
    *     and vice versa;
    * -1: all A values can be converted to B without a coercion failure
-   *     not not vice versa;
+   *     but not vice versa;
    * -2: there are (potentially) some A values that can be converted to B,
    *     and some B values can be converted to A;
    * -3: there are no A values that can be converted to B, and neither
@@ -608,27 +614,18 @@ public abstract class Type
     String name = toString();
     return name == null ? 0 : name.hashCode ();
   }
-}
 
-class ClassToTypeMap extends AbstractWeakHashTable<Class,Type>
-{
-  public ClassToTypeMap ()
+  static class ClassToTypeMap extends AbstractWeakHashTable<Class,Type>
   {
-    super(64);
-  }
+    protected Class getKeyFromValue (Type type)
+    {
+      return type.reflectClass;
+    }
 
-  public ClassToTypeMap (int capacity)
-  {
-    super(capacity);
-  }
-
-  protected Class getKeyFromValue (Type type)
-  {
-    return type.reflectClass;
-  }
-
-  protected boolean matches (Type oldValue, Type newValue)
-  {
-    return oldValue.reflectClass == newValue.reflectClass;
+    protected boolean matches (Class oldValue, Class newValue)
+    {
+      // Minor optimization.
+      return oldValue == newValue;
+    }
   }
 }
