@@ -14,14 +14,14 @@ public class setq extends Syntax
 {
   public Expression rewriteForm (Pair form, Translator tr)
   {
-    Object obj = form.cdr;
+    Object obj = form.getCdr();
     Vector results = null;
     while (obj != LList.Empty)
       {
 	if (! (obj instanceof Pair))
 	  return tr.syntaxError("invalid syntax for setq");
 	Pair pair = (Pair) obj;
-	Object sym = pair.car;
+	Object sym = pair.getCar();
 	Object name;
 	if (sym instanceof Symbol || sym instanceof String)
 	  name = sym;
@@ -29,12 +29,12 @@ public class setq extends Syntax
 	  name = "nil";
 	else
 	  return tr.syntaxError("invalid variable name in setq");
-	obj = pair.cdr;
+	obj = pair.getCdr();
 	if (! (obj instanceof Pair))
 	  return tr.syntaxError("wrong number of arguments for setq");
 	pair = (Pair) obj;
-	Expression value = tr.rewrite(pair.car);
-	obj = pair.cdr;
+	Expression value = tr.rewrite(pair.getCar());
+	obj = pair.getCdr();
 	SetExp sexp = new SetExp(name, value);
 	sexp.setFlag(SetExp.PREFER_BINDING2);
 	if (obj == LList.Empty)
