@@ -18,6 +18,7 @@ public class ApplyExp extends Expression
 
   public static final int TAILCALL = NEXT_AVAIL_FLAG;
   public static final int INLINE_IF_CONSTANT = NEXT_AVAIL_FLAG << 1;
+  public static final int MAY_CONTAIN_BACK_JUMP = NEXT_AVAIL_FLAG << 2;
 
   /** Containing LambdaExp. */
   LambdaExp context;
@@ -191,7 +192,7 @@ public class ApplyExp extends Expression
 	  throw new Error ("internal error - wrong number of parameters for "
 			   + func_lambda);
 	int conv = func_lambda.getCallConvention();
-        // Mostly duplicates logic with LambdaExp.inline.
+        // Mostly duplicates logic with LambdaExp.validateApply.
         // See comment there.
 	if (comp.inlineOk(func_lambda)
 	    && (conv <= Compilation.CALL_WITH_CONSUMER
@@ -405,27 +406,21 @@ public class ApplyExp extends Expression
     return copy;
   }
 
-  protected Expression walk (ExpWalker walker)
+  protected <R,D> R visit (ExpVisitor<R,D> visitor, D d)
   {
-    return walker.walkApplyExp(this);
+    return visitor.visitApplyExp(this, d);
   }
 
-  public void walkArgs (ExpWalker walker)
+  public void visitArgs (InlineCalls visitor)
   {
-    args = walker.walkExps(args, args.length);
+    args = visitor.visitExps(args, args.length, null);
   }
 
-  public void walkArgs (ExpWalker walker, boolean argsInlined)
+  protected <R,D> void visitChildren(ExpVisitor<R,D> visitor, D d)
   {
-    if (! argsInlined)
-      args = walker.walkExps(args, args.length);
-  }
-
-  protected void walkChildren(ExpWalker walker)
-  {
-    func = walker.walk(func);
-    if (walker.exitValue == null)
-      args = walker.walkExps(args, args.length);
+    func = visitor.visitAndUpdate(func, d);
+    if (visitor.exitValue == null)
+      args = visitor.visitExps(args, args.length, d);
   }
 
   public void print (OutPort out)
@@ -560,14 +555,14 @@ public class ApplyExp extends Expression
       return type;
     Expression afunc = derefFunc(func);
     // In case of cycles.
-    type = Type.pointer_type;
+    type = Type.objectType;
     if (afunc instanceof QuoteExp)
       {
         Object value = ((QuoteExp) afunc).getValue();
         Inlineable compiler;
-        if (value instanceof Procedure
-            && (compiler = asInlineable((Procedure) value)) != null)
-          type = compiler.getReturnType(args);
+        // This logic is deprecated - instead set type during InlineCalls.
+        if (value instanceof Procedure)
+          type = ((Procedure) value).getReturnType(args);
       }
     else if (afunc instanceof LambdaExp)
       {
@@ -593,9 +588,9 @@ public class ApplyExp extends Expression
     return true;
   }
 
-  public final Expression inlineIfConstant(Procedure proc, ExpWalker walker)
+  public final Expression inlineIfConstant(Procedure proc, InlineCalls visitor)
   {
-    return inlineIfConstant(proc, walker.getMessages());
+    return inlineIfConstant(proc, visitor.getMessages());
   }
 
   /** Inline this ApplyExp if parameters are constant.
