@@ -35,6 +35,18 @@ public class ClassExp extends LambdaExp
     return type != instanceType;
   }
 
+  /** The ClassType generated for this class.
+   * Note difference from {@code getClassType}: The value of a {@code ClassExp}
+   * (viewed as an expression) is a class/type object, so getType returns
+   * the type of a type.
+   */
+  public Type getType()
+  { return simple ? Compilation.typeClass : Compilation.typeClassType; }
+
+  /** The ClassType generated for this class.
+   */
+  public ClassType getClassType() { return type; }
+
   /** List of base classes and implemented interfaces. */
   public Expression[] supers;
   /** Index in supers array of class we extend, or -1. */
@@ -323,6 +335,7 @@ public class ClassExp extends LambdaExp
         // might be a TemplateScope.
         child.outer = this;
 	if ((child != initMethod && child != clinitMethod
+             && child.nameDecl != null // only if error
              && ! child.nameDecl.getFlag(Declaration.STATIC_SPECIFIED))
 	    || ! isMakingClassPair())
 	  child.addMethodFor(type, comp, null);
@@ -334,13 +347,9 @@ public class ClassExp extends LambdaExp
     if (isAbstract())
       instanceType.setModifiers(instanceType.getModifiers() | Access.ABSTRACT);
     if (nameDecl != null)
-      {
-        short access = nameDecl.getAccessFlags((short) 0);
-        if (access != 0)
-          instanceType.setModifiers((instanceType.getModifiers()
-                                     & ~Access.PUBLIC)
-                                    | access);
-      }
+      instanceType.setModifiers((instanceType.getModifiers()
+                                 & ~Access.PUBLIC)
+                                | nameDecl.getAccessFlags(Access.PUBLIC));
   }
 
   /** Return implementation method matching name and param types.
@@ -562,7 +571,7 @@ public class ClassExp extends LambdaExp
           }
         else
           {
-            methods = type.getMethods(AbstractMethodFilter.instance, 2);
+            methods = type.getAbstractMethods();
             nmethods = methods.length;
           }
 	for (int i = 0;  i < nmethods;  i++)
@@ -648,16 +657,17 @@ public class ClassExp extends LambdaExp
       }
   }
 
-  protected Expression walk (ExpWalker walker)
+  protected <R,D> R visit (ExpVisitor<R,D> visitor, D d)
   {
-    Compilation comp = walker.getCompilation();
+    Compilation comp = visitor.getCompilation();
     if (comp == null)
-      return walker.walkClassExp(this);
+      return visitor.visitClassExp(this, d);
     ClassType saveClass = comp.curClass;
     try
+
       {
 	comp.curClass = type;
-	return walker.walkClassExp(this);
+	return visitor.visitClassExp(this, d);
       }
     finally
       {
@@ -665,15 +675,15 @@ public class ClassExp extends LambdaExp
       }
   }
 
-  protected void walkChildren(ExpWalker walker)
+  protected <R,D> void visitChildren (ExpVisitor<R,D> visitor, D d)
   {
-    LambdaExp save = walker.currentLambda;
-    walker.currentLambda = this;
-    supers = walker.walkExps(supers, supers.length);
+    LambdaExp save = visitor.currentLambda;
+    visitor.currentLambda = this;
+    supers = visitor.visitExps(supers, supers.length, d);
     try
       {
 	for (LambdaExp child = firstChild;
-	     child != null && walker.exitValue == null;
+	     child != null && visitor.exitValue == null;
 	     child = child.nextSibling)
           {
             if (instanceType != null)
@@ -682,12 +692,12 @@ public class ClassExp extends LambdaExp
                 if (firstParam != null && firstParam.isThisParameter())
                   firstParam.setType(type);
               }
-            walker.walkLambdaExp(child);
+            visitor.visitLambdaExp(child, d);
           }
       }
     finally
       {
-	walker.currentLambda = save;
+	visitor.currentLambda = save;
       }
   }
 
@@ -737,12 +747,14 @@ public class ClassExp extends LambdaExp
     if (supers.length > 0)
       {
 
-        out.print(" supers:");
+        out.writeSpaceFill();
+        out.startLogicalBlock("supers:", "", 2);
         for (int i = 0;  i < supers.length;  i++)
           {
             supers[i].print(out);
-            out.print(' ');
+            out.writeSpaceFill();
           }
+        out.endLogicalBlock("");
       }
     out.print('(');
     Special prevMode = null;
@@ -777,31 +789,33 @@ public class ClassExp extends LambdaExp
   }
 
   /** Mangle a "slot" name to a get- or set- method name.
-   * @param prefix either "get" or "set"
+   * @param prefix either "get" or "set" or "add"
    * @param sname a "slot" (property) name.  This is mangled if needed.
    */
   public static String slotToMethodName(String prefix, String sname)
   {
     if (! Compilation.isValidJavaName(sname))
       sname = Compilation.mangleName(sname, false);
-    StringBuffer sbuf = new StringBuffer(sname.length()+3);
+    int slen = sname.length();
+    StringBuffer sbuf = new StringBuffer(slen+3);
     sbuf.append(prefix);
-    sbuf.append(Character.toTitleCase(sname.charAt(0)));
-    sbuf.append(sname.substring(1));
+    if (slen > 0)
+      {
+        sbuf.append(Character.toTitleCase(sname.charAt(0)));
+        sbuf.append(sname.substring(1));
+      }
     return sbuf.toString();
   }
 
-  /** Helper class uses by ClassExp.compile.
-   */
-  private static class AbstractMethodFilter implements gnu.bytecode.Filter
+  public Declaration addMethod (LambdaExp lexp, Object mname)
   {
-    public static final AbstractMethodFilter instance
-      = new AbstractMethodFilter();
-
-    public boolean select(Object value)
-    {
-      gnu.bytecode.Method method = (gnu.bytecode.Method) value;
-      return method.isAbstract();
-    }
+    Declaration mdecl = addDeclaration(mname, Compilation.typeProcedure);
+    lexp.outer = this;
+    lexp.setClassMethod(true);
+    mdecl.noteValue(lexp);
+    mdecl.setFlag(Declaration.FIELD_OR_METHOD);
+    mdecl.setProcedureDecl(true);
+    lexp.setSymbol(mname);
+    return mdecl;
   }
 }
