@@ -1,4 +1,4 @@
-// Copyright (c) 1999, 2000-2005, 2006 Per M.A. Bothner.
+// Copyright (c) 1999, 2000-2005, 2006, 2010 Per M.A. Bothner.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.expr;
@@ -23,7 +23,7 @@ public class Compilation implements SourceLocator
    * This is because it contains a construct we know how to compile, but not
    * evaluate, and it it outside a function (which we always compile).
    * This can be a let scope, or primitive procedure. */
-  public boolean mustCompile;
+  public boolean mustCompile = ModuleExp.alwaysCompile;
 
   /** Used by LambdaExp.getSelectorValue if need to allocate new selector. */
   int maxSelectorValue;
@@ -88,23 +88,21 @@ public class Compilation implements SourceLocator
    */
   Field moduleInstanceMainField;
 
-  public java.util.Stack pendingImports;
+  /** Stack of quads of (ModuleInfo, ScopeExp, position, formSize). */
+  public java.util.Stack<Object> pendingImports;
 
-  public void pushPendingImport (ModuleInfo info, ScopeExp defs)
+  public void pushPendingImport (ModuleInfo info, ScopeExp defs, int formSize)
   {
     if (pendingImports == null)
-      pendingImports = new java.util.Stack();
+      pendingImports = new java.util.Stack<Object>();
     pendingImports.push(info);
     pendingImports.push(defs);
     Expression posExp = new ReferenceExp((Object) null);
     posExp.setLine(this);
     pendingImports.push(posExp);
+    pendingImports.push(Integer.valueOf(formSize));
   }
 
-  /** If true, minimize the number of classes generated.
-   * Do this even if it makes things a little slower. */
-  public static boolean fewerClasses;
-
   /** If true, print out expressions after parsing and before optimizations. */
   public static boolean debugPrintExpr = false;
 
@@ -112,14 +110,39 @@ public class Compilation implements SourceLocator
   public static boolean debugPrintFinalExpr;
 
   public static Options options = new Options();
-  public Options currentOptions = new Options(options);
-  static {
-    options.add("warn-undefined-variable", Options.BOOLEAN_OPTION,
+  public static Options.OptionInfo warnUndefinedVariable =
+    options.add("warn-undefined-variable",
+                Options.BOOLEAN_OPTION, Boolean.TRUE,
 		"warn if no compiler-visible binding for a variable");
-    options.add("warn-invoke-unknown-method", Options.BOOLEAN_OPTION,
-		"warn if invoke calls an unknown method");
-    options.add("warn-as-error", Options.BOOLEAN_OPTION,
+  public static Options.OptionInfo warnUnknownMember =
+    options.add("warn-unknown-member",
+                Options.BOOLEAN_OPTION, Boolean.TRUE,
+		"warn if referencing an unknown method or field");
+  public static Options.OptionInfo warnInvokeUnknownMethod =
+    options.add("warn-invoke-unknown-method",
+                Options.BOOLEAN_OPTION, warnUnknownMember,
+		"warn if invoke calls an unknown method (subsumed by warn-unknown-member)");
+  public static Options.OptionInfo warnAsError =
+    options.add("warn-as-error", Options.BOOLEAN_OPTION, Boolean.FALSE,
 		"Make all warnings into errors");
+
+  public Options currentOptions = new Options(options);
+
+  public boolean warnUndefinedVariable ()
+  {
+    return currentOptions.getBoolean(warnUndefinedVariable);
+  }
+  public boolean warnUnknownMember ()
+  {
+    return currentOptions.getBoolean(warnUnknownMember);
+  }
+  public boolean warnInvokeUnknownMethod ()
+  {
+    return currentOptions.getBoolean(warnInvokeUnknownMethod);
+  }
+  public boolean warnAsError ()
+  {
+    return currentOptions.getBoolean(warnAsError);
   }
 
   /** Get a named boolean option. */
@@ -135,6 +158,9 @@ public class Compilation implements SourceLocator
   }
 
   public static int defaultClassFileVersion =
+    /* #ifdef JAVA7 */
+    // ClassType.JDK_1_7_VERSION
+    /* #else */
     /* #ifdef JAVA6 */
     ClassType.JDK_1_6_VERSION
     /* #else */
@@ -144,6 +170,7 @@ public class Compilation implements SourceLocator
     // ClassType.JDK_1_1_VERSION
     /* #endif */
     /* #endif */
+    /* #endif */
     ;
 
   /** The default calling convention.
@@ -214,9 +241,9 @@ public class Compilation implements SourceLocator
   static public ClassType typeRunnable = ClassType.make("java.lang.Runnable");
   public static ClassType typeType = ClassType.make("gnu.bytecode.Type");
   public static ClassType typeObjectType
-    = ClassType.make("gnu.bytecode.ObjectType", typeType);
+    = ClassType.make("gnu.bytecode.ObjectType");
   public static ClassType typeClass = Type.javalangClassType;
-  static public ClassType typeClassType = ClassType.make("gnu.bytecode.ClassType", typeObjectType);
+  static public ClassType typeClassType = ClassType.make("gnu.bytecode.ClassType");
   static public ClassType typeProcedure
     = ClassType.make("gnu.mapping.Procedure");
   static public ClassType typeLanguage
@@ -314,21 +341,21 @@ public class Compilation implements SourceLocator
     apply4method, applyNmethod };
 
   public static ClassType typeProcedure0
-    = ClassType.make("gnu.mapping.Procedure0", typeProcedure);
+    = ClassType.make("gnu.mapping.Procedure0");
   public static ClassType typeProcedure1
-    = ClassType.make("gnu.mapping.Procedure1", typeProcedure);
+    = ClassType.make("gnu.mapping.Procedure1");
   public static ClassType typeProcedure2
-    = ClassType.make("gnu.mapping.Procedure2", typeProcedure);
+    = ClassType.make("gnu.mapping.Procedure2");
   public static ClassType typeProcedure3
-    = ClassType.make("gnu.mapping.Procedure3", typeProcedure);
+    = ClassType.make("gnu.mapping.Procedure3");
   public static ClassType typeProcedure4
-    = ClassType.make("gnu.mapping.Procedure4", typeProcedure);
+    = ClassType.make("gnu.mapping.Procedure4");
   public static ClassType typeProcedureN
-    = ClassType.make("gnu.mapping.ProcedureN", typeProcedure);
+    = ClassType.make("gnu.mapping.ProcedureN");
   public static ClassType typeModuleBody
-    = ClassType.make("gnu.expr.ModuleBody", typeProcedure0);
+    = ClassType.make("gnu.expr.ModuleBody");
   public static ClassType typeModuleWithContext
-    = ClassType.make("gnu.expr.ModuleWithContext", typeModuleBody);
+    = ClassType.make("gnu.expr.ModuleWithContext");
   public static ClassType typeApplet = ClassType.make("java.applet.Applet");
   public static ClassType typeServlet = ClassType.make("gnu.kawa.servlet.KawaServlet");
 
@@ -346,9 +373,9 @@ public class Compilation implements SourceLocator
   public static Field pcCallContextField
     = typeCallContext.getDeclaredField("pc");
   public static ClassType typeMethodProc
-  = ClassType.make("gnu.mapping.MethodProc", typeProcedureN);
+  = ClassType.make("gnu.mapping.MethodProc");
   public static ClassType typeModuleMethod
-  = ClassType.make("gnu.expr.ModuleMethod", typeMethodProc);
+  = ClassType.make("gnu.expr.ModuleMethod");
   //  public static Field numArgsCallFrameField = typeCallFrame.getDeclaredField("numArgs");
   public static Field argsCallContextField
     = typeCallContext.getDeclaredField("values");
@@ -372,11 +399,32 @@ public class Compilation implements SourceLocator
 
   LitTable litTable;
 
+  int langOptions;
+  
   /** True if we should generate an Applet. */
-  public boolean generateApplet;
+  public boolean generatingApplet ()
+  {
+    return (langOptions & Language.PARSE_FOR_APPLET) != 0;
+  }
 
-  /** True if we should generate an Servlet. */
-  public boolean generateServlet;
+  /** True if we should generate a Servlet. */
+  public boolean generatingServlet ()
+  {
+    return (langOptions & Language.PARSE_FOR_SERVLET) != 0;
+  }
+
+  public boolean sharedModuleDefs ()
+  {
+    return (langOptions & Language.PARSE_CURRENT_NAMES) != 0;
+  }
+
+  public void setSharedModuleDefs (boolean shared)
+  {
+    if (shared)
+      langOptions |= Language.PARSE_CURRENT_NAMES;
+    else
+      langOptions &= ~Language.PARSE_CURRENT_NAMES;
+  }
 
   public final ClassType getModuleType()
   {
@@ -414,16 +462,27 @@ public class Compilation implements SourceLocator
   {
     if (proc instanceof LambdaExp)
       {
+        LambdaExp lproc = (LambdaExp) proc;
+        Declaration nameDecl = lproc.nameDecl;
 	// The compiler gets confused if we turn off inlining for nested
 	// procedures - and they can't be rebound anyway.
-	if (! (((LambdaExp) proc).currentLambda() instanceof ModuleExp))
-	  return true;
+        if (nameDecl == null || nameDecl.getSymbol() == null
+            || ! (nameDecl.context instanceof ModuleExp))
+          return true;
+        if (immediate
+            && nameDecl.isPublic()
+            && ! lproc.getFlag(LambdaExp.OVERLOADABLE_FIELD)
+            && (curLambda == null || lproc.topLevel() != curLambda.topLevel()))
+          return false;
       }
     return inlineOk;
   }
 
   public boolean inlineOk (Procedure proc)
   {
+    if (immediate && proc instanceof ModuleMethod
+        && ((ModuleMethod) proc).module.getClass().getClassLoader() instanceof ArrayClassLoader)
+      return false;
     return inlineOk;
   }
 
@@ -1055,7 +1114,7 @@ public class Compilation implements SourceLocator
 
   public boolean makeRunnable ()
   {
-    return ! generateServlet && ! generateApplet
+    return ! generatingServlet() && ! generatingApplet()
       && ! getModule().staticInitRun();
   }
 
@@ -1069,9 +1128,9 @@ public class Compilation implements SourceLocator
     ClassType sup = module.getSuperType();
     if (sup == null)
       {
-        if (generateApplet)
+        if (generatingApplet())
 	  sup = typeApplet;
-	else if (generateServlet)
+	else if (generatingServlet())
 	  sup = typeServlet;
 	else
 	  sup = getModuleType();
@@ -1162,7 +1221,7 @@ public class Compilation implements SourceLocator
     if (lexp instanceof ClassExp)
       {
 	ClassExp cexp = (ClassExp) lexp;
-	callInitMethods(cexp.getCompiledClassType(this), new Vector(10));
+	callInitMethods(cexp.getCompiledClassType(this), new Vector<ClassType>(10));
       }
 
     code.emitReturn();
@@ -1176,7 +1235,7 @@ public class Compilation implements SourceLocator
    * @param clas Class to search for $finit$, and to search supertypes.
    * @param seen array of seen classes, to avoid duplicate $finit$ calls.
    */
-  void callInitMethods (ClassType clas, Vector seen)
+  void callInitMethods (ClassType clas, Vector<ClassType> seen)
   {
     if (clas == null)
       return;
@@ -1456,7 +1515,7 @@ public class Compilation implements SourceLocator
 	      {
 		counter = code.addLocal(Type.intType);
 		code.emitLoad(ctxVar);
-		code.emitGetField(typeCallContext.getDeclaredField("count"));
+                code.emitInvoke(typeCallContext.getDeclaredMethod("getArgCount", 0));
 		if (source.min_args != 0)
 		  {
 		    code.emitPushInt(source.min_args);
@@ -1517,17 +1576,7 @@ public class Compilation implements SourceLocator
 	      {
 		Type lastArgType = primArgTypes[explicitFrameArg+singleArgs];
 		if (lastArgType instanceof ArrayType)
-		  {
-		    Type elType
-		      = ((ArrayType) lastArgType).getComponentType();
-		    boolean mustConvert
-		      = ! "java.lang.Object".equals(elType.getName());
-		    if (mustConvert)
-		      new Error("not implemented mustConvert restarg");
-		    code.emitLoad(ctxVar);
-		    code.emitPushInt(singleArgs);
-		    code.emitInvokeVirtual(typeCallContext.getDeclaredMethod("getRestArgsArray", 1));
-		  }
+                  varArgsToArray(source, singleArgs, counter, lastArgType, ctxVar);
 		else if ("gnu.lists.LList".equals
 			 (lastArgType.getName()))
 		  {	
@@ -1725,60 +1774,7 @@ public class Compilation implements SourceLocator
 	      {
 		Type lastArgType = primArgTypes[explicitFrameArg+singleArgs];
 		if (lastArgType instanceof ArrayType)
-		  {
-		    Type elType
-		      = ((ArrayType) lastArgType).getComponentType();
-		    boolean mustConvert
-		      = ! "java.lang.Object".equals(elType.getName());
-		    if (singleArgs == 0 && ! mustConvert)
-		      code.emitLoad(code.getArg(2)); // load args array.
-		    else
-		      {
-			code.pushScope();
-			if (counter == null)
-			  {
-			    counter = code.addLocal(Type.intType);
-			    code.emitLoad(code.getArg(2));
-			    code.emitArrayLength();
-			    if (singleArgs != 0)
-			      {
-				code.emitPushInt(singleArgs);
-				code.emitSub(Type.intType);
-			      }
-			    code.emitStore(counter);
-			  }
-			code.emitLoad(counter);
-			code.emitNewArray(elType);
-			Label testLabel = new Label(code);
-			Label loopTopLabel = new Label(code);
-                        loopTopLabel.setTypes(code);
-			code.emitGoto(testLabel);
-			loopTopLabel.define(code);
-
-			code.emitDup(1); // new array
-			code.emitLoad(counter);
-			code.emitLoad(code.getArg(2));
-			code.emitLoad(counter);
-			if (singleArgs != 0)
-			  {
-			    code.emitPushInt(singleArgs);
-			    code.emitAdd(Type.intType);
-			  }
-			code.emitArrayLoad(Type.objectType);
-			if (mustConvert)
-			  {
-			    CheckedTarget.emitCheckedCoerce
-			      (this, source, source.getName(),
-			       0, elType, null);
-			  }
-			code.emitArrayStore(elType);
-			testLabel.define(code);
-			code.emitInc(counter, (short) (-1));
-			code.emitLoad(counter);
-			code.emitGotoIfIntGeZero(loopTopLabel);
-			code.popScope();	
-		      }
-		  }
+                  varArgsToArray(source, singleArgs, counter, lastArgType, null);
 		else if ("gnu.lists.LList".equals
 			 (lastArgType.getName()))
 		  {	
@@ -1825,6 +1821,85 @@ public class Compilation implements SourceLocator
     curClass = save_class;
   }
 
+  /** Copy incoming arguments to varargs/#!rest array.
+   */
+  private void varArgsToArray (LambdaExp source, int singleArgs,
+                               Variable counter, Type lastArgType,
+                               Variable ctxVar)
+  {
+    CodeAttr code = getCode();
+    Type elType = ((ArrayType) lastArgType).getComponentType();
+    boolean mustConvert = ! "java.lang.Object".equals(elType.getName());
+    if (ctxVar != null && ! mustConvert)
+      {
+        code.emitLoad(ctxVar);
+        code.emitPushInt(singleArgs);
+        code.emitInvokeVirtual(typeCallContext.getDeclaredMethod("getRestArgsArray", 1));
+      }
+    else if (singleArgs == 0 && ! mustConvert)
+      code.emitLoad(code.getArg(2)); // load args array.
+    else
+      {
+        code.pushScope();
+        if (counter == null)
+          {
+            counter = code.addLocal(Type.intType);
+            if (ctxVar != null)
+              {
+                code.emitLoad(ctxVar);
+                code.emitInvoke(typeCallContext.getDeclaredMethod("getArgCount", 0));
+              }
+            else
+              {
+                code.emitLoad(code.getArg(2));
+                code.emitArrayLength();
+              }
+            if (singleArgs != 0)
+              {
+                code.emitPushInt(singleArgs);
+                code.emitSub(Type.intType);
+              }
+            code.emitStore(counter);
+          }
+        code.emitLoad(counter);
+        code.emitNewArray(elType.getImplementationType());
+        Label testLabel = new Label(code);
+        Label loopTopLabel = new Label(code);
+        loopTopLabel.setTypes(code);
+        code.emitGoto(testLabel);
+        loopTopLabel.define(code);
+
+        code.emitDup(1); // new array
+        code.emitLoad(counter);
+        if (ctxVar != null)
+          code.emitLoad(ctxVar);
+        else
+          code.emitLoad(code.getArg(2));
+        code.emitLoad(counter);
+        if (singleArgs != 0)
+          {
+            code.emitPushInt(singleArgs);
+            code.emitAdd(Type.intType);
+          }
+        if (ctxVar != null)
+          code.emitInvokeVirtual(typeCallContext.getDeclaredMethod("getArgAsObject", 1));
+        else
+          code.emitArrayLoad(Type.objectType);
+        if (mustConvert)
+          {
+            CheckedTarget.emitCheckedCoerce
+              (this, source, source.getName(),
+               0, elType, null);
+          }
+        code.emitArrayStore(elType);
+        testLabel.define(code);
+        code.emitInc(counter, (short) (-1));
+        code.emitLoad(counter);
+        code.emitGotoIfIntGeZero(loopTopLabel);
+        code.popScope();	
+      }
+  }
+
   private Method startClassInit ()
   {
     method = curClass.addMethod ("<clinit>", apply0args, Type.voidType,
@@ -1832,7 +1907,7 @@ public class Compilation implements SourceLocator
 
     CodeAttr code = method.startCode();
 
-    if (generateMain || generateApplet || generateServlet)
+    if (generateMain || generatingApplet() || generatingServlet())
       {
 	ClassType languageType
 	  = (ClassType) Type.make(getLanguage().getClass());
@@ -1844,17 +1919,16 @@ public class Compilation implements SourceLocator
     return method;
   }
 
-  /** Parse/walk/compile this module as needed and requested.
+  /** Parse/visit/compile this module as needed and requested.
    * This method does not process any dependent modules (expect indirectly,
    * such as may be done by a require form).
    * @param wantedState the desired value of getState().
    */
   public void process (int wantedState)
   {
-    Compilation saveCompilation = Compilation.getCurrent();
+    Compilation saveCompilation = Compilation.setSaveCurrent(this);
     try
       {
-        Compilation.setCurrent(this);
         ModuleExp mexp = getModule();
         if (wantedState >= BODY_PARSED && getState() < BODY_PARSED-1)
           {
@@ -1932,7 +2006,7 @@ public class Compilation implements SourceLocator
       }
     finally
       {
-        Compilation.setCurrent(saveCompilation);
+        Compilation.restoreCurrent(saveCompilation);
       }
   }
 
@@ -1974,8 +2048,7 @@ public class Compilation implements SourceLocator
 	arg_types = new Type[1];
 	arg_types[0] = typeCallContext;
       }
-    else if (module.min_args != module.max_args || module.min_args > 4
-	|| (fewerClasses && curClass == mainClass))
+    else if (module.min_args != module.max_args || module.min_args > 4)
       {
 	arg_count = 1;
 	arg_types = new Type[1];
@@ -2132,11 +2205,8 @@ public class Compilation implements SourceLocator
 
     curLambda = saveLambda;
 
-    if (Compilation.fewerClasses) // FIXME
-      method.popScope(); // Undoes pushScope in method.initCode.
-
     module.heapFrame = heapFrame;  // Restore heapFrame.
-    if (usingCPStyle() || (fewerClasses && curClass == mainClass))
+    if (usingCPStyle())
       {
 	code = getCode();
 	fswitch.finish(code);
@@ -2425,7 +2495,7 @@ public class Compilation implements SourceLocator
     ModuleExp module = new ModuleExp();
     if (filename != null)
       module.setFile(filename);
-    if (generateApplet || generateServlet)
+    if (generatingApplet() || generatingServlet())
       module.setFlag(ModuleExp.SUPERTYPE_SPECIFIED);
     if (immediate)
       {
@@ -2508,14 +2578,14 @@ public class Compilation implements SourceLocator
         column = getColumnNumber();
       }
 
-    if (severity == 'w' && getBooleanOption("warn-as-error", false))
+    if (severity == 'w' && warnAsError())
       severity = 'e';
     messages.error(severity, file, line, column, message);
   }
 
   public void error(char severity, String message)
   {
-    if (severity == 'w' && getBooleanOption("warn-as-error", false))
+    if (severity == 'w' && warnAsError())
       severity = 'e';
     
     messages.error(severity, this, message);
@@ -2529,7 +2599,7 @@ public class Compilation implements SourceLocator
   public void error(char severity, String message,
                     String code, Declaration decl)
   {
-    if (severity == 'w' && getBooleanOption("warn-as-error", false))
+    if (severity == 'w' && warnAsError())
       severity = 'e';
     
     String filename = getFileName();
@@ -2582,7 +2652,7 @@ public class Compilation implements SourceLocator
   }
 
   /** A help vector for building expressions. */
-  public Stack exprStack;
+  public Stack<Expression> exprStack;
 
   public void letStart ()
   {
@@ -2647,7 +2717,7 @@ public class Compilation implements SourceLocator
     LambdaExp loopLambda = (LambdaExp) current_scope;
     Declaration decl = loopLambda.addDeclaration(name, type);
     if (exprStack == null)
-      exprStack = new Stack();
+      exprStack = new Stack<Expression>();
     exprStack.push(init);
     loopLambda.min_args++;
     return decl;
@@ -2896,12 +2966,12 @@ public class Compilation implements SourceLocator
 
   protected SourceMessages messages;
 
-  private static final ThreadLocation current =
-    new ThreadLocation("current-compilation");
+  private static final ThreadLocal<Compilation> current =
+    new InheritableThreadLocal<Compilation>();
 
   public static Compilation getCurrent ()
   {
-    return (Compilation) current.get();
+    return current.get();
   }
 
   public static void setCurrent (Compilation comp)
@@ -2909,6 +2979,18 @@ public class Compilation implements SourceLocator
     current.set(comp);
   }
 
+  public static Compilation setSaveCurrent (Compilation comp)
+  {
+    Compilation save = current.get();
+    current.set(comp);
+    return save;
+  }
+
+  public static void restoreCurrent (Compilation saved)
+  {
+    current.set(saved);
+  }
+
   public String toString ()
   {
     return "<compilation "+mainLambda+">";
