@@ -1,4 +1,4 @@
-// Copyright (c) 2003, 2009  Per M.A. Bothner.
+// Copyright (c) 2003, 2009, 2010  Per M.A. Bothner.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.expr;
@@ -177,6 +177,14 @@ public class Declaration
             return;
           }
       }
+    if (isFluid())
+      {
+        if (context instanceof FluidLetExp)
+          {
+            base.load(access, flags, comp, target);
+            return;
+          }
+      }
     CodeAttr code = comp.getCode();
     Type rtype = getType();
     if (! isIndirectBinding()
@@ -242,7 +250,9 @@ public class Declaration
             comp.compileConstant(val, target);
             return;
           }
-        else if (value != QuoteExp.undefined_exp && ignorable())
+        else if (value != QuoteExp.undefined_exp && ignorable()
+                 && ! (value instanceof LambdaExp
+                       && ((LambdaExp) value).outer instanceof ModuleExp))
           {
             value.compile(comp, target);
             return;
@@ -390,7 +400,7 @@ public class Declaration
     return value;
   }
 
-  /** Set the value assoociated with this Declaration.
+  /** Set the value associated with this Declaration.
    * Most code should use noteValue instead. */
   public final void setValue(Expression value) { this.value = value; }
 
@@ -503,8 +513,17 @@ public class Declaration
   /** A reference to a module instance. */
   public static final int MODULE_REFERENCE = 0x40000000;
 
-  public static final long VOLATILE_ACCESS = 0x80000000;
+  public static final long VOLATILE_ACCESS = 0x80000000l;
   public static final long TRANSIENT_ACCESS = 0x100000000l;
+  public static final long ENUM_ACCESS = 0x200000000l;
+  public static final long FINAL_ACCESS = 0x400000000l;
+  public static final long CLASS_ACCESS_FLAGS =
+    PRIVATE_ACCESS|PROTECTED_ACCESS|ENUM_ACCESS|FINAL_ACCESS;
+  public static final long FIELD_ACCESS_FLAGS = PRIVATE_ACCESS|PROTECTED_ACCESS|
+    PUBLIC_ACCESS|PACKAGE_ACCESS|VOLATILE_ACCESS|TRANSIENT_ACCESS|
+    ENUM_ACCESS|FINAL_ACCESS;
+  public static final long METHOD_ACCESS_FLAGS = PRIVATE_ACCESS
+    |PROTECTED_ACCESS|PUBLIC_ACCESS|PACKAGE_ACCESS|FINAL_ACCESS;
 
   protected long flags = IS_SIMPLE;
 
@@ -553,13 +572,18 @@ public class Declaration
       flags |= Access.VOLATILE;
     if (getFlag(TRANSIENT_ACCESS))
       flags |= Access.TRANSIENT;
+    if (getFlag(ENUM_ACCESS))
+      flags |= Access.ENUM;
+    if (getFlag(FINAL_ACCESS))
+      flags |= Access.FINAL;
     return flags;
   }
 
   public final boolean isAlias() { return (flags & IS_ALIAS) != 0; }
   public final void setAlias(boolean flag) { setFlag(flag, IS_ALIAS); }
 
-  /** True if this is a fluid binding (in a FluidLetExp). */
+  /** True if this is a fluid binding (in a FluidLetExp).
+   * Also true if this binding is the one re-bound by a FluidLetExp. */
   public final boolean isFluid () { return (flags & IS_FLUID) != 0; }
 
   public final void setFluid (boolean fluid) { setFlag(fluid, IS_FLUID); }
@@ -893,6 +917,12 @@ public class Declaration
 	sbuf.append("::");
 	sbuf.append(t.getName());
       }
+    if (base != null)
+      {
+        sbuf.append("(base:#");
+        sbuf.append(base.id);
+        sbuf.append(')');
+      }
   }
 
 
