@@ -35,17 +35,14 @@ public abstract class Expression extends Procedure0
   public final Object eval (Environment env) throws Throwable
   {
     CallContext ctx = CallContext.getInstance();
-    Environment save = ctx.getEnvironmentRaw();
-    if (env != save)
-      ctx.setEnvironmentRaw(env);
+    Environment saveEnv = Environment.setSaveCurrent(env);
     try
       {
         return eval(ctx);
       }
     finally
       {
-        if (env != save)
-          ctx.setEnvironmentRaw(save);
+        Environment.restoreCurrent(saveEnv);
       }
   }
 
@@ -242,28 +239,26 @@ public abstract class Expression extends Procedure0
     return null;
   }
 
-  protected Expression walk (ExpWalker walker)
+  protected <R,D> R visit (ExpVisitor<R,D> visitor, D d)
   {
-    return walker.walkExpression(this);
+    return visitor.visitExpression(this, d);
   }
 
-  protected void walkChildren (ExpWalker walker) { }
+  protected <R,D> void visitChildren (ExpVisitor<R,D> visitor, D d) { }
 
   /** Apply inlining transformations on a given ApplyExp.
    * Assumes the ApplyExp's function is this expression,
    * or can be optimized to this expression.
    * @param exp an application whose function expression can be simplified
    *  to this expression.
-   * @param walker the context for the current inlining pass
+   * @param visitor the context for the current inlining pass
    * @param decl if non-null, a Declaration bound to this expression.
-   * @param argsInlined true iff {@code exp.getArgs()} have been walked.
    * @return an Expression equivalent to the passed-in exp.
    */
-  public Expression inline (ApplyExp exp, InlineCalls walker,
-                            Declaration decl, boolean argsInlined)
+  public Expression validateApply (ApplyExp exp, InlineCalls visitor,
+                                   Type required, Declaration decl)
   {
-    if (! argsInlined)
-      exp.args = walker.walkExps(exp.args, exp.args.length);
+    exp.args = visitor.visitExps(exp.args, null);
     return exp;
   }
 
@@ -385,7 +380,8 @@ public abstract class Expression extends Procedure0
   }
 
   protected int flags;
-  protected static final int NEXT_AVAIL_FLAG = 1;
+  public static final int VALIDATED = 1;
+  protected static final int NEXT_AVAIL_FLAG = 2;
 
   public void setFlag (boolean setting, int flag)
   {
