@@ -6,22 +6,27 @@ import java.util.Hashtable;
 import java.io.Externalizable;
 import gnu.bytecode.Type;
 import gnu.mapping.*;
+import gnu.text.SourceLocator;
 
-public class FindCapturedVars extends ExpWalker
+public class FindCapturedVars extends ExpExpVisitor<Void>
 {
   public static void findCapturedVars (Expression exp, Compilation comp)
   {
-    FindCapturedVars walker = new FindCapturedVars();
-    walker.setContext(comp);
-    exp.walk(walker);
+    FindCapturedVars visitor = new FindCapturedVars();
+    visitor.setContext(comp);
+    exp.visit(visitor, null);
   }
 
-  protected Expression walkApplyExp (ApplyExp exp)
+  int backJumpPossible = 0;
+
+  protected Expression visitApplyExp (ApplyExp exp, Void ignored)
   {
+    int oldBackJumpPossible = backJumpPossible;
     boolean skipFunc = false;
+    boolean skipArgs = false;
     // If the func is bound to a module-level known function, and it
     // doesn't need a closure yet (i.e. could be compiled to a static
-    // method), don't walk the function, since that might force it to
+    // method), don't visit the function, since that might force it to
     // unnecessarily get "captured" which might force the current
     // function to require a closure.  That would be wasteful if the
     // alternative is to just call func using invokestatic.  (It is
@@ -47,7 +52,7 @@ public class FindCapturedVars extends ExpWalker
 	  }
       }
     // Similar hack for constructor calls, but here we want to
-    // avoid walking the type argument.
+    // avoid visiting the type argument.
     else if (exp.func instanceof QuoteExp && exp.getArgCount() > 0)
       {
         Object val = ((QuoteExp) exp.func).getValue();
@@ -70,26 +75,28 @@ public class FindCapturedVars extends ExpWalker
                         exp.nextCall = decl.firstCall;
                         decl.firstCall = exp;
                         for (int i = 1;  i < args.length;  i++)
-                          args[i].walk(this);
-                        return exp;
+                          args[i].visit(this, ignored);
+                        skipFunc = skipArgs = true;
                       }
                   }
               }
           }
       }
     if (! skipFunc)
-      exp.func = (Expression) exp.func.walk(this);
-    if (exitValue == null)
-      exp.args = walkExps(exp.args);
+      exp.func = exp.func.visit(this, ignored);
+    if (exitValue == null && ! skipArgs)
+      exp.args = visitExps(exp.args, ignored);
+    if (backJumpPossible > oldBackJumpPossible)
+      exp.setFlag(ApplyExp.MAY_CONTAIN_BACK_JUMP);
     return exp;
   }
 
-  public void walkDefaultArgs (LambdaExp exp)
+  public void visitDefaultArgs (LambdaExp exp, Void ignored)
   {
     if (exp.defaultArgs == null)
       return;
 
-    super.walkDefaultArgs(exp);
+    super.visitDefaultArgs(exp, ignored);
 
     // Check if any default expression "captured" a parameters.
     // If so, evaluating a default expression cannot be done until the
@@ -108,9 +115,9 @@ public class FindCapturedVars extends ExpWalker
       }
   }
 
-  protected Expression walkClassExp (ClassExp exp)
+  protected Expression visitClassExp (ClassExp exp, Void ignored)
   {
-    Expression ret = super.walkClassExp(exp);
+    Expression ret = super.visitClassExp(exp, ignored);
     if (! exp.explicitInit && ! exp.instanceType.isInterface())
       // Make sure <init> has been declared, in case we need to invoke it.
       Compilation.getConstructor(exp.instanceType, exp);
@@ -129,7 +136,7 @@ public class FindCapturedVars extends ExpWalker
     return ret;
   }
 
-  protected Expression walkModuleExp (ModuleExp exp)
+  protected Expression visitModuleExp (ModuleExp exp, Void ignored)
   {
     ModuleExp saveModule = currentModule;
     Hashtable saveDecls = unknownDecls;
@@ -137,7 +144,7 @@ public class FindCapturedVars extends ExpWalker
     unknownDecls = null;
     try
       {
-	return walkLambdaExp(exp);
+	return visitLambdaExp(exp, ignored);
       }
     finally
       {
@@ -146,21 +153,29 @@ public class FindCapturedVars extends ExpWalker
       }
   }
 
-  protected Expression walkFluidLetExp (FluidLetExp exp)
+  void maybeWarnNoDeclarationSeen (Object name, Compilation comp, SourceLocator location)
+  {
+    if (comp.warnUndefinedVariable())
+      comp.error('w', "no declaration seen for "+name, location);
+  }
+
+  protected Expression visitFluidLetExp (FluidLetExp exp, Void ignored)
   {
     for (Declaration decl = exp.firstDecl(); decl != null; decl = decl.nextDecl())
       {
         if (decl.base == null)
           {
-            Declaration bind = allocUnboundDecl(decl.getSymbol(), false);
+            Object name = decl.getSymbol();
+            Declaration bind = allocUnboundDecl(name, false);
+            maybeWarnNoDeclarationSeen(name, comp, exp);
             capture(bind);
             decl.base = bind;
           }
       }
-    return super.walkLetExp(exp);
+    return super.visitLetExp(exp, ignored);
   }
 
-  protected Expression walkLetExp (LetExp exp)
+  protected Expression visitLetExp (LetExp exp, Void ignored)
   {
     if (exp.body instanceof BeginExp)
       {
@@ -203,7 +218,7 @@ public class FindCapturedVars extends ExpWalker
 	      }
 	  }
       }
-    return super.walkLetExp(exp);
+    return super.visitLetExp(exp, ignored);
   }
 
   static Expression checkInlineable (LambdaExp current,
@@ -224,7 +239,6 @@ public class FindCapturedVars extends ExpWalker
     Expression r = current.returnContinuation;
     if (current.tailCallers != null)
       {
-        LambdaExp outer = current.outerLambda();
         for (LambdaExp p : current.tailCallers)
           {
             Expression t = checkInlineable(p, seen);
@@ -258,7 +272,7 @@ public class FindCapturedVars extends ExpWalker
     return r;
   }
 
-  protected Expression walkLambdaExp (LambdaExp exp)
+  protected Expression visitLambdaExp (LambdaExp exp, Void ignored)
   {
     java.util.Set<LambdaExp> seen = new java.util.LinkedHashSet<LambdaExp>();
     // Finish the job that was started in FindTailCalls.
@@ -268,8 +282,11 @@ public class FindCapturedVars extends ExpWalker
         // makes stack traces less helpful, and increases the risk of
         // methods getting too big.
         && (! (exp.outer instanceof ModuleExp) || exp.nameDecl == null))
-      exp.setInlineOnly(true);
-    return super.walkLambdaExp(exp);
+      {
+        exp.setInlineOnly(true);
+        backJumpPossible++;
+      }
+    return super.visitLambdaExp(exp, ignored);
   }
 
   public void capture(Declaration decl)
@@ -284,7 +301,9 @@ public class FindCapturedVars extends ExpWalker
       return;
 
     LambdaExp curLambda = getCurrentLambda ();
-    LambdaExp declLambda = decl.getContext().currentLambda ();
+    ScopeExp sc = decl.getContext();
+    if (sc==null) throw new Error("null context for "+decl+" curL:"+curLambda);
+    LambdaExp declLambda = sc.currentLambda ();
 
     // If curLambda is inlined, the function that actually needs a closure
     // is its caller.  We get its caller using getCaller().
@@ -370,7 +389,8 @@ public class FindCapturedVars extends ExpWalker
 	if (! decl.isStatic())
 	  {
 	    LambdaExp heapLambda = curLambda;
-	    heapLambda.setImportsLexVars();
+            if (! decl.isFluid())
+              heapLambda.setImportsLexVars();
 	    LambdaExp parent = heapLambda.outerLambda();
 	    for (LambdaExp outer = parent;  outer != declLambda && outer != null; )
 	      {
@@ -393,7 +413,12 @@ public class FindCapturedVars extends ExpWalker
 		outer = heapLambda.outerLambda();
 	      }
 	  }
-
+        if (declLambda==null) {
+          System.err.println("null declLambda for "+decl+" curL:"+curLambda);
+          ScopeExp c = decl.context;
+          for (; c!=null; c = c.outer)
+            System.err.println("- context:"+c);
+        }
         declLambda.capture(decl);
       }
   }
@@ -430,14 +455,18 @@ public class FindCapturedVars extends ExpWalker
 	  decl.setFlag(Declaration.STATIC_SPECIFIED);
 	decl.setCanRead(true);
 	decl.setCanWrite(true);
-	decl.setFlag(Declaration.IS_UNKNOWN);
+        // Setting IS_SINGLE_VALUE unconditionally is a kludge.
+        // It is OK for Scheme/Lisp, since a variable can't be
+        // bound to multiple value.  It is OK for XQuery, since we
+        // XQuery doesn't allow unknown/dynamic variables.  FIXME.
+	decl.setFlag(Declaration.IS_UNKNOWN|Declaration.IS_SINGLE_VALUE);
 	decl.setIndirectBinding(true);
 	unknownDecls.put(key, decl);
       }
     return decl;
   }
 
-  protected Expression walkReferenceExp (ReferenceExp exp)
+  protected Expression visitReferenceExp (ReferenceExp exp, Void ignored)
   {
     Declaration decl = exp.getBinding();
     if (decl == null)
@@ -446,15 +475,10 @@ public class FindCapturedVars extends ExpWalker
 				exp.isProcedureName());
 	exp.setBinding(decl);
       }
-    if (decl.getFlag(Declaration.IS_UNKNOWN))
+    if (decl.getFlag(Declaration.IS_UNKNOWN)
+        && comp.resolve(exp.getSymbol(), exp.isProcedureName()) == null)
       {
-	if (comp.getBooleanOption("warn-undefined-variable", false))
-	  {
-	    Object resolved
-	      = comp.resolve(exp.getSymbol(), exp.isProcedureName());
-	    if (resolved == null)
-	      comp.error('w', "no declaration seen for "+exp.getName(), exp);
-	  }
+        maybeWarnNoDeclarationSeen(exp.getSymbol(), comp, exp);
       }
 
     capture(exp.contextDecl(), decl);
@@ -474,13 +498,17 @@ public class FindCapturedVars extends ExpWalker
 	    return;
 	  }
       }
+    while (decl.isFluid() && decl.context instanceof FluidLetExp)
+      {
+        decl = decl.base;
+      }
     if (containing != null && decl.needsContext())
       capture(containing);
     else
       capture(decl);
   }
 
-  protected Expression walkThisExp (ThisExp exp)
+  protected Expression visitThisExp (ThisExp exp, Void ignored)
   {
     if (exp.isForContext())
       {
@@ -490,10 +518,10 @@ public class FindCapturedVars extends ExpWalker
         return exp;
       }
     else
-      return walkReferenceExp (exp);
+      return visitReferenceExp(exp, ignored);
   }
 
-  protected Expression walkSetExp (SetExp exp)
+  protected Expression visitSetExp (SetExp exp, Void ignored)
   {
     Declaration decl = exp.binding;
     if (decl == null)
@@ -507,7 +535,7 @@ public class FindCapturedVars extends ExpWalker
 	  decl = Declaration.followAliases(decl);
 	capture(exp.contextDecl(), decl);
       }
-    return super.walkSetExp(exp);
+    return super.visitSetExp(exp, ignored);
   }
 
 }
