@@ -1,17 +1,36 @@
+// Copyright (c) 2010  Per M.A. Bothner.
+// This is free software; for terms and warranty disclaimer see ../../COPYING.
+
 package gnu.expr;
 import gnu.bytecode.*;
+import gnu.kawa.reflect.CompileReflect;
 import gnu.kawa.reflect.Invoke;
 import gnu.kawa.functions.Convert;
 import gnu.kawa.util.IdentityHashTable;
 import gnu.mapping.*;
+import gnu.math.IntNum;
 import java.lang.reflect.InvocationTargetException;
+/* #ifdef use:java.dyn */
+// import java.dyn.*;
+/* #endif */
 
-public class InlineCalls extends ExpWalker
+/**
+ * The main Expression re-writing pass.
+ * This pass handles type-checking (work in progress).
+ * Also checks for calls to known Procedures, and may call
+ * a procedure-specific handler, which may do inlining, constant-folding,
+ * error-checking, and general munging.
+ *
+ * Should perhaps rename to something like "Validate" since
+ * we do type-checking and other stuff beyond inlining.
+ */
+
+public class InlineCalls extends ExpExpVisitor<Type>
 {
-  public static void inlineCalls (Expression exp, Compilation comp)
+  public static Expression inlineCalls (Expression exp, Compilation comp)
   {
-    InlineCalls walker = new InlineCalls(comp);
-    walker.walk(exp);
+    InlineCalls visitor = new InlineCalls(comp);
+    return visitor.visit(exp, null);
   }
 
   public InlineCalls (Compilation comp)
@@ -19,39 +38,172 @@ public class InlineCalls extends ExpWalker
     setContext(comp);
   }
 
-  protected Expression walkApplyExp(ApplyExp exp)
+  public Expression visit (Expression exp, Type required)
+  {
+    if (! exp.getFlag(Expression.VALIDATED))
+      {
+        exp.setFlag(Expression.VALIDATED); // Protect againts cycles.
+        exp = super.visit(exp, required);
+        exp.setFlag(Expression.VALIDATED);
+      }
+    return checkType(exp, required);
+  }
+
+  public Expression checkType(Expression exp, Type required)
+  {
+    Type expType = exp.getType();
+    boolean incompatible;
+    if (required instanceof ClassType && ((ClassType) required).isInterface()
+        && expType.isSubtype(Compilation.typeProcedure)
+        && ! expType.isSubtype(required))
+      {
+        if (exp instanceof LambdaExp)
+          {
+            Method amethod = ((ClassType) required).checkSingleAbstractMethod();
+            if (amethod != null)
+              {
+                LambdaExp lexp = (LambdaExp) exp;
+                ObjectExp oexp = new ObjectExp();
+                oexp.setLocation(exp);
+                oexp.supers = new Expression[] { new QuoteExp(required) };
+                oexp.setTypes(getCompilation());
+                Object mname = amethod.getName();
+                oexp.addMethod(lexp, mname);
+                Declaration mdecl = oexp.addDeclaration(mname, Compilation.typeProcedure);
+                oexp.firstChild = lexp;
+                oexp.declareParts(comp);
+                return visit(oexp, required);
+             }
+          }
+        incompatible = true;
+      }
+    else
+      {
+        if (expType == Type.toStringType)
+          expType = Type.javalangStringType;
+        incompatible = required != null && required.compare(expType) == -3;
+        if (incompatible && required instanceof TypeValue)
+          {
+            Expression converted = ((TypeValue) required).convertValue(exp);
+            if (converted != null)
+              return converted;
+          }
+      }
+    if (incompatible)
+      {
+        Language language = comp.getLanguage();
+        comp.error('w', "type "+(language.formatType(expType)
+                                 +" is incompatible with required type "
+                                 +language.formatType(required)),
+                   exp);
+      }
+    return exp;
+  }
+
+  protected Expression visitApplyExp(ApplyExp exp, Type required)
   {
     Expression func = exp.func;
     // Replace (apply (lambda (param ...) body ...) arg ...)
     // by: (let ((param arg) ...) body ...).
-    // Note this should be done *before* we walk the lambda, so we can
-    // walk the body with params bound to the known args.
+    // Note this should be done *before* we visit the lambda, so we can
+    // visit the body with params bound to the known args.
     if (func instanceof LambdaExp)
       {
         LambdaExp lexp = (LambdaExp) func;
         Expression inlined = inlineCall((LambdaExp) func, exp.args, false);
         if (inlined != null)
-          return walk(inlined);
+          return visit(inlined, required);
       }
-    func = walk(func);
+    func = visit(func, null);
     exp.func = func;
-    return func.inline(exp, this, null, false);
+    return func.validateApply(exp, this, required, null);
   }
 
-  /** Walk an ApplyExp assuming function and arguments have been walked. */
-  public final Expression walkApplyOnly(ApplyExp exp)
+  /** Visit an ApplyExp assuming function and arguments have been visited. */
+  public final Expression visitApplyOnly(ApplyExp exp, Type required)
   {
-    return exp.func.inline(exp, this, null, true);
+    return exp.func.validateApply(exp, this, required, null);
   }
 
-  protected Expression walkReferenceExp (ReferenceExp exp)
+  public static Integer checkIntValue (Expression exp)
+  {
+    if (exp instanceof QuoteExp)
+      {
+        QuoteExp qarg = (QuoteExp) exp;
+        Object value = qarg.getValue();
+        if (! qarg.isExplicitlyTyped() && value instanceof IntNum)
+          {
+            IntNum ivalue = (IntNum) value;
+            if (ivalue.inIntRange())
+              return Integer.valueOf(ivalue.intValue());
+          }
+      }
+    return null;
+  }
+
+  public static Long checkLongValue (Expression exp)
+  {
+    if (exp instanceof QuoteExp)
+      {
+        QuoteExp qarg = (QuoteExp) exp;
+        Object value = qarg.getValue();
+        if (! qarg.isExplicitlyTyped() && value instanceof IntNum)
+          {
+            IntNum ivalue = (IntNum) value;
+            if (ivalue.inLongRange())
+              return Long.valueOf(ivalue.longValue());
+          }
+      }
+    return null;
+  }
+
+  public QuoteExp fixIntValue (Expression exp)
+  {
+    Integer ival = InlineCalls.checkIntValue(exp);
+    if (ival != null)
+      return new QuoteExp(ival, comp.getLanguage().getTypeFor(Integer.TYPE));
+    return null;
+  }
+
+  public QuoteExp fixLongValue (Expression exp)
+  {
+    Long ival = InlineCalls.checkLongValue(exp);
+    if (ival != null)
+      return new QuoteExp(ival, comp.getLanguage().getTypeFor(Long.TYPE));
+    return null;
+  }
+
+  protected Expression visitQuoteExp (QuoteExp exp, Type required)
+  {
+    Object value;
+    if (exp.getRawType() == null && ! exp.isSharedConstant()
+        && (value = exp.getValue()) != null)
+      {
+        Language language = comp.getLanguage();
+        Type vtype = language.getTypeFor(value.getClass());
+        if (vtype == Type.toStringType)
+          vtype = Type.javalangStringType;
+        exp.type = vtype;
+        if (required instanceof PrimType)
+          {
+            char sig1 = required.getSignature().charAt(0);
+            QuoteExp ret = sig1 == 'I' ? fixIntValue(exp)
+              : sig1 == 'J'? fixLongValue(exp) : null;
+            if (ret != null)
+              exp = ret;
+          }
+      }
+    return exp;
+  }
+
+  protected Expression visitReferenceExp (ReferenceExp exp, Type required)
   {
     Declaration decl = exp.getBinding();
     if (decl != null && decl.field == null && ! decl.getCanWrite())
       {
         Expression dval = decl.getValue();
         if (dval instanceof QuoteExp && dval != QuoteExp.undefined_exp)
-          return walkQuoteExp((QuoteExp) dval);
+          return visitQuoteExp((QuoteExp) dval, required);
         if (dval instanceof ReferenceExp && ! decl.isAlias())
           {
             ReferenceExp rval = (ReferenceExp) dval;
@@ -62,7 +214,7 @@ public class InlineCalls extends ExpWalker
                     // We could also allow (some) widening conversions.
                     || dtype == rdecl.getType())
                 && ! rval.getDontDereference())
-              return walkReferenceExp(rval);
+              return visitReferenceExp(rval, required);
           }
         if (! exp.isProcedureName()
             && ((decl.flags & Declaration.FIELD_OR_METHOD+Declaration.PROCEDURE)
@@ -77,12 +229,12 @@ public class InlineCalls extends ExpWalker
             comp.error('e', decl, "here is the definition of ", "");
           }
       }
-    return super.walkReferenceExp(exp);
+    return super.visitReferenceExp(exp, required);
   }
 
-  protected Expression walkIfExp (IfExp exp)
+  protected Expression visitIfExp (IfExp exp, Type required)
   {
-    Expression test = exp.test.walk(this);
+    Expression test = exp.test.visit(this, null);
     if (test instanceof ReferenceExp)
       {
         Declaration decl = ((ReferenceExp) test).getBinding();
@@ -95,9 +247,9 @@ public class InlineCalls extends ExpWalker
       }
     exp.test = test;
     if (exitValue == null)
-      exp.then_clause = walk(exp.then_clause);
+      exp.then_clause = visit(exp.then_clause, required);
     if (exitValue == null && exp.else_clause != null)
-      exp.else_clause = walk(exp.else_clause);
+      exp.else_clause = visit(exp.else_clause, required);
     if (test instanceof QuoteExp)
       {
         boolean truth = comp.getLanguage().isTrue(((QuoteExp) test).getValue());
@@ -112,10 +264,20 @@ public class InlineCalls extends ExpWalker
     return exp;
   }
 
-  protected Expression walkScopeExp (ScopeExp exp)
+  protected Expression visitBeginExp (BeginExp exp, Type required)
   {
-    exp.walkChildren(this);
-    walkDeclarationTypes(exp);
+    int last = exp.length - 1;
+    for (int i = 0;  i <= last;  i++)
+      {
+        exp.exps[i] = visit(exp.exps[i], i < last ? null : required);
+      }
+    return exp;
+  }
+
+  protected Expression visitScopeExp (ScopeExp exp, Type required)
+  {
+    exp.visitChildren(this, null);
+    visitDeclarationTypes(exp);
     for (Declaration decl = exp.firstDecl();  decl != null;
          decl = decl.nextDecl())
       {
@@ -131,26 +293,28 @@ public class InlineCalls extends ExpWalker
     return exp;
   }
 
-  protected Expression walkLetExp (LetExp exp)
+  protected Expression visitLetExp (LetExp exp, Type required)
   {
     Declaration decl = exp.firstDecl();
     int n = exp.inits.length;
     for (int i = 0; i < n; i++, decl = decl.nextDecl())
       {
 	Expression init0 = exp.inits[i];
-	Expression init = walk(init0);
+        boolean typeSpecified = decl.getFlag(Declaration.TYPE_SPECIFIED);
+        Type dtype = typeSpecified && init0 != QuoteExp.undefined_exp ? decl.getType() : null;
+	Expression init = visit(init0, dtype);
 	exp.inits[i] = init;
 	Expression dvalue = decl.value;
 	if (dvalue == init0)
 	  {
 	    decl.value = dvalue = init;
-	    if (! decl.getFlag(Declaration.TYPE_SPECIFIED))
+	    if (! typeSpecified)
 	      decl.setType(dvalue.getType());
 	  }
       }
 
     if (exitValue == null)
-      exp.body = (Expression) walk(exp.body);
+      exp.body = visit(exp.body, required);
     if (exp.body instanceof ReferenceExp)
       {
         ReferenceExp ref = (ReferenceExp) exp.body;
@@ -164,7 +328,7 @@ public class InlineCalls extends ExpWalker
                 // Note this optimization does yield worse error messages
                 // than using CheckedTarget.  FIXME.
                 if (texp != QuoteExp.classObjectExp)
-                  init = walkApplyOnly(Compilation.makeCoercion(init, texp));
+                  init = visitApplyOnly(Compilation.makeCoercion(init, texp), null);
                 return init;
               }
             // Can also optimize if n > 1, but have to check if any
@@ -174,7 +338,7 @@ public class InlineCalls extends ExpWalker
     return exp;
   }
 
-  protected Expression walkLambdaExp (LambdaExp exp)
+  protected Expression visitLambdaExp (LambdaExp exp, Type required)
   {
     Declaration firstDecl = exp.firstDecl();
     if (firstDecl != null && firstDecl.isThisParameter()
@@ -182,28 +346,34 @@ public class InlineCalls extends ExpWalker
       {
         firstDecl.setType(comp.mainClass);
       }
-    return walkScopeExp(exp);
+    return visitScopeExp(exp, required);
   }
 
-  protected Expression walkTryExp (TryExp exp)
+  protected Expression visitTryExp (TryExp exp, Type required)
   {
     if (exp.getCatchClauses() == null && exp.getFinallyClause() == null)
-      return walk(exp.try_clause);
+      return visit(exp.try_clause, required);
     else
-      return super.walkTryExp(exp);
+      return super.visitTryExp(exp, required);
   }
 
-  protected Expression walkSetExp (SetExp exp)
+  protected Expression visitSetExpValue (Expression new_value, Type required,
+                                         Declaration decl)
+  {
+    return visit(new_value, decl == null || decl.isAlias() ? null : decl.type);
+  }
+
+  protected Expression visitSetExp (SetExp exp, Type required)
   {
-    super.walkSetExp(exp);
     Declaration decl = exp.getBinding();
+    super.visitSetExp(exp, required);
     if (! exp.isDefining() && decl != null
         && ((decl.flags & Declaration.FIELD_OR_METHOD+Declaration.PROCEDURE)
             == (Declaration.FIELD_OR_METHOD+Declaration.PROCEDURE)))
       comp.error('e', "can't assign to method "+decl.getName(), exp);
     if (decl != null && decl.getFlag(Declaration.TYPE_SPECIFIED))
       {
-        if (Invoke.checkKnownClass(decl.getType(), comp) < 0)
+        if (CompileReflect.checkKnownClass(decl.getType(), comp) < 0)
           decl.setType(Type.errorType);
       }
     /*
@@ -222,6 +392,14 @@ public class InlineCalls extends ExpWalker
     return exp;
   }
 
+  /* #ifdef use:java.dyn */
+  // static final java.dyn.MethodType validateApplyMType =
+  //   java.dyn.MethodType.make(gnu.expr.Expression.class,
+  //                            gnu.expr.ApplyExp.class,
+  //                            gnu.expr.InlineCalls.class,
+  //                            gnu.bytecode.Type.class,
+  //                            gnu.mapping.Procedure.class);
+  /* #else */
   private static Class[] inlinerMethodArgTypes;
   private static synchronized Class[] getInlinerMethodArgTypes()
     throws Exception
@@ -231,44 +409,41 @@ public class InlineCalls extends ExpWalker
       {
         t = new Class[] { Class.forName("gnu.expr.ApplyExp"),
                          Class.forName("gnu.expr.InlineCalls"),
-                         Boolean.TYPE,
+                         Class.forName("gnu.bytecode.Type"),
                          Class.forName("gnu.mapping.Procedure") };
         inlinerMethodArgTypes = t;
       }
     return t;
   }
+  /* #endif */
 
-  public Expression maybeInline (ApplyExp exp, boolean argsInlined, Procedure proc)
+  public Expression maybeInline (ApplyExp exp, Type required, Procedure proc)
   {
     try
       {
-        Object inliner = Procedure.inlineCallsKey.get(proc);
-        if (inliner != null)
-          return ((CanInline) inliner).inline(exp, this, argsInlined);
-        inliner = proc.getProperty(Procedure.inlinerKey, null);
-
-        if (inliner != null)
+        Object inliner;
+        synchronized (proc)
           {
-            Boolean argsInlinedBoxed = Boolean.valueOf(argsInlined);
-            if (inliner instanceof Procedure)
-              return (Expression) ((Procedure) inliner).apply4(exp, this,
-                                                               argsInlinedBoxed,
-                                                               proc);
+            inliner = proc.getProperty(Procedure.validateApplyKey, null);
             if (inliner instanceof String)
               {
                 String inliners = (String) inliner;
                 int colon = inliners.indexOf(':');
+                /* #ifdef use:java.dyn */
+                // MethodHandle method = null;
+                /* #else */
                 java.lang.reflect.Method method = null;
+                /* #endif */
                 if (colon > 0)
                   {
                     String cname = inliners.substring(0, colon);
                     String mname = inliners.substring(colon+1);
-                    /* #ifdef JAVA2 */
                     Class clas = Class.forName(cname, true, proc.getClass().getClassLoader());
+                    /* #ifdef use:java.dyn */
+                    // method = MethodHandles.lookup().findStatic(clas, mname, validateApplyMType);
                     /* #else */
-                    // Class clas = Class.forName(cname);
-                    /* #endif */
                     method = clas.getDeclaredMethod(mname, getInlinerMethodArgTypes());
+                    /* #endif */
                   }
                 if (method == null)
                   {
@@ -276,15 +451,23 @@ public class InlineCalls extends ExpWalker
                     return null;
                   }
                 inliner = method;
-                proc.setProperty(Procedure.inlinerKey, method);
               }
-            return (Expression) ((java.lang.reflect.Method) inliner).invoke(null,
-                                             new Object[] { exp, this,
-                                                            argsInlinedBoxed,
-                                                            proc });
+          } /* end synchronized */
+        if (inliner != null)
+          {
+            /* #ifdef use:java.dyn */
+            // if (inliner instanceof MethodHandle)
+            //   return ((MethodHandle) inliner).<Expression>invokeExact(exp, this, required, proc);
+            /* #endif */
+            Object[] vargs = new Object[] { exp, this, required, proc };
+            if (inliner instanceof Procedure)
+              return (Expression) ((Procedure) inliner).applyN(vargs);
+            /* #ifndef use:java.dyn */
+            else if (inliner instanceof java.lang.reflect.Method)
+              return (Expression) ((java.lang.reflect.Method) inliner)
+                .invoke(null, vargs);
+            /* #endif */
           }
-        if (proc instanceof CanInline)
-          return ((CanInline) proc).inline(exp, this, argsInlined);
       }
     catch (Throwable ex)
       {
