@@ -91,12 +91,9 @@ public class KawaScriptEngine extends AbstractScriptEngine
   public KawaCompiledScript compile(InPort port, ScriptContext context, SourceMessages messages)
     throws SyntaxException, IOException
   {
-    CallContext ctx = CallContext.getInstance();
-    Language saveLang = Language.getDefaultLanguage();
-    Environment saveEnv = ctx.getEnvironmentRaw();
+    Language saveLang = Language.setSaveCurrent(factory.language);
     Environment env = factory.getEnvironment(context);
-    ctx.setEnvironmentRaw(env);
-    Language.setDefaultLanguage(factory.language);
+    Environment saveEnv = Environment.setSaveCurrent(env);
     try
       {
         Compilation comp = factory.language.parse(port, messages, Language.PARSE_IMMEDIATE);
@@ -116,8 +113,8 @@ public class KawaScriptEngine extends AbstractScriptEngine
       }
     finally
       {
-	Language.setDefaultLanguage(saveLang);
-        ctx.setEnvironmentRaw(saveEnv);
+	Language.restoreCurrent(saveLang);
+        Environment.restoreCurrent(saveEnv);
       }
   }
 }
