@@ -93,9 +93,9 @@ public class LambdaExp extends ScopeExp
   public LambdaExp inlineHome;
 
   /** Expressions that name classes that may be thrown. */
-  ReferenceExp[] throwsSpecification;
+  Expression[] throwsSpecification;
 
-  public void setExceptions(ReferenceExp[] exceptions)
+  public void setExceptions(Expression[] exceptions)
   {
     throwsSpecification = exceptions;
   }
@@ -124,7 +124,6 @@ public class LambdaExp extends ScopeExp
    * See declareClosureEnv and closureEnvField. */
   Variable closureEnv;
 
-  static final int INLINE_ONLY = 1;
   static final int CAN_READ = 2;
   static final int CAN_CALL = 4;
   static final int IMPORTS_LEX_VARS = 8;
@@ -139,7 +138,8 @@ public class LambdaExp extends ScopeExp
   public static final int SEQUENCE_RESULT = 1024;
   public static final int OVERLOADABLE_FIELD = 2048;
   public static final int ATTEMPT_INLINE = 4096;
-  protected static final int NEXT_AVAIL_FLAG = 8192;
+  static final int INLINE_ONLY = 8192;
+  protected static final int NEXT_AVAIL_FLAG = 16384;
 
   /** True iff this lambda is only "called" inline. */
   public final boolean getInlineOnly() { return (flags & INLINE_ONLY) != 0; }
@@ -282,6 +282,11 @@ public class LambdaExp extends ScopeExp
     return type;
   }
 
+  /** The ClassType generated for this class.
+   * Only used for ClassExp (which overrides this method) or ModuleExp.
+   */
+  public ClassType getClassType() { return type; }
+
   public void setType (ClassType type)
   {
     this.type = type;
@@ -538,8 +543,7 @@ public class LambdaExp extends ScopeExp
 		|| isModuleBody() || isClassMethod() || isHandlingTailCalls()))
 	  code.emitReturn();
 	popScope(code);        // Undoes enterScope in allocParameters
-	if (! Compilation.fewerClasses) // FIXME
-	  code.popScope(); // Undoes pushScope in method.initCode.
+        code.popScope(); // Undoes pushScope in method.initCode.
       }
 
     for (LambdaExp child = firstChild;  child != null; )
@@ -1001,18 +1005,18 @@ public class LambdaExp extends ScopeExp
 	    Type lastType = var.getType();
 	    String lastTypeName = lastType.getName();
             if (ctype.getClassfileVersion() >= ClassType.JDK_1_5_VERSION
-                && "java.lang.Object[]".equals(lastTypeName))
+                && lastType instanceof ArrayType)
               mflags |= Access.VARARGS;
             else 
               nameBuf.append("$V");
 	    if (key_args > 0 || numStubs < opt_args
                 // We'd like to support the the #!rest parameter an arbitrary
                 // array type or implementation of java.util.List.  However,
-                // we currently only support gnu.lists.LList and Object[].
-                // For Object[] we prefer to use the Java5 VARARGS feature;
+                // we currently only support gnu.lists.LList plus array types.
+                // For array types we prefer to use the Java5 VARARGS feature;
                 // for gnu.lists.LList we use the old $V mechanism.
 		|| ! ("gnu.lists.LList".equals(lastTypeName)
-		      || "java.lang.Object[]".equals(lastTypeName)))
+		      || lastType instanceof ArrayType))
 	      {
 		lastType = Compilation.objArrayType;
 		argsArray = new Variable("argsArray",
@@ -1069,27 +1073,43 @@ public class LambdaExp extends ScopeExp
 	    for (int j = 0;  j < n;  j++)
 	      {
 		ClassType exception = null;
-		Declaration decl = throwsSpecification[j].getBinding();
-		if (decl != null)
-		  {
-		    Expression declValue = decl.getValue();
-		    if (declValue instanceof ClassExp)
-		      exception
-			= ((ClassExp) declValue).getCompiledClassType(comp);
-		    else
-		      comp.error('e', "throws specification "+decl.getName()
-				 + " has non-class lexical binding");
-		  }
-		if (exception == null)
-		  {
-		    String exName = throwsSpecification[j].getName();
-		    int nlen = exName.length();
-		    if (nlen > 2
-			&& exName.charAt(0) == '<'
-			&& exName.charAt(nlen-1) == '>')
-		      exName = exName.substring(1, nlen-1);
-		    exception = ClassType.make(exName);
-		  }
+                Expression throwsExpr = throwsSpecification[j];
+                String msg = null;
+                if (throwsExpr instanceof ReferenceExp)
+                  {
+                    ReferenceExp throwsRef = (ReferenceExp) throwsExpr;
+                    Declaration decl = throwsRef.getBinding();
+                    if (decl != null)
+                      {
+                        Expression declValue = decl.getValue();
+                        if (declValue instanceof ClassExp)
+                          exception
+                            = ((ClassExp) declValue).getCompiledClassType(comp);
+                        else
+                          msg = "throws specification "+decl.getName()
+                            + " has non-class lexical binding";
+                      }
+                    else
+                      msg = "unknown class "+throwsRef.getName();
+                  }
+                else if (throwsExpr instanceof QuoteExp)
+                  {
+                    Object value = ((QuoteExp) throwsExpr).getValue();
+                    if (value instanceof Class)
+                      value = Type.make((Class) value);
+                    if (value instanceof ClassType)
+                      exception = (ClassType) value;
+                    if (exception != null
+                        && ! exception.isSubtype(Type.javalangThrowableType))
+                      msg = exception.getName() + " does not extend Throwable";
+                  }
+                if (exception == null && msg == null)
+                  msg = "invalid throws specification";
+                if (msg != null)
+                  {
+                    comp.error('e', msg, throwsExpr);
+                    exception = Type.javalangThrowableType;
+                  }
 		exceptions[j] = exception;
 	      }
 	    ExceptionsAttr attr = new ExceptionsAttr(method);
@@ -1242,27 +1262,6 @@ public class LambdaExp extends ScopeExp
     if (line > 0)
       code.putLineNumber(getFileName(), line);
 
-    for (Declaration decl = firstDecl();  decl != null;  )
-      {
-	// If the only reason we are using an argsArray is because there
-	// are more than 4 arguments, copy the arguments in local register.
-	// Then forget about the args array.  We do this first, before
-	// the label that tail-recursion branches back to.
-	// This way, a self-tail-call knows where to leave the argumnents.
-	if (argsArray != null && min_args == max_args
-	    && primMethods == null
-	    && getCallConvention() < Compilation.CALL_WITH_CONSUMER)
-	  {
-	    code.emitLoad(argsArray);
-	    code.emitPushInt(j);
-	    code.emitArrayLoad(Type.objectType);
-	    decl.getType().emitCoerceFromObject(code);
-	    code.emitStore(decl.getVariable());
-	  }
-	j++;
-	i++;
-	decl = decl.nextDecl();
-      }
     if (heapFrame != null)
       heapFrame.allocateLocal(code);
   }
@@ -1336,7 +1335,7 @@ public class LambdaExp extends ScopeExp
       }
 
     Variable argsArray = this.argsArray;
-    if (min_args == max_args && ! Compilation.fewerClasses
+    if (min_args == max_args
 	&& primMethods == null
 	&& getCallConvention () < Compilation.CALL_WITH_CONSUMER)
       argsArray = null;
@@ -1380,9 +1379,7 @@ public class LambdaExp extends ScopeExp
 	    || param.isIndirectBinding())
 	  {
 	    Type paramType = param.getType();
-	    Type stackType
-	      = (mainMethod == null || plainArgs >= 0 ? Type.objectType
-		 : paramType);
+	    Type stackType = plainArgs >= 0 ? Type.objectType : paramType;
 	    // If the parameter is captured by an inferior lambda,
 	    // then the incoming parameter needs to be copied into its
 	    // slot in the heapFrame.  Thus we emit an aaload instruction.
@@ -1479,7 +1476,12 @@ public class LambdaExp extends ScopeExp
 	    if (param.isIndirectBinding())
               param.pushIndirectBinding(comp);
 	    if (param.isSimple())
-	      code.emitStore(param.getVariable());
+              {
+                Variable var = param.getVariable();
+                if (param.isIndirectBinding())
+                  var.setType(Compilation.typeLocation);
+                code.emitStore(var);
+              }
             else
 	      code.emitPutField(param.field);
 	  }
@@ -1622,34 +1624,52 @@ public class LambdaExp extends ScopeExp
   /** A cache if this has already been evaluated. */
   Procedure thisValue;
 
-  protected Expression walk (ExpWalker walker)
+  protected <R,D> R visit (ExpVisitor<R,D> visitor, D d)
   {
-    return walker.walkLambdaExp(this);
-  }
-
-  protected void walkChildren(ExpWalker walker)
-  {
-    walkChildrenOnly(walker);
-    walkProperties(walker);
-  }
-
-  protected final void walkChildrenOnly(ExpWalker walker)
-  {
-    LambdaExp save = walker.currentLambda;
-    walker.currentLambda = this;
+    Compilation comp = visitor.getCompilation();
+    LambdaExp saveLambda;
+    if (comp == null)
+      saveLambda = null;
+    else
+      {
+        saveLambda = comp.curLambda;
+        comp.curLambda = this;
+      }
     try
       {
-	walker.walkDefaultArgs(this);
-	if (walker.exitValue == null && body != null)
-	  body = walker.walk(body);
+        return visitor.visitLambdaExp(this, d);
       }
     finally
       {
-	walker.currentLambda = save;
+        if (comp != null)
+          comp.curLambda = saveLambda;
       }
   }
 
-  protected final void walkProperties(ExpWalker walker)
+  protected <R,D> void visitChildren (ExpVisitor<R,D> visitor, D d)
+  {
+    visitChildrenOnly(visitor, d);
+    visitProperties(visitor, d);
+  }
+
+  protected final <R,D> void visitChildrenOnly (ExpVisitor<R,D> visitor, D d)
+  {
+    LambdaExp save = visitor.currentLambda;
+    visitor.currentLambda = this;
+    try
+      {
+        throwsSpecification = visitor.visitExps(throwsSpecification, d);
+	visitor.visitDefaultArgs(this, d);
+	if (visitor.exitValue == null && body != null)
+          body = visitor.update(body, visitor.visit(body, d));
+      }
+    finally
+      {
+	visitor.currentLambda = save;
+      }
+  }
+
+  protected final <R,D> void visitProperties (ExpVisitor<R,D> visitor, D d)
   {
     if (properties != null)
       {
@@ -1659,7 +1679,7 @@ public class LambdaExp extends ScopeExp
 	    Object val = properties[i];
 	    if (val instanceof Expression)
 	      {
-		properties[i] = walker.walk((Expression) properties[i]);
+		properties[i] = visitor.visitAndUpdate((Expression) val, d);
 	      }
 	  }
       }
@@ -1704,27 +1724,24 @@ public class LambdaExp extends ScopeExp
       }
   }
 
-  public Expression inline (ApplyExp exp, InlineCalls walker,
-                            Declaration decl, boolean argsInlined)
+  public Expression validateApply (ApplyExp exp, InlineCalls visitor,
+                                   Type required, Declaration decl)
   {
     Expression[] args = exp.getArgs();
-    if (! argsInlined)
+    if ((flags & ATTEMPT_INLINE) != 0)
       {
-        if ((flags & ATTEMPT_INLINE) != 0)
-          {
-            Expression inlined = InlineCalls.inlineCall(this, args, true);
-            if (inlined != null)
-              return walker.walk(inlined);
-          }
-        exp.args = walker.walkExps(exp.args, exp.args.length);
+        Expression inlined = InlineCalls.inlineCall(this, args, true);
+        if (inlined != null)
+          return visitor.visit(inlined, required);
       }
+    exp.visitArgs(visitor);
     int args_length = exp.args.length;
     String msg = WrongArguments.checkArgCount(getName(),
                                               min_args, max_args, args_length);
     if (msg != null)
-      return walker.noteError(msg);
+      return visitor.noteError(msg);
     int conv = getCallConvention();
-    Compilation comp = walker.getCompilation();
+    Compilation comp = visitor.getCompilation();
     Method method;
     // Mostly duplicates logic with ApplyExp.compile.
     if (comp.inlineOk(this) && isClassMethod()
@@ -1756,20 +1773,20 @@ public class LambdaExp extends ScopeExp
           margs = exp.args;
         else
           {
-            LambdaExp curLambda = walker.getCurrentLambda();
+            LambdaExp curLambda = visitor.getCurrentLambda();
             for (;;)
               {
                 if (curLambda == null)
-                  return walker.noteError("internal error: missing "+this);
+                  return visitor.noteError("internal error: missing "+this);
                 if (curLambda.outer == outer) // I.e. same class.
                   break;
                 curLambda = curLambda.outerLambda();
               }
             Declaration d = curLambda.firstDecl();
             if (d==null || ! d.isThisParameter())
-              return walker.noteError("calling non-static method "
-                                      +getName()+" from static method "
-                                      +curLambda.getName());
+              return visitor.noteError("calling non-static method "
+                                       +getName()+" from static method "
+                                       +curLambda.getName());
             int nargs = exp.getArgCount();
             margs = new Expression[1 + nargs];
             System.arraycopy(exp.getArgs(), 0, margs, 1, nargs);
