@@ -23,19 +23,35 @@ import gnu.kawa.lispexpr.ClassNamespace; // FIXME
 
 public abstract class Language
 {
-   protected static final ThreadLocation current
-     = new ThreadLocation("language");
+   protected static final InheritableThreadLocal<Language> current
+   = new InheritableThreadLocal<Language>();
+  protected static Language global;
 
   public static Language getDefaultLanguage()
-  { return (Language) current.get(null); }
+  {
+    Language lang = current.get();
+    return lang != null ? lang : global;
+  }
 
   static { Environment.setGlobal(BuiltinEnvironment.getInstance()); }
 
-  public static void setDefaultLanguage(Language language)
+  public static void setCurrentLanguage (Language language)
   {
     current.set(language);
   }
 
+  public static Language setSaveCurrent (Language language)
+  {
+    Language save = current.get();
+    current.set(language);
+    return save;
+  }
+
+  public static void restoreCurrent (Language saved)
+  {
+    current.set(saved);
+  }
+
   /**
    * List of known languages and their Language classes.
    * Each element is one or more language names, or filename extensions,
@@ -462,7 +478,7 @@ public abstract class Language
   /** Parse one or more expressions.
    * @param port the InPort to read the expressions from.
    * @param messages where to send error messages and warnings
-   * @param options various flags, includding PARSE_IMMEDIATE 
+   * @param options various flags, including PARSE_IMMEDIATE 
    *   and PARSE_ONE_LINE
    * @return a new Compilation.
    *   May return null if PARSE_ONE_LINE on end-of-file.
@@ -502,11 +518,10 @@ public abstract class Language
     Compilation tr = getCompilation(lexer, messages, lexical);
     if (requirePedantic)
       tr.pedantic = true;
+    if (! immediate)
+      tr.mustCompile = true;
     tr.immediate = immediate;
-    if ((options & PARSE_FOR_APPLET) != 0)
-      tr.generateApplet = true;
-    if ((options & PARSE_FOR_SERVLET) != 0)
-      tr.generateServlet = true;
+    tr.langOptions = options;
     if ((options & PARSE_EXPLICIT) != 0)
       tr.explicit = true;
     if ((options & PARSE_PROLOG) != 0)
@@ -538,7 +553,7 @@ public abstract class Language
 
   public final Type getLangTypeFor (Type type)
   {
-    if (! (type instanceof ObjectType) || ((ObjectType) type).isExisting())
+    if (type.isExisting())
       {
         Class clas = type.getReflectClass();
         if (clas != null)
@@ -549,7 +564,7 @@ public abstract class Language
 
   public String formatType (Type type)
   {
-    return type.toString();
+    return type.getName();
   }
 
   public static Type string2Type (String name)
@@ -633,6 +648,11 @@ public abstract class Language
 		Object val = ((QuoteExp) exp).getValue();
                 return getTypeFor(val, lenient);
               }
+            else if (exp instanceof ClassExp || exp instanceof ModuleExp)
+              {
+                decl.setCanRead(true);
+                return ((LambdaExp) exp).getClassType();
+              }
 	    else if (decl.isAlias()
 		&& exp instanceof QuoteExp)
 	      {
@@ -662,7 +682,7 @@ public abstract class Language
       }
     else if (exp instanceof ClassExp || exp instanceof ModuleExp)
       {
-	return ((LambdaExp) exp).getType();
+	return ((LambdaExp) exp).getClassType();
       }
     return null;
   }
@@ -736,7 +756,12 @@ public abstract class Language
     Declaration fdecl = mod.addDeclaration(fdname, dtype);
     boolean isStatic = (fld.getModifiers() & Access.STATIC) != 0;
     if (isAlias)
-      fdecl.setIndirectBinding(true);
+      {
+        fdecl.setIndirectBinding(true);
+        if (ftype instanceof ClassType
+            && ((ClassType) ftype).isSubclass("gnu.mapping.ThreadLocation"))
+          fdecl.setFlag(Declaration.IS_DYNAMIC);
+      }
     else if (isFinal && ftype instanceof ClassType)
       {
         if (ftype.isSubtype(Compilation.typeProcedure))
@@ -812,8 +837,8 @@ public abstract class Language
 
   public static synchronized void setDefaults (Language lang)
   {
-    Language.setDefaultLanguage(lang);
-    current.setGlobal(lang);
+    Language.setCurrentLanguage(lang);
+    global = lang;
     // Assuming this is the initial (main) thread, make its Environment
     // the default (global) one, so child threads can inherit from it.
     // Thus command-line definitions etc get inherited.
@@ -911,8 +936,7 @@ public abstract class Language
   public void eval (InPort port, CallContext ctx) throws Throwable
   {
     SourceMessages messages = new SourceMessages();
-    Language saveLang = getDefaultLanguage();
-    setDefaultLanguage(this);
+    Language saveLang = Language.setSaveCurrent(this);
     try
       {
 	Compilation comp = parse(port, messages, PARSE_FOR_EVAL);
@@ -920,7 +944,7 @@ public abstract class Language
       }
     finally
       {
-	setDefaultLanguage(saveLang);
+	Language.restoreCurrent(saveLang);
       }
     if (messages.seenErrors())
       throw new RuntimeException("invalid syntax in eval form:\n"
