@@ -197,21 +197,13 @@ public class ModuleExp extends LambdaExp
   {
     ModuleExp mexp = comp.getModule();
     mexp.info = comp.minfo;
-    Environment orig_env = Environment.getCurrent();
-    Compilation orig_comp = Compilation.getCurrent();
+    Environment orig_env = Environment.setSaveCurrent(env);
+    Compilation orig_comp = Compilation.setSaveCurrent(comp);
     SourceMessages messages = comp.getMessages();
     ClassLoader savedLoader = null;
     Thread thread = null; // Non-null if we need to restore context ClassLoader.
     try
       {
-	if (env != orig_env)
-	  Environment.setCurrent(env);
-        if (comp != orig_comp)
-          Compilation.setCurrent(comp);
-
-        if (alwaysCompile)
-          comp.mustCompile = true;
-
         comp.process(Compilation.RESOLVED);
         comp.minfo.loadByStages(Compilation.WALKED);
 
@@ -255,10 +247,8 @@ public class ModuleExp extends LambdaExp
       }
     finally
       {
-	if (env != orig_env)
-	  Environment.setCurrent(orig_env);
-	if (comp != orig_comp)
-	  Compilation.setCurrent(orig_comp);
+        Environment.restoreCurrent(orig_env);
+        Compilation.restoreCurrent(orig_comp);
         if (thread != null)
           thread.setContextClassLoader(savedLoader);
       }
@@ -269,14 +259,11 @@ public class ModuleExp extends LambdaExp
                                         Object inst)
     throws Throwable
   {
-    Environment orig_env = Environment.getCurrent();
+    Environment orig_env = Environment.setSaveCurrent(env);
     ClassLoader savedLoader = null;
     Thread thread = null; // Non-null if we need to restore context ClassLoader.
     try
       {
-	if (env != orig_env)
-	  Environment.setCurrent(env);
-
 	if (inst == Boolean.TRUE)
 	  { // optimization - don't generate unneeded Class.
 	    mexp.body.apply(ctx);
@@ -362,8 +349,7 @@ public class ModuleExp extends LambdaExp
       }
     finally
       {
-	if (env != orig_env)
-	  Environment.setCurrent(orig_env);
+        Environment.restoreCurrent(orig_env);
         if (thread != null)
           thread.setContextClassLoader(savedLoader);
       }
@@ -456,9 +442,9 @@ public class ModuleExp extends LambdaExp
       }
   }
 
-  protected Expression walk (ExpWalker walker)
+  protected <R,D> R visit (ExpVisitor<R,D> visitor, D d)
   {
-    return walker.walkModuleExp(this);
+    return visitor.visitModuleExp(this, d);
   }
 
   public void print (OutPort out)
