@@ -8,9 +8,11 @@ public class ObjectExp extends ClassExp
     super(true);
   }
 
-  protected Expression walk (ExpWalker walker)
+  public Type getType() { return type; }
+
+  protected <R,D> R visit (ExpVisitor<R,D> visitor, D d)
   {
-    return walker.walkObjectExp(this);
+    return visitor.visitObjectExp(this, d);
   }
 
   public void compile (Compilation comp, Target target)
