@@ -7,16 +7,25 @@ package gnu.expr;
     by making it more likely the application will be to a known procedure.
     This optimization has to be done after Declarations are bound. */
 
-public class PushApply extends ExpFullWalker
+public class PushApply extends ExpVisitor<Expression,Void>
 {
   public static void pushApply (Expression exp)
   {
-    PushApply walker = new PushApply();
-    exp.walk(walker);
-    //or:  walter.walkExpression(exp);
+    PushApply visitor = new PushApply();
+    visitor.visit(exp, null);
   }
 
-  public Object walkApplyExp(ApplyExp exp)
+  protected Expression update (Expression exp, Expression r)
+  {
+    return r;
+  }
+
+  protected Expression defaultValue(Expression r, Void ignored)
+  {
+    return r;
+  }
+
+  protected Expression visitApplyExp(ApplyExp exp, Void ignored)
   {
     Expression func = exp.func;
     if (func instanceof LetExp
@@ -27,7 +36,7 @@ public class PushApply extends ExpFullWalker
 	Expression body = let.body;
 	let.body = exp;
 	exp.func = body;
-	return let.walk(this);
+	return visit(let, ignored);
       }
     if (func instanceof BeginExp)  // [APPLY-BEGIN]
       {
@@ -37,8 +46,9 @@ public class PushApply extends ExpFullWalker
 	int last_index = begin.exps.length - 1;
 	exp.func = stmts[last_index];
 	stmts[last_index] = exp;
-	return begin.walk(this);
+	return visit(begin, ignored);
       }
+    exp.visitChildren(this, ignored);
     return exp;
   }
 }
