@@ -1,6 +1,7 @@
 package gnu.expr;
 import gnu.mapping.*;
 import gnu.bytecode.Type;
+import gnu.text.SourceLocator;
 
 /**
  * An Expression that evaluates to a constant value.
@@ -15,6 +16,9 @@ public class QuoteExp extends Expression
 
   public final Object valueIfConstant() { return value; }
 
+  public static final int EXPLICITLY_TYPED = Expression.NEXT_AVAIL_FLAG;
+  public static final int SHARED_CONSTANT = EXPLICITLY_TYPED << 1;
+
   protected Type type;
   public final gnu.bytecode.Type getRawType() { return type; }
   public final gnu.bytecode.Type getType()
@@ -31,20 +35,36 @@ public class QuoteExp extends Expression
       return Type.make(value.getClass());
   }
 
-  static public QuoteExp undefined_exp =
-    new QuoteExp (Special.undefined);
-  static public QuoteExp abstractExp =
-    new QuoteExp(Special.abstractSpecial);
-  static public QuoteExp voidExp =
-    new QuoteExp (Values.empty, Type.voidType);
-  static public QuoteExp trueExp = new QuoteExp(Boolean.TRUE);
-  static public QuoteExp falseExp = new QuoteExp(Boolean.FALSE);
-  static public QuoteExp nullExp =
-    new QuoteExp(null, Type.nullType);
-  public static final QuoteExp classObjectExp =
-    new QuoteExp(Type.pointer_type);
+  public void setType (Type type)
+  {
+    this.type = type;
+    setFlag(EXPLICITLY_TYPED);
+  }
+
+  public boolean isExplicitlyTyped ()
+  {
+    return getFlag(EXPLICITLY_TYPED);
+  }
+
+  public boolean isSharedConstant ()
+  {
+    return getFlag(SHARED_CONSTANT);
+  }
+
+  static public QuoteExp undefined_exp =  makeShared(Special.undefined);
+  static public QuoteExp abstractExp = makeShared(Special.abstractSpecial);
+  static public QuoteExp voidExp = makeShared(Values.empty, Type.voidType);
+  static public QuoteExp trueExp = makeShared(Boolean.TRUE);
+  static public QuoteExp falseExp = makeShared(Boolean.FALSE);
+  static public QuoteExp nullExp = makeShared(null, Type.nullType);
+  public static final QuoteExp classObjectExp = makeShared(Type.objectType);
 
   public static QuoteExp getInstance (Object value)
+  {
+    return getInstance(value, null);
+  }
+
+  public static QuoteExp getInstance (Object value, SourceLocator position)
   {
     if (value == null)
       return nullExp;
@@ -57,12 +77,29 @@ public class QuoteExp extends Expression
     // Note we deliberately don't map abstractSpecial to abstractExp.
     if (value instanceof Boolean)
       return ((Boolean) value).booleanValue() ? trueExp : falseExp;
-    return new QuoteExp(value);
+    QuoteExp q = new QuoteExp(value);
+    if (position != null)
+      q.setLocation(position);
+    return q;
+  }
+
+  static QuoteExp makeShared (Object value)
+  {
+    QuoteExp exp = new QuoteExp(value);
+    exp.setFlag(SHARED_CONSTANT);
+    return exp;
+  }
+
+  static QuoteExp makeShared (Object value, Type type)
+  {
+    QuoteExp exp = new QuoteExp(value, type);
+    exp.setFlag(SHARED_CONSTANT);
+    return exp;
   }
 
   public QuoteExp (Object val) { value = val; }
 
-  public QuoteExp (Object val, Type type) { value = val; this.type = type; }
+  public QuoteExp (Object val, Type type) { value = val; setType(type); }
   
   protected boolean mustCompile () { return false; }
 
@@ -90,38 +127,47 @@ public class QuoteExp extends Expression
     return this;
   }
 
-  protected Expression walk (ExpWalker walker)
+  protected <R,D> R visit (ExpVisitor<R,D> visitor, D d)
   {
-    return walker.walkQuoteExp(this);
+    return visitor.visitQuoteExp(this, d);
   }
 
-  public Expression inline (ApplyExp exp, InlineCalls walker,
-                            Declaration decl, boolean argsInlined)
+  public Expression validateApply (ApplyExp exp, InlineCalls visitor,
+                                   Type required, Declaration decl)
   {
     if (this == QuoteExp.undefined_exp)
       return exp;
     Object fval = getValue();
     if (! (fval instanceof Procedure))
-      return walker.noteError(decl == null || fval == null ? "called value is not a procedure"
+      return visitor.noteError(decl == null || fval == null ? "called value is not a procedure"
 			      : ("calling " + decl.getName()
 				 + " which is a "+fval.getClass().getName()));
     Procedure proc = (Procedure) fval;
     int nargs = exp.getArgCount();
     String msg = WrongArguments.checkArgCount(proc, nargs);
     if (msg != null)
-      return walker.noteError(msg);
-    Expression inlined = walker.maybeInline(exp, argsInlined, proc);
+      return visitor.noteError(msg);
+    Expression inlined = visitor.maybeInline(exp, required, proc);
     if (inlined != null)
       return inlined;
-    if (! argsInlined)
-      exp.args = walker.walkExps(exp.args, exp.args.length);
+    Expression[] args = exp.args;
+    MethodProc asMProc = proc instanceof MethodProc ? (MethodProc) proc : null;
+    for (int i = 0;  i < nargs;  i++)
+      {
+        Type ptype = asMProc != null ? asMProc.getParameterType(i) : null;
+        // The final varargs parameter T[] can match T or T[].
+        if (i == nargs - 1 && ptype != null
+            && asMProc.maxArgs() < 0 && i == asMProc.minArgs())
+          ptype = null;
+        args[i] = visitor.visit(args[i], ptype);
+      }
     if (exp.getFlag(ApplyExp.INLINE_IF_CONSTANT))
       {
-	Expression e = exp.inlineIfConstant(proc, walker);
+	Expression e = exp.inlineIfConstant(proc, visitor);
 	if (e != exp)
-	  return walker.walk(e);
+	  return visitor.visit(e, required);
       }
-    Compilation comp = walker.getCompilation();
+    Compilation comp = visitor.getCompilation();
     if (comp.inlineOk(proc))
       {
 	if (ApplyExp.asInlineable(proc) != null)
@@ -172,6 +218,11 @@ public class QuoteExp extends Expression
       {
 	out.objectFormat = Language.getDefaultLanguage().getFormat(true);
 	out.print(val);
+        if (type != null)
+          {
+            out.print(" ::");
+            out.print(type.getName());
+          }
         /*
         if (value != null)
           {
