@@ -131,7 +131,7 @@ public class ReferenceExp extends AccessExp
              || (binding.context instanceof ModuleExp
                 && ! binding.isPrivate()))
       {
-        Environment env = ctx.getEnvironment();
+        Environment env = Environment.getCurrent();
         Symbol sym = symbol instanceof Symbol ? (Symbol) symbol
           : env.getSymbol(symbol.toString());
         Object property = getFlag(PREFER_BINDING2) && isProcedureName()
@@ -172,12 +172,13 @@ public class ReferenceExp extends AccessExp
     return copy;
   }
 
-  protected Expression walk (ExpWalker walker)
+  protected <R,D> R visit (ExpVisitor<R,D> visitor, D d)
   {
-    return walker.walkReferenceExp(this);
+    return visitor.visitReferenceExp(this, d);
   }
-  public Expression inline (ApplyExp exp, InlineCalls walker,
-                            Declaration decl, boolean argsInlined)
+
+  public Expression validateApply (ApplyExp exp, InlineCalls visitor,
+                                   Type required, Declaration decl)
   {
     decl = this.binding; // We don't use the passed-in Declaration.
     if (decl != null && ! decl.getFlag(Declaration.IS_UNKNOWN))
@@ -187,7 +188,7 @@ public class ReferenceExp extends AccessExp
           {
             Expression dval = decl.getValue();
             if (dval != null)
-              return dval.inline(exp, walker, decl, argsInlined);
+              return dval.validateApply(exp, visitor, required, decl);
           }
       }
     else if (getSymbol() instanceof Symbol)
@@ -195,10 +196,9 @@ public class ReferenceExp extends AccessExp
         Symbol symbol = (Symbol) getSymbol();
         Object fval = Environment.getCurrent().getFunction(symbol, null);
         if (fval instanceof Procedure)
-          return new QuoteExp(fval).inline(exp, walker, null, argsInlined);
+          return new QuoteExp(fval).validateApply(exp, visitor, required, null);
       }
-    if (! argsInlined)
-      exp.args = walker.walkExps(exp.args, exp.args.length);
+    exp.visitArgs(visitor);
     return exp;
   }
 
@@ -232,7 +232,7 @@ public class ReferenceExp extends AccessExp
     if (type == null || type == Type.pointer_type)
       {
         Expression value = decl.getValue();
-        if (value != null)
+        if (value != null && value != QuoteExp.undefined_exp)
           {
             // Kludge to guard against cycles.
             // Not verified if it is really needed, but just in case ...
@@ -242,6 +242,8 @@ public class ReferenceExp extends AccessExp
             decl.value = save;
           }
       }
+    if (type == Type.toStringType)
+      type = Type.javalangStringType;
     return type;
   }
 
