@@ -127,6 +127,19 @@ public abstract class ScopeExp extends Expression
       }
   }
 
+  /** Return the outermost non-module scope, if there is one. */
+  public ScopeExp topLevel ()
+  {
+    ScopeExp exp = this;
+    for (;; )
+      {
+        ScopeExp outer = exp.outer;
+        if (outer == null || outer instanceof ModuleExp)
+          return exp;
+        exp = outer;
+      }
+  }
+
   public ModuleExp currentModule ()
   {
     ScopeExp exp = this;
@@ -291,9 +304,9 @@ public abstract class ScopeExp extends Expression
     frameSize = i;
   }
 
-  protected Expression walk (ExpWalker walker)
+  protected <R,D> R visit (ExpVisitor<R,D> visitor, D d)
   {
-    return walker.walkScopeExp(this);
+    return visitor.visitScopeExp(this, d);
   }
 
   public String toString() { return getClass().getName()+"#"+id; }
