@@ -14,6 +14,7 @@ public abstract class Buffer extends AbstractSequence implements CharSeq
   Path path;
   String encoding;
   //boolean modified;
+  boolean readOnly;
 
   static Buffer current;
 
@@ -51,7 +52,7 @@ public abstract class Buffer extends AbstractSequence implements CharSeq
   public Path getPath () { return path; }
   public void setPath (Path path) { this.path = path; }
 
-  public String getFileName() { return path.toString(); }
+  public String getFileName() { return path == null ? null : path.toString(); }
 
   public void setFileName(String fname)
   {
@@ -60,11 +61,11 @@ public abstract class Buffer extends AbstractSequence implements CharSeq
       fileBuffers.remove(filename);
     if (name != null && buffers.get(name) == this)
       buffers.remove(name);
-    File file = new File(fname);
-    setPath(FilePath.valueOf(file));
-    name = generateNewBufferName(file.getName());
+    Path path = Path.valueOf(fname);
+    setPath(path);
+    name = generateNewBufferName(path.getLast());
     buffers.put(name, this);
-    fileBuffers.put(filename, this);
+    fileBuffers.put(path.toString(), this);
     redrawModeline();
   }
 
@@ -117,34 +118,34 @@ public abstract class Buffer extends AbstractSequence implements CharSeq
   }
   /* #endif */
   /* #ifdef JAVA5 */
-  // /**
-  //  * @see gnu.lists.CharSeq#writeTo(int, int, Appendable)
-  //  */
-  // public void writeTo(int start, int count, Appendable dest)
-  //   throws java.io.IOException
-  // {
-  //   getStringContent().writeTo(start, count, dest);
-  // }
-
-  // public void writeTo(Appendable dest)
-  //   throws java.io.IOException
-  // {
-  //   writeTo(0, length(), dest);
-  // }
-  /* #else */
   /**
-   * @see gnu.lists.CharSeq#writeTo(int, int, java.io.Writer)
+   * @see gnu.lists.CharSeq#writeTo(int, int, Appendable)
    */
-  public void writeTo(int start, int count, java.io.Writer dest)
+  public void writeTo(int start, int count, Appendable dest)
     throws java.io.IOException
   {
     getStringContent().writeTo(start, count, dest);
   }
 
-  public void writeTo(java.io.Writer str) throws java.io.IOException
+  public void writeTo(Appendable dest)
+    throws java.io.IOException
   {
-    writeTo(0, length(), str);
+    writeTo(0, length(), dest);
   }
+  /* #else */
+  // /**
+  //  * @see gnu.lists.CharSeq#writeTo(int, int, java.io.Writer)
+  //  */
+  // public void writeTo(int start, int count, java.io.Writer dest)
+  //   throws java.io.IOException
+  // {
+  //   getStringContent().writeTo(start, count, dest);
+  // }
+
+  // public void writeTo(java.io.Writer str) throws java.io.IOException
+  // {
+  //   writeTo(0, length(), str);
+  // }
   /* #endif */
 
   /**
@@ -165,12 +166,12 @@ public abstract class Buffer extends AbstractSequence implements CharSeq
 	buffer.encoding = System.getProperty("file.encoding", "UTF8");
         try
           {
-	    Reader in = new InputStreamReader(new FileInputStream(fname),
+	    Reader in = new InputStreamReader(buffer.path.openInputStream(),
 					      buffer.encoding);
             buffer.insertFile(in);
             in.close();
           }
-        catch (java.io.FileNotFoundException ex)
+        catch (java.io.IOException ex)
           {
             Signal.message("New file");
           }
@@ -293,7 +294,7 @@ public abstract class Buffer extends AbstractSequence implements CharSeq
 
   /** Insert count copies of ch at Pos ipos. */
   /*
-  public void insert (char ch, int count, Object style, int ipos)
+  public void insertChar (char ch, int count, Object style, int ipos)
   {
   }
   */
@@ -315,7 +316,7 @@ public abstract class Buffer extends AbstractSequence implements CharSeq
       {
 	Object value = values[i];
 	if (value instanceof Char)
-	  insert(((Char) value).charValue(), 1, style);
+	  insertChar(((Char) value).charValue(), 1, style);
 	else
 	  pointMarker.insert(value.toString(), style);
       }
@@ -329,21 +330,21 @@ public abstract class Buffer extends AbstractSequence implements CharSeq
   public void insert (Object value, Object style)
   {
     if (value instanceof Char)
-      insert(((Char) value).charValue(), 1, style);
+      insertChar(((Char) value).intValue(), 1, style);
     else
       pointMarker.insert(value.toString(), style);
   }
 
   /** Insert count copies of ch at point. */
-  public void insert (char ch, int count)
+  public void insertChar (int ch, int count)
   {
-    pointMarker.insert(ch, count, null);
+    insertChar(ch, count, null);
   }
 
   /** Insert count copies of ch at point. */
-  public void insert (char ch, int count, Object style)
+  public void insertChar (int ch, int count, Object style)
   {
-    pointMarker.insert(ch, count, style);
+    pointMarker.insertChar(ch, count, style);
   }
 
   public void removeChar (int count)
@@ -351,7 +352,13 @@ public abstract class Buffer extends AbstractSequence implements CharSeq
     pointMarker.removeChar(count);
   }
 
-  public abstract void removeAll ();
+  public abstract void removeRegion (int start, int end);
+
+  public void removeAll ()
+  {
+    removeRegion(0, maxDot());
+
+  }
 
   public Marker getPointMarker (boolean share)
   {
@@ -600,4 +607,7 @@ public abstract class Buffer extends AbstractSequence implements CharSeq
    */
   public abstract void invoke(Runnable doRun);
 
+  public boolean getReadOnly () { return readOnly; }
+  public void setReadOnly (boolean readOnly) { this.readOnly = readOnly; }
+
 }
