@@ -2,6 +2,7 @@ package gnu.kawa.functions;
 import gnu.bytecode.*;
 import gnu.expr.*;
 import gnu.mapping.*;
+import gnu.kawa.reflect.CompileReflect;
 import gnu.kawa.reflect.Invoke;
 import gnu.kawa.reflect.ArrayGet;
 import gnu.kawa.reflect.ArraySet;
@@ -33,47 +34,43 @@ public class CompilationHelpers
   // = ClassType.make("gnu.lists.Sequence");
   /* #endif */
 
-  public static Expression inlineApplyToArgs
-  (ApplyExp exp, InlineCalls walker,
-   boolean argsInlined, Procedure applyToArgs)
+  public static Expression validateApplyToArgs
+  (ApplyExp exp, InlineCalls visitor,
+   Type required, Procedure applyToArgs)
   {
     Expression[] args = exp.getArgs();
     int nargs = args.length - 1;
     if (nargs >= 0)
       {
         Expression proc = args[0];
-        if (! argsInlined)
+        if (! proc.getFlag(Expression.VALIDATED))
           {
             if (proc instanceof LambdaExp)
               {
                 Expression[] rargs = new Expression[nargs];
                 System.arraycopy(args, 1, rargs, 0, nargs);
-                return walker.walk(new ApplyExp(proc, rargs));
+                return visitor.visit(new ApplyExp(proc, rargs).setLine(exp), required);
               }
-            proc = walker.walk(proc);
+            proc = visitor.visit(proc, null);
             args[0] = proc;
           }
         Type ptype = proc.getType().getRealType();
-        ApplyExp result;
-        Compilation comp = walker.getCompilation();
+        Compilation comp = visitor.getCompilation();
         Language language = comp.getLanguage();
         if (ptype.isSubtype(Compilation.typeProcedure))
           {
             Expression[] rargs = new Expression[nargs];
             System.arraycopy(args, 1, rargs, 0, nargs);
-            return proc.inline(new ApplyExp(proc, rargs), walker, null,
-                               argsInlined);
+            ApplyExp nexp = new ApplyExp(proc, rargs);
+            nexp.setLine(exp);
+            return proc.validateApply(nexp, visitor, required, null);
           }
-        if (! argsInlined)
-          {
-            for (int i = 1; i <= nargs;  i++)
-              args[i] = walker.walk(args[i]);
-          }
-        // This might be more cleanly handled at the type specifier. FIXME
-        if (Invoke.checkKnownClass(ptype, comp) < 0)
-          return exp;
+
         ClassType ctype;
-        if (ptype.isSubtype(Compilation.typeType)
+        ApplyExp result = null;
+        if (CompileReflect.checkKnownClass(ptype, comp) < 0)
+          ; // This might be more cleanly handled at the type specifier. FIXME
+        else if (ptype.isSubtype(Compilation.typeType)
                  || language.getTypeFor(proc,false) != null)
           {
             result = new ApplyExp(Invoke.make, args);
@@ -93,11 +90,13 @@ public class CompilationHelpers
             Method get = ctype.getMethod("get", new Type[] { Type.intType  });
             result = new ApplyExp(get, args);
           }
-        else
-          return exp;
-        result.setLine(exp);
-        return ((InlineCalls) walker).walkApplyOnly(result);
+        if (result != null)
+          {
+            result.setLine(exp);
+            return ((InlineCalls) visitor).visitApplyOnly(result, required);
+          }
       }
+    exp.visitArgs(visitor);
     return exp;
   }
 
@@ -106,11 +105,10 @@ public class CompilationHelpers
   public static final Declaration setterDecl = new Declaration("setter", setterField);
   static { setterDecl.noteValue(new QuoteExp(Setter.setter)); }
 
-  public static Expression inlineSetter
-  (ApplyExp exp, InlineCalls walker,
-   boolean argsInlined, Procedure proc)
+  public static Expression validateSetter
+  (ApplyExp exp, InlineCalls visitor, Type required, Procedure proc)
   {
-    exp.walkArgs(walker, argsInlined);
+    exp.visitArgs(visitor);
     Expression[] args = exp.getArgs();
     if (args.length == 1)
       {
@@ -156,11 +154,10 @@ public class CompilationHelpers
     return exp;
   }
 
-  public static Expression inlineIsEqv
-  (ApplyExp exp, InlineCalls walker,
-   boolean argsInlined, Procedure proc)
+  public static Expression validateIsEqv
+  (ApplyExp exp, InlineCalls visitor, Type required, Procedure proc)
   {
-    exp.walkArgs(walker, argsInlined);
+    exp.visitArgs(visitor);
     Expression[] args = exp.getArgs();
     if (nonNumeric(args[0]) || nonNumeric(args[1]))
       return new ApplyExp(((IsEqv) proc).isEq, args);
@@ -181,11 +178,10 @@ class SetArrayExp extends ApplyExp
     elementType = arrayType.getComponentType();
   }
 
-  public Expression inline (ApplyExp exp, InlineCalls walker,
-                            Declaration decl, boolean argsInlined)
+  public Expression validateApply (ApplyExp exp, InlineCalls visitor,
+                                   Type required, Declaration decl)
   {
-    if (! argsInlined)
-      exp.walkArgs(walker);
+    exp.visitArgs(visitor);
     Expression[] args = exp.getArgs();
     if (args.length == 2)
       {
@@ -195,7 +191,7 @@ class SetArrayExp extends ApplyExp
         xargs[1] = args[0];
         xargs[2] = args[1];
         ArraySet arrSetter = new ArraySet(elementType);
-        return walker.walkApplyOnly(new ApplyExp(arrSetter, xargs));
+        return visitor.visitApplyOnly(new ApplyExp(arrSetter, xargs), required);
       }
     return exp;
   }
@@ -208,11 +204,10 @@ class SetListExp extends ApplyExp
     super(func, args);
   }
 
-  public Expression inline (ApplyExp exp, InlineCalls walker,
-                            Declaration decl, boolean argsInlined)
+  public Expression validateApply (ApplyExp exp, InlineCalls visitor,
+                                   Type required, Declaration decl)
   {
-    if (! argsInlined)
-      exp.walkArgs(walker);
+    exp.visitArgs(visitor);
     Expression[] args = exp.getArgs();
     if (args.length == 2)
       {
@@ -222,7 +217,7 @@ class SetListExp extends ApplyExp
         xargs[2] = Compilation.makeCoercion(args[0], Type.intType);
         xargs[3] = args[1];
         Expression set
-          = walker.walkApplyOnly(new ApplyExp(Invoke.invoke, xargs));
+          = visitor.visitApplyOnly(new ApplyExp(Invoke.invoke, xargs), required);
         return Compilation.makeCoercion(set, Type.voidType);
       }
     return exp;
