@@ -6,7 +6,7 @@ import gnu.expr.*;
 import gnu.kawa.lispexpr.*;
 import static gnu.kawa.functions.ArithOp.*;
 
-public class CompileArith implements CanInline, Inlineable
+public class CompileArith implements Inlineable
 {
   int op;
   Procedure proc;
@@ -27,8 +27,7 @@ public class CompileArith implements CanInline, Inlineable
 
   public static CompileArith forDiv(Object proc)
   {
-    int op = ((DivideOp) proc).op == DivideOp.MODULO ? REM : DIV;
-    return new CompileArith(proc, op);
+    return new CompileArith(proc, ((DivideOp) proc).op);
   }
 
   public static CompileArith forBitwise(Object proc)
@@ -36,54 +35,40 @@ public class CompileArith implements CanInline, Inlineable
     return new CompileArith(proc, ((BitwiseOp) proc).op);
   }
 
-  public boolean appropriateIntConstant(Expression[] args, int iarg)
+  public static boolean appropriateIntConstant(Expression[] args, int iarg, InlineCalls visitor)
   {
-    Expression arg = args[iarg];
-    if (arg instanceof QuoteExp)
+    Expression exp = visitor.fixIntValue(args[iarg]);
+    if (exp != null)
       {
-        QuoteExp qarg = (QuoteExp) arg;
-        Object value = qarg.getValue();
-        if (qarg.getRawType() == null && value instanceof IntNum
-            && inRange((IntNum) value, Integer.MIN_VALUE, Integer.MAX_VALUE))
-          {
-            value = Integer.valueOf(((IntNum) value).intValue());
-            arg = new QuoteExp(value, Type.intType);
-            args[iarg] = arg;
-            return true;
-          }
+        args[iarg] = exp;
+        return true;
       }
     return false;
   }
 
-  public boolean appropriateLongConstant(Expression[] args, int iarg)
+  public static boolean appropriateLongConstant(Expression[] args, int iarg, InlineCalls visitor)
   {
-    Expression arg = args[iarg];
-    if (arg instanceof QuoteExp)
+    Expression exp = visitor.fixLongValue(args[iarg]);
+    if (exp != null)
       {
-        QuoteExp qarg = (QuoteExp) arg;
-        Object value = qarg.getValue();
-        if (qarg.getRawType() == null && value instanceof IntNum
-            && inRange((IntNum) value, Long.MIN_VALUE, Long.MAX_VALUE))
-          {
-            value = Long.valueOf(((IntNum) value).longValue());
-            arg = new QuoteExp(value, Type.longType);
-            args[iarg] = arg;
-            return true;
-          }
+        args[iarg] = exp;
+        return true;
       }
     return false;
   }
 
-  public Expression inline (ApplyExp exp, InlineCalls walker,
-                            boolean argsInlined)
+  public static Expression validateApplyArithOp
+  (ApplyExp exp, InlineCalls visitor, Type required, Procedure proc)
   {
-    exp.walkArgs(walker, argsInlined);
+    ArithOp aproc = (ArithOp) proc;
+    int op = aproc.op;
+    exp.visitArgs(visitor);
  
     Expression[] args = exp.getArgs();
     if (args.length > 2)
-      return pairwise(proc, exp.getFunction(), args, walker);
+      return pairwise(proc, exp.getFunction(), args, visitor);
 
-    Expression folded = exp.inlineIfConstant(proc, walker);
+    Expression folded = exp.inlineIfConstant(proc, visitor);
     if (folded != exp)
       return folded;
 
@@ -96,16 +81,16 @@ public class CompileArith implements CanInline, Inlineable
             && (op < ASHIFT_GENERAL || op > LSHIFT_RIGHT))
           {
             int kind2 = Arithmetic.classifyType(args[1].getType());
-            rkind = getReturnKind(kind1, kind2);
+            rkind = getReturnKind(kind1, kind2, op);
             if (rkind == Arithmetic.INTNUM_CODE)
               {
-                if (kind1 == Arithmetic.INT_CODE && appropriateIntConstant(args, 1))
+                if (kind1 == Arithmetic.INT_CODE && appropriateIntConstant(args, 1, visitor))
                   rkind = Arithmetic.INT_CODE;
-                else if (kind2 == Arithmetic.INT_CODE && appropriateIntConstant(args, 0))
+                else if (kind2 == Arithmetic.INT_CODE && appropriateIntConstant(args, 0, visitor))
                   rkind = Arithmetic.INT_CODE;
-                else if (kind1 ==Arithmetic. LONG_CODE && appropriateLongConstant(args, 1))
+                else if (kind1 == Arithmetic. LONG_CODE && appropriateLongConstant(args, 1, visitor))
                   rkind = Arithmetic.LONG_CODE;
-                else if (kind2 == Arithmetic.LONG_CODE && appropriateLongConstant(args, 0))
+                else if (kind2 == Arithmetic.LONG_CODE && appropriateLongConstant(args, 0, visitor))
                   rkind = Arithmetic.LONG_CODE;
               }
           }
@@ -113,26 +98,29 @@ public class CompileArith implements CanInline, Inlineable
           {
             rkind = kind1;
           }
-        rkind = adjustReturnKind(rkind);
+        rkind = adjustReturnKind(rkind, op);
         exp.setType(Arithmetic.kindType(rkind));
       }
 
     // Inlining may yield PrimProcedure instructions of bytecode instructions
     // which we don't know how to interpret (yet).
-    if (! walker.getCompilation().mustCompile)
+    if (! visitor.getCompilation().mustCompile)
       return exp;
 
     switch (op)
       {
       case ADD:
       case SUB:
-        return inlineAdd((AddOp) proc, exp, walker);
-      case DIV:
-      case REM:
-        return inlineDiv((DivideOp) proc, exp, walker);
+        return validateApplyAdd((AddOp) proc, exp, visitor);
+      case DIVIDE_GENERIC:
+      case DIVIDE_INEXACT:
+      case QUOTIENT:
+      case QUOTIENT_EXACT:
+      case MODULO:
+        return validateApplyDiv((DivideOp) proc, exp, visitor);
       case NOT:
         if (rkind > 0)
-          return inlineNot(exp, rkind, walker);
+          return validateApplyNot(exp, rkind, visitor);
         // else fall through ...
       default:
         return exp;
@@ -158,7 +146,7 @@ public class CompileArith implements CanInline, Inlineable
     // We expect len == 2, assuming inline has been run.
     int kind1 = Arithmetic.classifyType(args[0].getType());
     int kind2 = Arithmetic.classifyType(args[1].getType());
-    int kind = getReturnKind(kind1, kind2);
+    int kind = getReturnKind(kind1, kind2, op);
     Type type = Arithmetic.kindType(kind);
     if (kind == 0 || len != 2 /* just in case */)
       {
@@ -194,26 +182,30 @@ public class CompileArith implements CanInline, Inlineable
     else
       wtype = type;
 
-    if (op == DIV || op == REM)
+    if (op >= DIVIDE_GENERIC && op <= MODULO)
       {
         DivideOp dproc = (DivideOp) proc;
-        if (dproc.op == DivideOp.GENERIC
+        if (dproc.op == DivideOp.DIVIDE_GENERIC
             && (kind <= Arithmetic.INTNUM_CODE
                 || (kind >= Arithmetic.RATNUM_CODE || kind <= Arithmetic.FLONUM_CODE)))
           ;
         else if ((dproc.op == DivideOp.DIVIDE_INEXACT
                   && kind <= Arithmetic.REALNUM_CODE && kind != Arithmetic.FLOAT_CODE)
-                 || (dproc.op == DivideOp.GENERIC && kind == Arithmetic.REALNUM_CODE))
+                 || (dproc.op == DivideOp.DIVIDE_GENERIC && kind == Arithmetic.REALNUM_CODE))
           kind = Arithmetic.DOUBLE_CODE;
         else if ((dproc.op == DivideOp.QUOTIENT_EXACT
                   || (dproc.op == DivideOp.QUOTIENT
                       && kind <= Arithmetic.INTNUM_CODE))
                  && (dproc.getRoundingMode() == Numeric.TRUNCATE
+                     || kind == Arithmetic.INTNUM_CODE
                      || kind == Arithmetic.FLOAT_CODE
-                     || kind == Arithmetic.DOUBLE_CODE))
+                     || kind == Arithmetic.DOUBLE_CODE)) 
+          // FIXME Should optimize primitive non-TRUNCATE quotient.
           ;
         else if (dproc.op == DivideOp.MODULO
-                 && dproc.getRoundingMode() == Numeric.TRUNCATE)
+                 && (dproc.getRoundingMode() == Numeric.TRUNCATE
+                     || kind == Arithmetic.INTNUM_CODE))
+          // FIXME Should optimize primitive non-TRUNCATE modulo.
           ;
         else
           {
@@ -221,7 +213,7 @@ public class CompileArith implements CanInline, Inlineable
             return;
           }
       }
-    if (op == DIV && ((DivideOp) proc).op == DivideOp.GENERIC
+    if (op == DIVIDE_GENERIC
         && kind <= Arithmetic.REALNUM_CODE 
         && kind != Arithmetic.DOUBLE_CODE && kind != Arithmetic.FLOAT_CODE)
       {
@@ -249,6 +241,7 @@ public class CompileArith implements CanInline, Inlineable
     else if (kind == Arithmetic.INTNUM_CODE
              && (op == ADD || op == MUL || op == SUB
                  || op == AND || op == IOR || op == XOR
+                 || op == QUOTIENT_EXACT || op == MODULO
                  || (op >= ASHIFT_GENERAL && op <= ASHIFT_RIGHT)))
       {
         compileIntNum(args[0], args[1], kind1, kind2, comp);
@@ -257,7 +250,7 @@ public class CompileArith implements CanInline, Inlineable
              || kind == Arithmetic.LONG_CODE
              || ((kind == Arithmetic.FLOAT_CODE
                   || kind == Arithmetic.DOUBLE_CODE)
-                 && (op <= REM || op >= AND)))
+                 && (op <= MODULO || op >= AND)))
       {
         Target wtarget = StackTarget.getInstance(wtype);
 
@@ -295,17 +288,6 @@ public class CompileArith implements CanInline, Inlineable
     target.compileFromStack(comp, wtype);
   }
 
-  static boolean inRange (Expression exp, int lo, int hi)
-  {
-    Object val = exp.valueIfConstant();
-    return val instanceof IntNum && inRange((IntNum) val, lo, hi);
-  }
-
-  public static boolean inRange (IntNum val, long lo, long hi)
-  {
-    return IntNum.compare(val, lo) >= 0 && IntNum.compare(val, hi) <= 0;
-  }
-
   public boolean compileIntNum (Expression arg1, Expression arg2, int kind1, int kind2, Compilation comp)
   {
     // Check if we can replace ARG1-CONSTANT by ARG1+(-CONSTANT),
@@ -324,7 +306,7 @@ public class CompileArith implements CanInline, Inlineable
           {
             IntNum ival = (IntNum) val;
             lval =  ival.longValue();
-            negateOk = inRange(ival, Integer.MIN_VALUE+1, Integer.MAX_VALUE);
+            negateOk = ival.inRange(Integer.MIN_VALUE+1, Integer.MAX_VALUE);
           }
         else
           {
@@ -342,9 +324,9 @@ public class CompileArith implements CanInline, Inlineable
     Method meth;
     if (addOrMul)
       {
-        if (inRange(arg1, Integer.MIN_VALUE, Integer.MAX_VALUE))
+        if (InlineCalls.checkIntValue(arg1) != null)
           kind1 = Arithmetic.INT_CODE;
-        if (inRange(arg2, Integer.MIN_VALUE, Integer.MAX_VALUE))
+        if (InlineCalls.checkIntValue(arg2) != null)
           kind2 = Arithmetic.INT_CODE;
         swap = kind1 == Arithmetic.INT_CODE && kind2 != Arithmetic.INT_CODE;
         if (swap && ! (arg1.side_effects() && arg2.side_effects()))
@@ -392,11 +374,14 @@ public class CompileArith implements CanInline, Inlineable
           mname = "xor";
         mclass = ClassType.make("gnu.math.BitOps");
         break;
-      case DIV:
-      case REM:
-        mname = op == DIV ? "quotient" : "remainder";
+      case DIVIDE_GENERIC:
+      case DIVIDE_INEXACT:
+      case QUOTIENT:
+      case QUOTIENT_EXACT:
+      case MODULO:
+        mname = op == MODULO ? "remainder" : "quotient";
         DivideOp dproc = (DivideOp) proc;
-        if (op == REM && dproc.rounding_mode == Numeric.FLOOR)
+        if (op == MODULO && dproc.rounding_mode == Numeric.FLOOR)
           mname = "modulo";
         else if (dproc.rounding_mode != Numeric.TRUNCATE)
           {
@@ -421,7 +406,7 @@ public class CompileArith implements CanInline, Inlineable
     return true;
   }
 
-  public int getReturnKind (int kind1, int kind2)
+  public static int getReturnKind (int kind1, int kind2, int op)
   {
     if (op >= ASHIFT_GENERAL && op <= LSHIFT_RIGHT)
       return kind1;
@@ -452,17 +437,16 @@ public class CompileArith implements CanInline, Inlineable
   // semi-deprecated.
   public gnu.bytecode.Type getReturnType (Expression[] args)
   {
-    return Arithmetic.kindType(adjustReturnKind(getReturnKind(args)));
+    return Arithmetic.kindType(adjustReturnKind(getReturnKind(args), op));
   }
 
-  int adjustReturnKind (int rkind)
+  static int adjustReturnKind (int rkind, int op)
   {
-    if (op == DIV && rkind > 0)
+    if (op >= DIVIDE_GENERIC && op <= QUOTIENT_EXACT && rkind > 0)
       {
-        DivideOp dproc = (DivideOp) proc;
-        switch (dproc.op)
+        switch (op)
           {
-          case DivideOp.GENERIC:
+          case DivideOp.DIVIDE_GENERIC:
             if (rkind <= Arithmetic.INTNUM_CODE)
               rkind = Arithmetic.RATNUM_CODE;
             break;
@@ -480,7 +464,7 @@ public class CompileArith implements CanInline, Inlineable
     return rkind;
   }
 
-  public Expression inlineAdd (AddOp proc, ApplyExp exp, InlineCalls walker)
+  public static Expression validateApplyAdd (AddOp proc, ApplyExp exp, InlineCalls visitor)
   {
     Expression[] args = exp.getArgs();
     if (args.length == 1 && proc.plusOrMinus < 0)
@@ -526,8 +510,8 @@ public class CompileArith implements CanInline, Inlineable
     return exp;
   }
 
-  public static Expression inlineDiv (DivideOp proc,
-                                      ApplyExp exp, InlineCalls walker)
+  public static Expression validateApplyDiv (DivideOp proc,
+                                      ApplyExp exp, InlineCalls visitor)
   {
     Expression[] args = exp.getArgs();
     if (args.length == 1)
@@ -538,7 +522,7 @@ public class CompileArith implements CanInline, Inlineable
     return exp;
   }
 
-  public Expression inlineNot (ApplyExp exp, int kind, InlineCalls walker)
+  public static Expression validateApplyNot (ApplyExp exp, int kind, InlineCalls visitor)
   {
     if (exp.getArgCount() == 1)
       {
@@ -546,7 +530,7 @@ public class CompileArith implements CanInline, Inlineable
         if (kind == Arithmetic.INT_CODE || kind == Arithmetic.LONG_CODE)
           {
             Expression[] args = {arg, QuoteExp.getInstance(IntNum.minusOne())};
-            return walker.walkApplyOnly(new ApplyExp(BitwiseOp.xor, args));
+            return visitor.visitApplyOnly(new ApplyExp(BitwiseOp.xor, args), null); // FIXME
           }
         String cname;
         if (kind == Arithmetic.INTNUM_CODE)
@@ -562,6 +546,16 @@ public class CompileArith implements CanInline, Inlineable
     return exp;
   }
 
+  public static Expression validateApplyNumberCompare
+  (ApplyExp exp, InlineCalls visitor, Type required, Procedure proc)
+  {
+    exp.visitArgs(visitor);
+    Expression folded = exp.inlineIfConstant(proc, visitor);
+    if (folded != exp)
+      return folded;
+    return exp;
+  }
+
   public int primitiveOpcode ()
   {
     switch (op)
@@ -569,8 +563,12 @@ public class CompileArith implements CanInline, Inlineable
       case ADD:    return 96; /* iadd */
       case SUB:    return 100; /* isub */
       case MUL:    return 104;
-      case DIV:    return 108;
-      case REM:    return 112;
+      case DIVIDE_GENERIC:
+      case DIVIDE_INEXACT:
+      case QUOTIENT:
+      case QUOTIENT_EXACT:
+        return 108;
+      case MODULO:    return 112;
       case ASHIFT_LEFT:  return 120; // ishl
       case ASHIFT_RIGHT:  return 122; // ishr
       case LSHIFT_RIGHT:  return 124; // iushr
@@ -585,7 +583,7 @@ public class CompileArith implements CanInline, Inlineable
    */
   public static Expression pairwise(Procedure proc,
                                     Expression rproc, Expression[] args,
-				    InlineCalls walker)
+				    InlineCalls visitor)
   {
     int len = args.length;
     Expression prev = args[0];
@@ -595,9 +593,20 @@ public class CompileArith implements CanInline, Inlineable
         args2[0] = prev;
         args2[1] = args[i];
         ApplyExp next = new ApplyExp(rproc, args2);
-        Expression inlined = walker.maybeInline(next, true, proc);
+        Expression inlined = visitor.maybeInline(next, null/*FIXME*/, proc);
         prev = inlined != null ? inlined : next;
       }
     return prev;
   }
+
+  public static Expression validateApplyNumberPredicate
+  (ApplyExp exp, InlineCalls visitor, Type required, Procedure proc)
+  {
+    NumberPredicate nproc = (NumberPredicate) proc;
+    int op = nproc.op;
+    Expression[] args = exp.getArgs();
+    args[0] = visitor.visit(args[0], LangObjType.integerType);
+    exp.setType(Type.booleanType);
+    return exp;
+  }
 }
