@@ -7,11 +7,10 @@ import kawa.lang.Translator;
 
 public class CompileNamedPart
 {
-  public static Expression inlineGetNamedPart
-  (ApplyExp exp, InlineCalls walker,
-   boolean argsInlined, Procedure proc)
+  public static Expression validateGetNamedPart
+  (ApplyExp exp, InlineCalls visitor, Type required, Procedure proc)
   {
-    exp.walkArgs(walker, argsInlined);
+    exp.visitArgs(visitor);
     Expression[] args = exp.getArgs();
     if (args.length != 2 || ! (args[1] instanceof QuoteExp)
         || ! (exp instanceof GetNamedExp))
@@ -29,7 +28,7 @@ public class CompileNamedPart
     String mname = ((QuoteExp) args[1]).getValue().toString();
     Type type = context.getType();
     boolean isInstanceOperator = context == QuoteExp.nullExp;
-    Compilation comp = walker.getCompilation();
+    Compilation comp = visitor.getCompilation();
     Language language = comp.getLanguage();
     Type typeval = language.getTypeFor(context, false);
     ClassType caller = comp == null ? null
@@ -61,7 +60,7 @@ public class CompileNamedPart
             // However, it makes optimzing the 'setter' case harder.
             return new QuoteExp(new NamedPart(typeval, mname, 'D'));
           }
-        if (Invoke.checkKnownClass(typeval, comp) < 0)
+        if (CompileReflect.checkKnownClass(typeval, comp) < 0)
           return exp;
         PrimProcedure[] methods
           = ClassMethods.getMethods((ObjectType) typeval,
@@ -74,7 +73,7 @@ public class CompileNamedPart
           }
         ApplyExp aexp = new ApplyExp(SlotGet.staticField, args);
         aexp.setLine(exp);
-        return walker.walkApplyOnly(aexp);
+        return visitor.visitApplyOnly(aexp, required);
                             
       }
     if (typeval != null)
@@ -134,45 +133,49 @@ public class CompileNamedPart
             // args = new Expression[] { context, new QuoteExp(part) });
             ApplyExp aexp = new ApplyExp(SlotGet.field, args);
             aexp.setLine(exp);
-            return walker.walkApplyOnly(aexp);
+            return visitor.visitApplyOnly(aexp, required);
           }
       }
 
-    if (comp.getBooleanOption("warn-invoke-unknown-method", ! comp.immediate))
+    if (comp.warnUnknownMember())
       comp.error('w', "no known slot '"+mname+"' in "+type.getName());
     return exp;
   }
 
-  public static Expression inlineSetNamedPart
-  (ApplyExp exp, InlineCalls walker,
-   boolean argsInlined, Procedure proc)
+  public static Expression validateSetNamedPart
+  (ApplyExp exp, InlineCalls visitor, Type required, Procedure proc)
   {
-    exp.walkArgs(walker, argsInlined);
+    exp.visitArgs(visitor);
     Expression[] args = exp.getArgs();
     if (args.length != 3 || ! (args[1] instanceof QuoteExp))
       return exp;
     Expression context = args[0];
     String mname = ((QuoteExp) args[1]).getValue().toString();
     Type type = context.getType();
-    Compilation comp = walker.getCompilation();
+    Compilation comp = visitor.getCompilation();
     Language language = comp.getLanguage();
     Type typeval = language.getTypeFor(context);
     ClassType caller = comp == null ? null
       : comp.curClass != null ? comp.curClass
       : comp.mainClass;
+    ApplyExp original = exp;
     if (typeval instanceof ClassType)
-      return new ApplyExp(SlotSet.set$Mnstatic$Mnfield$Ex, args);
-
-    if (type instanceof ClassType)
+      {
+        exp = new ApplyExp(SlotSet.set$Mnstatic$Mnfield$Ex, args);
+      }
+    else if (type instanceof ClassType)
       {
         Object part = SlotSet.lookupMember((ClassType) type, mname, caller);
         if (part != null)
           {
             // FIXME: future kludge to avoid re-doing SlotGet.getField.
             // args = new Expression[] { context, new QuoteExp(part) });
-            return new ApplyExp(SlotSet.set$Mnfield$Ex, args);
+            exp = new ApplyExp(SlotSet.set$Mnfield$Ex, args);
           }
       }
+    if (exp != original)
+      exp.setLine(original);
+    exp.setType(Type.voidType);
     return exp;
   }
 
@@ -241,11 +244,10 @@ public class CompileNamedPart
     return makeExp(new QuoteExp(type), new QuoteExp(member));
   }
 
-  public static Expression inlineNamedPart
-  (ApplyExp exp, InlineCalls walker,
-   boolean argsInlined, Procedure proc)
+  public static Expression validateNamedPart
+  (ApplyExp exp, InlineCalls visitor, Type required, Procedure proc)
   {
-    exp.walkArgs(walker, argsInlined);
+    exp.visitArgs(visitor);
     Expression[] args = exp.getArgs();
     NamedPart namedPart = (NamedPart) proc;
     switch (namedPart.kind)
@@ -268,16 +270,15 @@ public class CompileNamedPart
           }
         ApplyExp aexp = new ApplyExp(slotProc, xargs);
         aexp.setLine(exp);
-        return walker.walkApplyOnly(aexp);
+        return visitor.visitApplyOnly(aexp, required);
       }
     return exp;
   }
 
-  public static Expression inlineNamedPartSetter
-  (ApplyExp exp, InlineCalls walker,
-   boolean argsInlined, Procedure proc)
+  public static Expression validateNamedPartSetter
+  (ApplyExp exp, InlineCalls visitor, Type required, Procedure proc)
   {
-    exp.walkArgs(walker, argsInlined);
+    exp.visitArgs(visitor);
     NamedPart get = (NamedPart) ((NamedPartSetter) proc).getGetter();
     if (get.kind == 'D')
       {
@@ -300,7 +301,7 @@ public class CompileNamedPart
           return exp;
         ApplyExp aexp = new ApplyExp(slotProc, xargs);
         aexp.setLine(exp);
-        return walker.walkApplyOnly(aexp);
+        return visitor.visitApplyOnly(aexp, required);
       }
     return exp;
   }
@@ -323,11 +324,10 @@ public class CompileNamedPart
     return new ApplyExp(Invoke.make, args);
   }
 
-  public static Expression inlineGetNamedInstancePart
-  (ApplyExp exp, InlineCalls walker,
-   boolean argsInlined, Procedure proc)
+  public static Expression validateGetNamedInstancePart
+  (ApplyExp exp, InlineCalls visitor, Type required, Procedure proc)
   {
-    exp.walkArgs(walker, argsInlined);
+    exp.visitArgs(visitor);
     Expression[] args = exp.getArgs();
     Expression[] xargs;
     Procedure property;
@@ -346,19 +346,18 @@ public class CompileNamedPart
         System.arraycopy(args, 1, xargs, 2, nargs-1);
         property = Invoke.invoke;
       }
-    return walker.walkApplyOnly(new ApplyExp(property, xargs));
+    return visitor.visitApplyOnly(new ApplyExp(property, xargs), required);
   }
 
-  public static Expression inlineSetNamedInstancePart
-  (ApplyExp exp, InlineCalls walker,
-   boolean argsInlined, Procedure proc)
+  public static Expression validateSetNamedInstancePart
+  (ApplyExp exp, InlineCalls visitor, Type required, Procedure proc)
   {
-    exp.walkArgs(walker, argsInlined);
+    exp.visitArgs(visitor);
     Expression[] args = exp.getArgs();
     String pname = ((SetNamedInstancePart) proc).pname;
     Expression[] xargs = new Expression[]
       { args[0], new QuoteExp(pname), args[1] };
-    return walker.walkApplyOnly(new ApplyExp(SlotSet.set$Mnfield$Ex, xargs));
+    return visitor.visitApplyOnly(new ApplyExp(SlotSet.set$Mnfield$Ex, xargs), required);
   }
 }
 
@@ -382,7 +381,7 @@ class GetNamedExp extends ApplyExp
   {
     if (combinedName != null)
       {
-        Environment env = ctx.getEnvironment();
+        Environment env = Environment.getCurrent();
         Symbol sym = env.getSymbol(combinedName);
         Object unb = gnu.mapping.Location.UNBOUND;
         Object property = null;  // FIXME?
@@ -399,8 +398,6 @@ class GetNamedExp extends ApplyExp
   public GetNamedExp(Expression[] args)
   {
     super(GetNamedPart.getNamedPart, args);
-    setProperty(Procedure.inlinerKey,
-                "gnu.kawa.functions.CompileGetNamedPart:inlineGetNamedExp");
   }
 
   protected GetNamedExp setProcedureKind (char kind)
@@ -416,11 +413,9 @@ class GetNamedExp extends ApplyExp
     return this;
   }
 
- public Expression inline (ApplyExp exp, InlineCalls walker,
-                            Declaration decl, boolean argsInlined)
+  public Expression validateApply (ApplyExp exp, InlineCalls visitor,
+                                   Type required, Declaration decl)
   {
-    if (! argsInlined)
-      exp.walkArgs(walker);
     Expression[] pargs = getArgs();
     Expression context = pargs[0];
     Expression[] args = exp.getArgs();
@@ -466,7 +461,7 @@ class GetNamedExp extends ApplyExp
       }
     ApplyExp result = new ApplyExp(new ReferenceExp(decl), xargs);
     result.setLine(exp);
-    return walker.walkApplyOnly(result);
+    return visitor.visit(result, required);
   }
 
   public boolean side_effects ()
