@@ -235,7 +235,8 @@ public class DisplayFormat extends AbstractFormat
     /* #ifdef enable:XML */
     else if (obj instanceof KNode)
       {
-        write("#", out);
+        if (getReadableOutput())
+          write("#", out);
         Writer wout = out instanceof Writer ? (Writer) out
           : new ConsumerWriter(out);
         XMLPrinter xout = new XMLPrinter(wout);
@@ -243,6 +244,8 @@ public class DisplayFormat extends AbstractFormat
         xout.closeThis();
       }
     /* #endif */
+    else if (obj == Values.empty && getReadableOutput())
+      write("#!void", out);
     else if (obj instanceof Consumable)
       ((Consumable) obj).consume(out);
     else if (obj instanceof Printable)
@@ -277,6 +280,12 @@ public class DisplayFormat extends AbstractFormat
         if (showRadix && b == 10 && obj instanceof IntNum)
           write(".", out);
       }
+    else if (obj instanceof java.lang.Enum && getReadableOutput())
+      {
+        write(obj.getClass().getName(), out);
+        write(":", out);
+        write(((java.lang.Enum) obj).name(), out);
+      }
     else
       {
         String asString;
