@@ -186,6 +186,8 @@ public class LangPrimType extends PrimType implements TypeValue
       }
     if (sig1 == 'V' || sig1 == 'Z')
       return 1;
+    if (sig1 == 'C' && other.getName().equals("gnu.text.Char"))
+      return -1;
     if (other instanceof LangObjType)
       return swappedCompareResult(other.compare(this));
     return super.compare(other);
@@ -212,6 +214,11 @@ public class LangPrimType extends PrimType implements TypeValue
     code.emitIfIntNotZero();
   }
 
+  public Expression convertValue (Expression value)
+  {
+    return null;
+  }
+
   public void emitIsInstance(Variable incoming,
 			     Compilation comp, Target target)
   {
