@@ -4,7 +4,7 @@ import gnu.bytecode.*;
 import gnu.lists.FString;
 import gnu.expr.*;
 
-public class SlotSet extends Procedure3 implements CanInline, Inlineable
+public class SlotSet extends Procedure3 implements Inlineable
 {
   /** True if this is a "static-field" operation. */
   boolean isStatic;
@@ -24,6 +24,8 @@ public class SlotSet extends Procedure3 implements CanInline, Inlineable
   {
     super(name);
     this.isStatic = isStatic;
+    setProperty(Procedure.validateApplyKey,
+                   "gnu.kawa.reflect.CompileReflect:validateApplySlotSet");
   }
 
   public static void setField (Object obj, String name, Object value)
@@ -36,15 +38,29 @@ public class SlotSet extends Procedure3 implements CanInline, Inlineable
     apply(true, obj, name, value);
   }
 
-  public static void apply (boolean isStatic, Object obj, String name, Object value)
+  public static void apply (boolean isStatic, Object obj, Object member, Object value)
   {
     Language language = Language.getDefaultLanguage();
     boolean illegalAccess = false;
-    String fname = gnu.expr.Compilation.mangleNameIfNeeded(name);
-    Class clas = isStatic ? SlotGet.coerceToClass(obj) : obj.getClass();
+    String name;
+    String fname;
+    Class clas;
+    if (member instanceof String
+        || member instanceof FString
+        || member instanceof Symbol) {
+      name = member.toString();
+      fname = gnu.expr.Compilation.mangleNameIfNeeded(name);
+      clas = isStatic ? SlotGet.coerceToClass(obj) : obj.getClass();
+    }
+    else {
+      fname = name = ((Member) member).getName();
+      clas = null;
+    }
     try
       {
-        java.lang.reflect.Field field = clas.getField(fname);
+        java.lang.reflect.Field field
+          = member instanceof Field ? ((Field) member).getReflectField()
+          : clas.getField(fname);
 	Class ftype = field.getType();
         field.set(obj, language.coerceFromObject(ftype, value));
         return;
@@ -63,15 +79,22 @@ public class SlotSet extends Procedure3 implements CanInline, Inlineable
       {
         java.lang.reflect.Method getmethod = null;
     
+        boolean haveSetter = member instanceof Method;
+	String setName = haveSetter ? fname
+          : ClassExp.slotToMethodName("set", name);
+        if (haveSetter && ! setName.startsWith("set"))
+          haveSetter = false;
+
         try {
-          String getName = ClassExp.slotToMethodName("get", name);
+          String getName = haveSetter ? "get" + setName.substring(3)
+            : ClassExp.slotToMethodName("get", name);
           getmethod = clas.getMethod(getName, SlotGet.noClasses);
         } catch (Exception getEx) {
-          String getName = ClassExp.slotToMethodName("is", name);
+          String getName = haveSetter ? "is" + setName.substring(3)
+            : ClassExp.slotToMethodName("is", name);
           getmethod = clas.getMethod(getName, SlotGet.noClasses);
         }
         
-	String setName = ClassExp.slotToMethodName("set", name);
         Class[] setArgTypes = new Class[1];
         setArgTypes[0] = getmethod.getReturnType();
         java.lang.reflect.Method setmethod
@@ -102,11 +125,12 @@ public class SlotSet extends Procedure3 implements CanInline, Inlineable
 
   public Object apply3 (Object obj, Object fname, Object value)
   {
-    // Should actually check type rather than just calling toString.  FIXME.
-    apply(isStatic, obj, fname.toString(), value);
+    apply(isStatic, obj, fname, value);
     return returnSelf ? obj : Values.empty;
   }
 
+  static final Type[] type1Array = new Type[1];
+
   public static Member
   lookupMember (ObjectType clas, String name, ClassType caller)
   {
@@ -121,14 +145,14 @@ public class SlotSet extends Procedure3 implements CanInline, Inlineable
       }
 
     String setName = ClassExp.slotToMethodName("set", name);
-    gnu.bytecode.Method method = clas.getMethod(setName, new Type[1]);
+    gnu.bytecode.Method method = clas.getMethod(setName, type1Array);
     if (method == null)
       return field;
     else
       return method;
   }
 
-  static void compileSet(Procedure thisProc, ClassType ctype,
+  static void compileSet(Procedure thisProc, ObjectType ctype,
                          Expression valArg, Object part, Compilation comp)
   {
     CodeAttr code = comp.getCode();
@@ -164,24 +188,12 @@ public class SlotSet extends Procedure3 implements CanInline, Inlineable
           code.emitInvokeStatic(method);
         else
           code.emitInvoke(method);
+        if (! method.getReturnType().isVoid())
+          code.emitPop(1);
         return;
       }
   }
 
-  public Expression inline (ApplyExp exp, InlineCalls walker,
-                            boolean argsInlined)
-  {
-    exp.walkArgs(walker, argsInlined);
-    // Unlike, for SlotGet, we do the field-lookup at compile time
-    // rather than inline time.  The main reason is that optimizing
-    // (set! CLASS-OR-OBJECT:FIELD-NAME VALUE) is tricky, since (currently)
-    // afte we've inlined setter, this method doesn't get called.
-    if (isStatic && walker.getCompilation().mustCompile)
-      return Invoke.inlineClassName (exp, 0, walker);
-    else
-      return exp;
-  }
-
   public void compile (ApplyExp exp, Compilation comp, Target target)
   {
     Expression[] args = exp.getArgs();
@@ -199,10 +211,10 @@ public class SlotSet extends Procedure3 implements CanInline, Inlineable
     Type type = isStatic ? kawa.standard.Scheme.exp2Type(arg0)
       : arg0.getType();
     Member part = null;
-    if (type instanceof ClassType && arg1 instanceof QuoteExp)
+    if (type instanceof ObjectType && arg1 instanceof QuoteExp)
       {
         Object val1 = ((QuoteExp) arg1).getValue();
-        ClassType ctype = (ClassType) type;
+        ObjectType ctype = (ObjectType) type;
 	String name;
         ClassType caller = comp.curClass != null ? comp.curClass
           : comp.mainClass;
@@ -212,8 +224,9 @@ public class SlotSet extends Procedure3 implements CanInline, Inlineable
 	  {
             name = val1.toString();
 	    part = lookupMember(ctype, name, caller);
-	    if (part == null && type != Type.pointer_type)
-	      comp.error('e', "no slot `"+name+"' in "+ctype.getName());
+	    if (part == null && type != Type.pointer_type
+                && comp.warnUnknownMember())
+	      comp.error('w', "no slot `"+name+"' in "+ctype.getName());
 	  }
 	else if (val1 instanceof Member)
 	  {
@@ -237,7 +250,7 @@ public class SlotSet extends Procedure3 implements CanInline, Inlineable
 			    isStaticField ? Target.Ignore
 			    : Target.pushValue(ctype));
 	    if (returnSelf)
-	      comp.getCode().emitDup(ctype);
+	      comp.getCode().emitDup(ctype.getImplementationType());
 	    compileSet(this, ctype, args[2], part, comp);
 	    if (returnSelf)
 	      target.compileFromStack(comp, ctype);
@@ -248,11 +261,4 @@ public class SlotSet extends Procedure3 implements CanInline, Inlineable
       }
     ApplyExp.compile(exp, comp, target);
   }
-
-  public Type getReturnType (Expression[] args)
-  {
-    if (returnSelf && args.length == 3)
-      return args[0].getType();
-    return Type.void_type;
-  }
 }
