@@ -1,4 +1,4 @@
-// Copyright (c) 2002. 2006  Per M.A. Bothner and Brainfood Inc.
+// Copyright (c) 2002, 2006, 2010  Per M.A. Bothner and Brainfood Inc.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.kawa.xml;
@@ -15,11 +15,8 @@ import java.util.Vector;
 public class HttpPrinter extends FilterConsumer
 {
   Vector headers = new Vector();
-  /* #ifdef JAVA5 */
+  /** Used as output buffer if base is null. */
   StringBuilder sbuf = new StringBuilder(100);
-  /* #else */
-  // StringBuffer sbuf = new StringBuffer(100);
-  /* #endif */
   Object currentHeader;
 
   /** 1 - implicit; 2: explicit. */
@@ -37,14 +34,12 @@ public class HttpPrinter extends FilterConsumer
   {
     super(null);
     ostream = out;
-    //ostream = System.out;
   }
 
   public HttpPrinter(OutPort out)
   {
     super(null);
     writer = out;
-    //ostream = System.out;
   }
 
   public static HttpPrinter make (OutPort out)
@@ -65,6 +60,13 @@ public class HttpPrinter extends FilterConsumer
       }
   }
 
+  protected void beforeNode ()
+  {
+    if (sawContentType == null)
+      addHeader("Content-type", "text/xml");
+    beginData();
+  }
+
   public void printHeader(String label, String value)
     throws java.io.IOException
   {
@@ -81,7 +83,6 @@ public class HttpPrinter extends FilterConsumer
     for (int i = 0;  i < num;  i += 2)
       printHeader(headers.elementAt(i).toString(),
 		  headers.elementAt(i + 1).toString());
-    //  if (sawContentType == null) writeRaw("Content-Type: text/html"); FIXME
     writeRaw("\n");
   }
 
@@ -120,13 +121,13 @@ public class HttpPrinter extends FilterConsumer
     if (base == null)
       {
 	if (sawContentType == null)
-	  addHeader("Content-type", "text/html");
+	  addHeader("Content-type", "text/plain");
 	if (writer == null)
 	  writer = new OutPort(ostream); // FIXME use encoding.
 	String style = null;
 	if ("text/html".equalsIgnoreCase(sawContentType))
 	  style = "html";
-	else if ("text/xhtml".equalsIgnoreCase(sawContentType))
+	else if ("application/xhtml+xml".equalsIgnoreCase(sawContentType))
 	  style = "xhtml";
 	else if ("text/plain".equalsIgnoreCase(sawContentType))
 	  style = "plain";
@@ -192,40 +193,16 @@ public class HttpPrinter extends FilterConsumer
   }
 
   /* #ifdef use:java.lang.CharSequence */
-  public Consumer append (CharSequence csq, int start, int end)
-  {
-    if (base == null)
-      {
-        /* #ifdef JAVA5 */
-        sbuf.append(csq, start, end);
-        /* #else */
-        // if (csq == null)
-        //   csq = "null";
-        // sbuf.append(csq.subSequence(start, end).toString());
-        /* #endif */
-      }
-    else
-      base.write(csq, start, end);
-    return this;
-  }
-
-  public Consumer append (CharSequence csq)
-  {
-    if (base == null)
-      {
-        /* #ifdef JAVA5 */
-        sbuf.append(csq);
-        /* #else */
-        // sbuf.append(csq.toString());
-        /* #endif */
-      }
-    else if (csq == null)
-      base.write("null");
-    else
-      base.write(csq, 0, csq.length());
-    return this;
-  }
+  public void write (CharSequence str, int start, int length)
+  /* #else */
+  // public void write (String str, int start, int length)
   /* #endif */
+  {
+    if (base == null)
+      sbuf.append(str, start, start+length);
+    else
+      base.write(str, start, length);
+  }
 
   public void write(char[] buf, int off, int len)
   {
@@ -248,6 +225,17 @@ public class HttpPrinter extends FilterConsumer
       base.endDocument();
     try
       {
+        if (sawContentType == null)
+          addHeader("Content-type", "text/plain");
+        if (sbuf.length() > 0)
+          {
+            String str = sbuf.toString();
+            sbuf.setLength(0);
+            if (writer != null)
+              writer.write(str);
+            else
+              ostream.write(str.getBytes());
+          }
 	// else ???;
 	if (writer != null)
 	  writer.close();
@@ -258,4 +246,28 @@ public class HttpPrinter extends FilterConsumer
       {
       }
   }
+
+  /** Try to reset (delete) any response generated so far.
+   * @param headersAlso if response headers should also be reset.
+   * @return true on success, false if it's too late.
+   */
+  public boolean reset (boolean headersAlso)
+  {
+    if (headersAlso)
+      {
+        headers.clear();
+        sawContentType = null;
+        currentHeader = null;
+        elementNesting = 0;
+      }
+    sbuf.setLength(0);
+    base = null;
+    boolean ok = true;
+    if (ostream != null)
+      {
+        ok = writer == null;
+        writer = null;
+      }
+    return ok;
+  }
 }
