@@ -227,6 +227,14 @@ public class OutPort extends PrintConsumer implements Printable
       print(numberFormat.format((double) v));
   }
 
+  public void print(boolean v)
+  {
+    if (objectFormat == null)
+      super.print(v);
+    else
+      objectFormat.writeBoolean(v, this);
+  }
+
   public void print(String v)
   {
     write(v == null ? "(null)" : v);
@@ -332,6 +340,21 @@ public class OutPort extends PrintConsumer implements Printable
     bout.clearBuffer();
   }
 
+  /** Flush and close this local Writer, but not underlying Writers. */
+  public void closeThis()
+  {
+    try
+      {
+        if (! (base instanceof OutPort && ((OutPort) base).bout == bout))
+          bout.closeThis();
+      }
+    catch (IOException ex)
+      {
+        setError();
+      }
+    WriterManager.instance.unregister(unregisterRef);
+  }
+
   public void close()
   {
     try
