@@ -12,6 +12,15 @@ public abstract class RealNum extends Complex
   public final RealNum re() { return this; }
   public final RealNum im() { return IntNum.zero(); }
 
+  public static RealNum asRealNumOrNull (Object value)
+  {
+    if (value instanceof RealNum)
+      return (RealNum) value;
+    if (value instanceof Float || value instanceof Double)
+      return new DFloNum(((Number) value).doubleValue());
+    return RatNum.asRatNumOrNull(value);
+  }
+
   public abstract boolean isNegative ();
 
   /** Return 1 if >0; 0 if ==0; -1 if <0; -2 if NaN. */
@@ -60,7 +69,7 @@ public abstract class RealNum extends Complex
     return isNegative () ? neg () : this;
   }
 
-  public final RealNum rneg() { return (RealNum) neg(); }
+  public RealNum rneg() { return (RealNum) neg(); }
 
   public boolean isZero ()
   {
@@ -68,11 +77,19 @@ public abstract class RealNum extends Complex
   }
 
   /** Convert to an exact number.
-   * Implements the Scheme inexact->exact (for real numbers).
+   * Implements the Scheme {@code inexact->exact} (for real numbers).
    */
   public RatNum toExact ()
   {
-    return DFloNum.toExact (doubleValue ());
+    return DFloNum.toExact(doubleValue ());
+  }
+
+  public RealNum toInexact ()
+  {
+    if (isExact())
+      return new DFloNum(doubleValue());
+    else
+      return this;
   }
 
   /** Converts a real to an integer, according to a specified rounding mode.
