@@ -77,6 +77,30 @@ public class Q2 extends Scheme
     rt.setFinalColonIsKeyword(true);
     return rt;
   }
+
+  /** Compare two indentation amounts.
+   * An indentation is @{code (numberOfTabs<<16)+numberOfSpaces}.
+   * A comparison is indeterminate if it depends on tab-width - e.g.
+   * the number of tabs in indentation1 is less than indentation2,
+   * but it's the reverse when it comes to spaces.
+   * @return Integer.MIN_VALUE if the comparison is indeterminate;
+   *   otherwise returns the "difference" between the two (where
+   *   tabs count as 8 spaces).
+   */
+  public static int compareIndentation (int indentation1, int indentation2)
+  {
+    int numTabs1 = indentation1 >>> 16;
+    int numTabs2 = indentation1 >>> 16;
+    int numSpaces1 = indentation1 & 0xFF;
+    int numSpaces2 = indentation2 & 0xFF;
+    if (numTabs1 == numTabs2)
+      return numSpaces1 - numSpaces2;
+    else if ((numTabs1 < numTabs2 && numSpaces1 <= numSpaces2)
+             || (numTabs1 > numTabs2 && numSpaces1 >= numSpaces2))
+      return 8 * (numTabs1 - numTabs2);
+    else
+      return Integer.MIN_VALUE;
+  }
 }
 
 class Prompter extends Procedure1
@@ -92,7 +116,7 @@ class Prompter extends Procedure1
       {
 	if (state == '\n')
 	  state = '-';
-	return "#|--Q2:"+line+state+"|#";
+	return "#|--Q2:"+line+"|#"+state;
       }
   }
 }
