@@ -3,6 +3,7 @@ import java.text.Format;
 import java.text.FieldPosition;
 import java.io.Writer;
 import java.io.CharArrayWriter;
+import gnu.lists.Consumer;
 
 public abstract class ReportFormat extends Format
 {
@@ -30,7 +31,7 @@ public abstract class ReportFormat extends Format
    * @param start the index (in args) of the argument to start with
    * @return an integer result(resultCode, nextArg), where
    * nextArg is the index following the last argument processed, and
-   * code is a result code (normally 0, or negative if early termintation
+   * code is a result code (normally 0, or negative if early termintation)
    */
   public abstract int format(Object[] args, int start,
 			     Writer dst, FieldPosition fpos)
@@ -118,6 +119,7 @@ public abstract class ReportFormat extends Format
     return start + nargs;
   }
 
+  /** (Parameters in non-standard order.) */
   public static void print (Writer dst, String str)
     throws java.io.IOException
   {
@@ -127,6 +129,16 @@ public abstract class ReportFormat extends Format
       dst.write(str.toCharArray());
   }
 
+  public static void print (Object value, Consumer out)
+  {
+    if (value instanceof Printable)
+      ((Printable) value).print(out);
+    else
+      // Should we use out.writeObject?
+      // We need make consistent rules to avoid infinite recursion.
+      out.write(value == null ? "null" : value.toString());
+  }
+
   public Object parseObject(String text, java.text.ParsePosition status)
   {
     throw new Error("ReportFormat.parseObject - not implemented");
