@@ -15,17 +15,17 @@ public class URIPath
   extends Path
   /* #ifdef JAVA2 */
   /* #ifdef JAVA5 */
-  // implements Comparable<URIPath>
+  implements Comparable<URIPath>
   /* #else */
-  implements Comparable
+  // implements Comparable
   /* #endif */
   /* #endif */
 {
   /* #ifdef use:java.net.URI */
-  URI uri;
+  final URI uri;
   URIPath (URI uri) { this.uri = uri; } 
   /* #else */
-  // String uri;
+  // final String uri;
   // protected URIPath (String uri) { this.uri = uri; } 
   /* #endif */
 
@@ -115,10 +115,10 @@ public class URIPath
   }
 
   /* #ifdef use:java.net.URI */
-  public URI toURI () { return uri; }
+  public URI toUri () { return uri; }
   public String toURIString () { return uri.toString(); }
   /* #else */
-  // public String toURI () { return uri; }
+  // public String toUri () { return uri; }
   // public String toURIString () { return uri; }
   /* #endif */
 
@@ -191,10 +191,10 @@ public class URIPath
   }
 
   /* #ifndef JAVA5 */
-  public int compareTo (Object obj)
-  {
-    return compareTo((URIPath) obj);
-  }
+  // public int compareTo (Object obj)
+  // {
+  //   return compareTo((URIPath) obj);
+  // }
   /* #endif */
 
   public boolean equals (Object obj)
