@@ -215,7 +215,7 @@ public class Shell
                                InPort inp, Consumer out, OutPort perr,
                                java.net.URL url, SourceMessages messages)
   {
-    Language saveLanguage = Language.getDefaultLanguage();
+    Language saveLanguage = Language.setSaveCurrent(language);
     Lexer lexer = language.getLexer(inp, messages);
     // Wrong for the case of '-f' '-':
     //boolean interactive = inp instanceof TtyInPort;
@@ -228,8 +228,6 @@ public class Shell
 	saveConsumer = ctx.consumer;
 	ctx.consumer = out;
       }
-    if (language != saveLanguage)
-      Language.setDefaultLanguage(language);
     try
       {
         Thread thread = Thread.currentThread();
@@ -299,8 +297,7 @@ public class Shell
       {
 	if (out != null)
 	  ctx.consumer = saveConsumer;
-        if (language != saveLanguage)
-          Language.setDefaultLanguage(saveLanguage);
+        Language.restoreCurrent(saveLanguage);
       }
     return null;
   }
@@ -438,16 +435,25 @@ public class Shell
       }
     catch (Throwable e)
       {
+        Class clas;
+        try
+          {
+            clas = Class.forName(fname);
+          }
+        catch (Throwable ex)
+          {
+            System.err.println("Cannot read file "+e.getMessage());
+            return false;
+          }
         try
           {
-            Class clas = Class.forName(fname);
             CompiledModule cmodule = CompiledModule.make(clas, language);
             cmodule.evalModule(Environment.getCurrent(), OutPort.outDefault());
             return true;
           }
         catch (Throwable ex)
           {
-            System.err.println("Cannot read file "+e.getMessage());
+            ex.printStackTrace();
             return false;
           }
       }
@@ -518,7 +524,7 @@ public class Shell
     ModuleManager manager = ModuleManager.getInstance();
     ModuleInfo minfo = manager.findWithSourcePath(port.getName());
     Compilation comp
-      = language.parse(port, messages, Language.PARSE_FOR_EVAL, minfo);
+      = language.parse(port, messages, Language.PARSE_IMMEDIATE, minfo);
     CallContext ctx = CallContext.getInstance();
     ctx.values = Values.noArgs;
     Object inst = ModuleExp.evalModule1(env, comp, url, null);
