@@ -44,20 +44,27 @@ public class AutoloadSyntax extends Syntax implements Externalizable
 
   public void print(java.io.PrintWriter ps)
   {
-    ps.print ("#<syntax ");
+    ps.print(toString());
+  }
+
+  public String toString ()
+  {
+    StringBuffer sbuf = new StringBuffer(100);
+    sbuf.append("#<syntax ");
     if (getName() != null)
       {
-	ps.print(getName());
-	ps.print(' ');
+	sbuf.append(getName());
+	sbuf.append(' ');
       }
     if (loaded != null)
-      ps.print ("autoloaded>");
+      sbuf.append("autoloaded>");
     else
       {
-	ps.print ("autoload ");
-	ps.print (className);
-	ps.print (">");
+	sbuf.append("autoload ");
+	sbuf.append(className);
+	sbuf.append(">");
       }
+    return sbuf.toString();
   }
 
   private void throw_error (String prefix)
@@ -70,30 +77,16 @@ public class AutoloadSyntax extends Syntax implements Externalizable
   /** Load the class named in className. */
   void load ()
   {
-    Environment env = this.env != null ? this.env : Environment.getCurrent();
     String name = this.getName();
     try
       {
 	Object value = Class.forName (className).newInstance ();
-	if (value instanceof ModuleBody)
+        if (value instanceof Syntax)
 	  {
-	    gnu.kawa.reflect.ClassMemberConstraint.defineAll(value, env);
-	    ((ModuleBody) value).run();
-	    try
-	      {
-		value = env.getFunction(name);
-	      }
-	    catch (Exception ex)
-	      {
-		value = null;
-	      }
-	    if (value == null || value == this
-		|| !(value instanceof Syntax))
-	      throw_error("syntax not found in ");
-	    loaded = (Syntax) value;
+	    loaded = (Syntax) value;	
+	    if (name != null && loaded.getName() == null)
+	      loaded.setName(name);
 	  }
-	else if (value instanceof Syntax)
-          loaded = (Syntax) value;
 	else
 	  throw_error ("failed to autoload valid syntax object ");
       }
@@ -103,14 +96,13 @@ public class AutoloadSyntax extends Syntax implements Externalizable
       { throw_error ("failed to instantiate class "); }
     catch (IllegalAccessException ex)
       { throw_error ("illegal access in class "); }
-    catch (UnboundSymbol e)
+    catch (UnboundLocationException e)
       { throw_error ("missing symbol '" + e.getMessage () + "' "); }
     catch (WrongArguments ex)
       { throw_error ("type error"); }
   }
 
-  public boolean scanForDefinitions (Pair st, java.util.Vector forms,
-                                     ScopeExp defs, Translator tr)
+  public void scanForm (Pair st, ScopeExp defs, Translator tr)
   {
     if (loaded == null)
       {
@@ -120,11 +112,11 @@ public class AutoloadSyntax extends Syntax implements Externalizable
 	  }
 	catch (RuntimeException e)
 	  {
-	    tr.syntaxError (e.getMessage ());
-            return false;
+	    tr.syntaxError(e.getMessage ());
+            return;
 	  }
       }
-    return loaded.scanForDefinitions(st, forms, defs, tr);
+    loaded.scanForm(st, defs, tr);
   }
 
   public Expression rewriteForm (Pair form, Translator tr)
@@ -144,7 +136,16 @@ public class AutoloadSyntax extends Syntax implements Externalizable
 	    return tr.syntaxError (e.getMessage ());
 	  }
       }
-    return loaded.rewriteForm(form, tr);
+    Syntax saveSyntax = tr.currentSyntax;
+    tr.currentSyntax = loaded;
+    try
+      {
+	return loaded.rewriteForm(form, tr);
+      }
+    finally
+      {
+	tr.currentSyntax = saveSyntax;
+      }
   }
 
   public void writeExternal(ObjectOutput out) throws IOException
