@@ -67,27 +67,6 @@ public class Translator extends Compilation
     LispLanguage.getNamedPartLocation.setDeclaration(getNamedPartDecl);
   }
 
-  /** Return true if decl is lexical and not fluid. */
-  public boolean isLexical (Declaration decl)
-  {
-    if (decl == null)
-      return false;
-    if (! decl.isFluid())
-      return true;
-    ScopeExp scope = currentScope();
-    ScopeExp context = decl.getContext();
-    for (;; scope = scope.outer)
-      {
-	if (scope == null)
-	  return false;
-	if (scope == context)
-	  return true;
-	if (scope instanceof LambdaExp
-	    && ! ((LambdaExp) scope).getInlineOnly())
-	  return false;
-      }
-  }
-
   private static Expression errorExp = new ErrorExp ("unknown syntax error");
 
   public Translator (Language language, SourceMessages messages, NameLookup lexical)
@@ -657,7 +636,7 @@ public class Translator extends Compilation
                   break;
                 LambdaExp caller = (LambdaExp) scope;
                 ClassExp cexp = (ClassExp) scope.outer;
-                ClassType ctype = (ClassType) cexp.getType();
+                ClassType ctype = (ClassType) cexp.getClassType();
                 Object part = SlotGet.lookupMember(ctype, dname, ctype);
                 boolean contextStatic
                   = (caller == cexp.clinitMethod
@@ -707,10 +686,7 @@ public class Translator extends Compilation
 	boolean separate = getLanguage().hasSeparateFunctionNamespace();
         if (decl != null)
           {
-            if (! isLexical(decl)
-                || (separate && decl.isProcedureDecl()))
-              decl = null;
-            else if (current_scope instanceof TemplateScope && decl.needsContext())
+            if (current_scope instanceof TemplateScope && decl.needsContext())
               cdecl = ((TemplateScope) current_scope).macroContext;
             else if (decl.getFlag(Declaration.FIELD_OR_METHOD)
                      && ! decl.isStatic())
@@ -808,8 +784,6 @@ public class Translator extends Compilation
                 && decl.getConstantValue() instanceof kawa.standard.object)
               return QuoteExp.getInstance(Object.class);
 
-            // In Scheme/Lisp a variable cannot donate multiple values.
-            decl.setFlag(Declaration.IS_SINGLE_VALUE);
             if (decl.getContext() instanceof PatternScope)
               return syntaxError("reference to pattern variable "+decl.getName()+" outside syntax template");
           }
@@ -828,7 +802,7 @@ public class Translator extends Compilation
     else if (exp == Special.abstractSpecial)
       return QuoteExp.abstractExp;
     else
-      return QuoteExp.getInstance(Quote.quote(exp, this));
+      return QuoteExp.getInstance(Quote.quote(exp, this), this);
   }
 
   public static void setLine(Expression exp, Object location)
@@ -887,7 +861,9 @@ public class Translator extends Compilation
 
   public void setLineOf (Expression exp)
   {
-    if (exp instanceof QuoteExp)
+    // "Special" QuoteExps may be shared, but the position gets set (in the
+    // call to QuoteExp.getInstance at end of re-write) for normal ones.
+    if (exp instanceof QuoteExp) 
       return;
     exp.setLocation(this);
   }
@@ -899,7 +875,7 @@ public class Translator extends Compilation
     try
       {
 	Expression texp = rewrite_car(typeSpecPair, false);
-        texp = new InlineCalls(this).walk(texp);
+        texp = InlineCalls.inlineCalls(texp, this);
 	if (texp instanceof ErrorExp)
 	  return null;
         Type type = getLanguage().getTypeFor(texp);
@@ -1116,9 +1092,9 @@ public class Translator extends Compilation
    * in a <code>SyntaxForm</code>); otherwise <code>null</code>.
    */
 
-  public Object scanBody (Object body, ScopeExp defs, boolean makeList)
+  public LList scanBody (Object body, ScopeExp defs, boolean makeList)
   {
-    Object list = makeList ? LList.Empty : null;
+    LList list = makeList ? LList.Empty : null;
     Pair lastPair = null;
     while (body != LList.Empty)
       {
@@ -1130,10 +1106,10 @@ public class Translator extends Compilation
 	      {
 		setCurrentScope(sf.getScope());
 		int first = formStack.size();
-		Object f = scanBody(sf.getDatum(), defs, makeList);
+		LList f = scanBody(sf.getDatum(), defs, makeList);
                 if (makeList)
                   {
-                    f = wrapSyntax(f, sf);
+                    f = (LList) SyntaxForms.fromDatumIfNeeded(f, sf);
                     if (lastPair == null)
 		      return f;
                     lastPair.setCdrBackdoor(f);
@@ -1210,8 +1186,8 @@ public class Translator extends Compilation
     current_scope = defs;
     try
       {
-	scanBody(exp, defs, false);
-	if (formStack.size() == first)
+        LList list = scanBody(exp, defs, true);
+	if (list.isEmpty())
 	  formStack.add(syntaxError ("body with no expressions"));
 	int ndecls = defs.countNonDynamicDecls();
 	if (ndecls != 0)
@@ -1221,6 +1197,7 @@ public class Translator extends Compilation
 	      inits[i] = QuoteExp.undefined_exp;
 	    defs.inits = inits;
 	  }
+        rewriteBody(list);
 	Expression body = makeBody(first, null);
 	setLineOf(body);
 	if (ndecls == 0)
@@ -1236,32 +1213,27 @@ public class Translator extends Compilation
       }
   }
 
-  /* Rewrite forms on formStack above first. */
-  public void rewriteBody (int first)
+  private void rewriteBody (LList forms)
   {
-    int nforms = formStack.size() - first;
-    if (nforms == 0)
-      return;
-    else if (nforms == 1)
+    while (forms != LList.Empty)
       {
-	Object f = formStack.pop();
-	rewriteInBody(f);
-      }
-    else
-      {
-	Object[] forms = new Object [nforms];
-	for (int i = 0; i < nforms; i++)
-	  forms[i] = formStack.elementAt(first + i);
-	formStack.setSize(first);
-	for (int i = 0; i < nforms; i++)
-	  rewriteInBody(forms[i]);
+        Pair pair = (Pair) forms;
+        Object saved = pushPositionOf(pair);
+        try
+          {
+            rewriteInBody(pair.getCar());
+          }
+        finally
+          {
+            popPositionOf(saved);
+          }
+        forms = (LList) pair.getCdr();
       }
   }
 
   /** Combine a list of zero or more expression forms into a "body". */
-  public Expression makeBody(int first, ScopeExp scope)
+  private Expression makeBody(int first, ScopeExp scope)
   {
-    rewriteBody(first);
     int nforms = formStack.size() - first;
     if (nforms == 0)
       return QuoteExp.voidExp; 
@@ -1419,10 +1391,10 @@ public class Translator extends Compilation
     processAccesses();
 
     setModule(mexp);
-    Compilation save_comp = Compilation.getCurrent();
+    Compilation save_comp = Compilation.setSaveCurrent(this);
     try
       {
-	Compilation.setCurrent(this);
+        rewriteInBody(popForms(firstForm));
 	mexp.body = makeBody(firstForm, mexp);
         // In immediate mode need to preseve Declaration for current "seesion".
         if (! immediate)
@@ -1430,7 +1402,7 @@ public class Translator extends Compilation
       }
     finally
       {
-	Compilation.setCurrent(save_comp);
+	Compilation.restoreCurrent(save_comp);
       }
 
     /* DEBUGGING:
