@@ -21,6 +21,31 @@
 (define (symbol->string (s <symbol>)) :: constant-string
   (s:toString))
 
+(define-procedure symbol=?
+  (lambda (s1::symbol s2::symbol)::boolean
+	  (gnu.mapping.Symbol:equals s1 s2))
+  (lambda (s1::symbol s2::symbol #!rest r)::boolean
+	  (and (gnu.mapping.Symbol:equals s1 s2)
+	       (apply symbol=? s2 r))))
+
+(define (symbol-local-name s::symbol) ::constant-string
+  (s:getLocalPart))
+
+(define (symbol-namespace s::symbol) ::namespace
+  (s:getNamespace))
+
+(define (symbol-namespace-uri s::symbol) ::constant-string
+  (s:getNamespaceURI))
+
+(define (symbol-prefix s::symbol) ::constant-string
+  (s:getPrefix))
+
+(define (namespace-uri (ns::gnu.mapping.Namespace)) ::string
+  (invoke ns 'getName))
+
+(define (namespace-prefix (ns::gnu.mapping.Namespace)) ::string
+  (invoke ns 'getPrefix))
+
 (define (string->symbol (str <string>))
   (gnu.mapping.SimpleSymbol:valueOf (str:toString)))
 
@@ -30,20 +55,6 @@
 (define (values #!rest (args :: <Object[]>))
   (invoke-static <gnu.mapping.Values> 'make args))
 
-(define-syntax (provide form)
-  (syntax-case form ()
-    ((provide 'feature)
-     (cons (syntax define-constant)
-	   (cons (datum->syntax-object
-		  form
-		  (string->symbol
-		   (string-append "%provide%"
-				  (symbol->string
-				   (syntax-object->datum (syntax feature))))))
-		 (syntax (:: <int> 123)))))
-    ((_ . rest)
-     (syntax-error form "provide requires a quoted feature-name"))))
-	   
 
 (define (environment-bound? (env :: <gnu.mapping.Environment>) sym)
   :: <boolean>
@@ -85,11 +96,7 @@
    (after)))
 
 (define (force arg)
-  (cond ((instance? arg <kawa.lang.Promise>)
-	 (invoke (as <kawa.lang.Promise> arg) 'force))
-	((instance? arg <gnu.mapping.Future>)
-	 (invoke (as <gnu.mapping.Future> arg) 'waitForResult))
-	(else arg)))
+  (kawa.lang.Promise:force arg))
 
 ;;; The one-argument case is a standard DSSSL procedure.
 ;;; The multi-argument extension matches Guile.
@@ -116,16 +123,10 @@
   (make <gnu.kawa.util.Parameter> init converter))
 |#
 
-(define-syntax test-begin
-  (syntax-rules ()
-    ((test-begin suite-name)
-     (begin
-       (cond-expand (srfi-64 #!void) (else (require 'srfi-64)))
-       (%test-begin suite-name #f)))
-    ((test-begin suite-name count)
-     (begin
-       (cond-expand (srfi-64 #!void) (else (require 'srfi-64)))
-       (%test-begin suite-name count)))))
+(define (gentemp) :: <symbol>
+  (invoke-static <gnu.expr.Symbols> 'gentemp))
 
-(define (namespace name) :: <gnu.mapping.Namespace>
-  (invoke-static <gnu.mapping.Namespace> 'getInstance name))
+(define (add-procedure-properties
+	 (proc :: <gnu.expr.GenericProc>)
+	 #!rest (args :: <object[]>)) :: <void>
+  (invoke proc 'setProperties args))
