@@ -26,12 +26,6 @@
     ((define name value)
      (%define name 0 #!null value))))
 
-;; For now the same as plain define.
-(%define-syntax define-for-syntax
-  (syntax-rules (::)
-    ((define-for-syntax . rest)
-     (define . rest))))
-
 (%define-syntax define-private
   (syntax-rules (:: $lookup$)
     ((define-private ($lookup$ part1 'part2) :: type value)
@@ -64,12 +58,12 @@
 (%define-syntax syntax->expression
   (syntax-rules ()
     ((syntax->expression x)
-     (invoke-static <kawa.lang.SyntaxForm> 'rewrite x))))
+     (kawa.lang.SyntaxForms:rewrite x))))
 
 (%define-syntax syntax-body->expression
   (syntax-rules ()
     ((syntax-body->expression x)
-     (invoke-static <kawa.lang.SyntaxForm> 'rewriteBody x))))
+     (kawa.lang.SyntaxForms:rewriteBody x))))
 
 (%define-syntax if
   (lambda (x)
