@@ -6,6 +6,7 @@ import gnu.expr.*;
 import gnu.text.SourceMessages;
 import gnu.text.SyntaxException;
 import gnu.lists.*;
+import java.util.ArrayList;
 import java.util.Vector;
 import gnu.bytecode.ClassType;
 import gnu.kawa.servlet.HttpRequestContext;
@@ -105,10 +106,10 @@ public class repl extends Procedure0or1
     printOption(out, "--servlet", "Generate a servlet");
     printOption(out, "--module-static", "Top-level definitions are by default static");
 
-    Vector keys = Compilation.options.keys();
+    ArrayList<String> keys = Compilation.options.keys();
     for (int i = 0; i < keys.size(); ++i)
       {
-        String name = (String) keys.get(i);
+        String name = keys.get(i);
         printOption(out, "--" + name, Compilation.options.getDoc(name));
       }
         
@@ -508,9 +509,9 @@ public class repl extends Procedure0or1
                 gnu.kawa.servlet.KawaHttpHandler.addAutoHandler(args[iArg-1], args[iArg]);
               }
             catch (java.io.IOException ex)
-	      {
-		throw new RuntimeException(ex);
-	      }
+              {
+                throw new RuntimeException(ex);
+              }
             catch (NoClassDefFoundError ex)
               {
                 System.err.println("kawa: HttpServer classes not found");
@@ -602,10 +603,6 @@ public class repl extends Procedure0or1
           {
             Compilation.moduleStatic = Compilation.MODULE_STATIC_RUN;
           }
-	else if (arg.equals("--fewer-classes"))
-	  {
-	    Compilation.fewerClasses = true;
-	  }
 	else if (arg.equals("--no-inline")
 		 || arg.equals("--inline=none"))
 	  {
@@ -619,7 +616,6 @@ public class repl extends Procedure0or1
 	  }
 	else if (arg.equals("--cps"))
 	  {
-	    Compilation.fewerClasses = true;
 	    Compilation.defaultCallConvention
 	      = Compilation.CALL_WITH_CONTINUATIONS;
 	  }
@@ -666,7 +662,7 @@ public class repl extends Procedure0or1
 		if (previousLanguage == null)
 		  Language.setDefaults(lang);
                 else
-                  Language.setDefaultLanguage(lang);
+                  Language.setCurrentLanguage(lang);
 		previousLanguage = lang;
 	      }
 	    else
