@@ -6,8 +6,7 @@ import gnu.bytecode.ArrayType;
 import gnu.mapping.*;
 import gnu.math.IntNum;
 import gnu.expr.*;
-import java.util.Hashtable;
-import java.util.Vector;
+import java.util.*;
 import gnu.text.Lexer;
 import gnu.text.SourceMessages;
 import gnu.kawa.lispexpr.*;
@@ -32,8 +31,8 @@ public class Scheme extends LispLanguage
 
   public static final gnu.kawa.reflect.InstanceOf instanceOf;
   public static final Not not;
-  public static final kawa.standard.map map;
-  public static final kawa.standard.map forEach;
+  public static final gnu.kawa.functions.Map map;
+  public static final gnu.kawa.functions.Map forEach;
   public static final gnu.kawa.functions.IsEq isEq;
   public static final gnu.kawa.functions.IsEqv isEqv;
   public static final gnu.kawa.functions.IsEqual isEqual;
@@ -43,6 +42,8 @@ public class Scheme extends LispLanguage
   public static final NumberCompare numGEq;
   public static final NumberCompare numLss;
   public static final NumberCompare numLEq;
+  public static final NumberPredicate isOdd;
+  public static final NumberPredicate isEven;
 
   public static final Apply apply;
   public static final ApplyToArgs applyToArgs;
@@ -62,12 +63,12 @@ public class Scheme extends LispLanguage
     applyFieldDecl
       = Declaration.getDeclarationFromStatic("kawa.standard.Scheme",
                                              "applyToArgs");
-    map = new map(true, applyToArgs, applyFieldDecl);
-    forEach = new map(false, applyToArgs, applyFieldDecl);
     apply = new Apply("apply", applyToArgs);
     isEq = new gnu.kawa.functions.IsEq(instance, "eq?");
     isEqv = new gnu.kawa.functions.IsEqv(instance, "eqv?", isEq);
     isEqual = new gnu.kawa.functions.IsEqual(instance, "equal?");
+    map = new gnu.kawa.functions.Map(true, applyToArgs, applyFieldDecl, isEq);
+    forEach = new gnu.kawa.functions.Map(false, applyToArgs, applyFieldDecl, isEq);
     numEqu = NumberCompare.make(instance, "=",
                                 NumberCompare.TRUE_IF_EQU);
     numGrt = NumberCompare.make(instance, ">",
@@ -78,6 +79,8 @@ public class Scheme extends LispLanguage
                                 NumberCompare.TRUE_IF_LSS);
     numLEq = NumberCompare.make(instance, "<=",
                                 NumberCompare.TRUE_IF_LSS|NumberCompare.TRUE_IF_EQU);
+    isOdd = new NumberPredicate(instance, "odd?", NumberPredicate.ODD);
+    isEven = new NumberPredicate(instance, "even?", NumberPredicate.EVEN);
 
     instance.initScheme();
 
@@ -241,8 +244,8 @@ public class Scheme extends LispLanguage
       defProcStFld("zero?", "kawa.lib.numbers");
       defProcStFld("positive?", "kawa.lib.numbers");
       defProcStFld("negative?", "kawa.lib.numbers");
-      defProcStFld("odd?", "kawa.lib.numbers");
-      defProcStFld("even?", "kawa.lib.numbers");
+      defProcStFld("odd?", "kawa.standard.Scheme", "isOdd");
+      defProcStFld("even?", "kawa.standard.Scheme", "isEven");
       defProcStFld("max", "kawa.lib.numbers");
       defProcStFld("min", "kawa.lib.numbers");
       defProcStFld("+", "gnu.kawa.functions.AddOp", "$Pl");
@@ -368,7 +371,7 @@ public class Scheme extends LispLanguage
       defProcStFld("map", "kawa.standard.Scheme", "map");
       defProcStFld("for-each", "kawa.standard.Scheme", "forEach");
       defProcStFld("call-with-current-continuation",
-                   "kawa.standard.callcc", "callcc");
+                   "gnu.kawa.functions.CallCC", "callcc");
       defProcStFld("call/cc", "kawa.standard.callcc", "callcc");
       defProcStFld("force", "kawa.lib.misc");
 
@@ -675,6 +678,7 @@ public class Scheme extends LispLanguage
                    "define_variable");
       defSntxStFld("define-member-alias", "kawa.standard.define_member_alias",
                    "define_member_alias");
+      defSntxStFld("define-enum", "gnu.kawa.slib.enums");
       defSntxStFld("import", "kawa.lib.syntax");
       defSntxStFld("require", "kawa.standard.require", "require");
       defSntxStFld("module-name", "kawa.standard.module_name",
@@ -945,15 +949,16 @@ public class Scheme extends LispLanguage
     return getInstance().getTypeFor(exp);
   }
 
-  static Hashtable types;
+  static HashMap<String,Type> types;
+  static HashMap<Type,String> typeToStringMap;
 
-  public static Type getNamedType (String name)
+  static synchronized HashMap<String,Type> getTypeMap ()
   {
     if (types == null)
       {
 	booleanType
 	  = new LangPrimType(Type.booleanType, Scheme.getInstance());
-	types = new Hashtable ();
+	types = new HashMap<String,Type> ();
 	types.put ("void", LangPrimType.voidType);
 	types.put ("int", LangPrimType.intType);
 	types.put ("char", LangPrimType.charType);
@@ -981,12 +986,6 @@ public class Scheme extends LispLanguage
 	types.put ("pair", ClassType.make("gnu.lists.Pair"));
 	types.put ("pair-with-position",
 		   ClassType.make("gnu.lists.PairWithPosition"));
-        /* #ifdef use:java.lang.CharSequence */
-	types.put ("string", ClassType.make("java.lang.CharSequence"));
-        /* #else */
-        // /* better would be a union of CharSeq and java.lang.String. */
-	// types.put ("string", ClassType.make("gnu.lists.FString"));
-        /* #endif */
 	types.put ("constant-string", ClassType.make("java.lang.String"));
 	types.put ("abstract-string", ClassType.make("gnu.lists.CharSeq"));
 	types.put ("character", ClassType.make("gnu.text.Char"));
@@ -1017,6 +1016,12 @@ public class Scheme extends LispLanguage
         types.put ("document", ClassType.make("gnu.kawa.xml.KDocument"));
         types.put ("readtable", ClassType.make("gnu.kawa.lispexpr.ReadTable"));
       }
+    return types;
+  }
+
+  public static Type getNamedType (String name)
+  {
+    getTypeMap();
     Type type = (Type) types.get(name);
     if (type == null
 	&& (name.startsWith("elisp:") || name.startsWith("clisp:")))
@@ -1072,6 +1077,28 @@ public class Scheme extends LispLanguage
     return Type.make(clas);
   }
 
+  public String formatType (Type type)
+  {
+    // FIXME synchronize
+    if (typeToStringMap == null)
+      {
+        typeToStringMap = new HashMap<Type,String>();
+        for (java.util.Map.Entry<String,Type> e : getTypeMap().entrySet())
+          {
+            String s = e.getKey();
+            Type t = e.getValue();
+            typeToStringMap.put(t, s);
+            Type it = t.getImplementationType();
+            if (it != t)
+              typeToStringMap.put(it, s);
+          }
+      }
+    String str = typeToStringMap.get(type);
+    if (str != null)
+      return str;
+    return super.formatType(type);
+  }
+
   public static Type string2Type (String name)
   {
     Type t;
@@ -1335,7 +1362,7 @@ public class Scheme extends LispLanguage
           {
             Symbol tsymbol = namespace.getSymbol(cname.intern());
             Expression texp = tr.rewrite(tsymbol, false);
-            texp = new InlineCalls(tr).walk(texp);
+            texp = InlineCalls.inlineCalls(texp, tr);
             if (! (texp instanceof ErrorExp))
               type = tr.getLanguage().getTypeFor(texp);
           }
