@@ -75,6 +75,7 @@ public class define extends Syntax
       }
     if (makeConstant)
       decl.setFlag(Declaration.IS_CONSTANT);
+    decl.setFlag(Declaration.IS_SINGLE_VALUE);
 
     if ((options & 2) != 0)
       {
@@ -96,14 +97,9 @@ public class define extends Syntax
 	decl.noteValue(lexp);
       }
 
-    if (defs instanceof ModuleExp)
-      {
-	if (! makePrivate)
-	  {
-            if (! Compilation.inlineOk || tr.immediate)
-	      decl.setCanWrite(true);
-	  }
-      }
+    if (defs instanceof ModuleExp && ! makePrivate
+        && (! Compilation.inlineOk || tr.sharedModuleDefs()))
+      decl.setCanWrite(true);
 
     if ((options & 1) != 0)
       {
@@ -132,11 +128,14 @@ public class define extends Syntax
       return tr.syntaxError(getName(options) + " is only allowed in a <body>");
     Declaration decl = (Declaration) name;
 
-    Expression texp = decl.getTypeExp();
-    if (texp instanceof LangExp)
+    if (decl.getFlag(Declaration.TYPE_SPECIFIED))
       {
-        Pair typeSpecPair = (Pair) ((LangExp) texp).getLangValue(); 
-        decl.setType(tr.exp2Type(typeSpecPair));
+        Expression texp = decl.getTypeExp();
+        if (texp instanceof LangExp)
+          {
+            Pair typeSpecPair = (Pair) ((LangExp) texp).getLangValue(); 
+            decl.setType(tr.exp2Type(typeSpecPair));
+          }
       }
 
     Expression value;
@@ -154,7 +153,7 @@ public class define extends Syntax
 	value = tr.rewrite (p4.getCar());
 	decl.noteValue((decl.context instanceof ModuleExp && ! makePrivate
 			&& decl.getCanWrite())
-		       ? null : value);
+		        ? null : value);
       }
     SetExp sexp = new SetExp(decl, value);
     sexp.setDefining (true);
