@@ -100,6 +100,7 @@ public class require extends Syntax
     map("gui", SLIB_PREFIX + "gui");
     map("swing-gui", SLIB_PREFIX + "swing");
     map("android-defs", "gnu.kawa.android.defs");
+    map("syntax-utils", SLIB_PREFIX + "syntaxutils");
   }
 
   public static String mapFeature(String featureName)
@@ -272,15 +273,14 @@ public class require extends Syntax
             && info.getState() < Compilation.RESOLVED)
           {
             // Oops.  We found a cycle.
-            tr.pushPendingImport(info, defs);
-            tr.pendingImports.push(Integer.valueOf(forms.size()));
+            tr.pushPendingImport(info, defs, forms.size());
             return true;
           }
       }
 
     ClassType type = info.getClassType();
     String tname = type.getName();
-    boolean immediate = tr.immediate && defs instanceof ModuleExp;
+    boolean sharedModule = tr.sharedModuleDefs();
     boolean isRunnable = (info.getState() < Compilation.RESOLVED
                           ? info.getCompilation().makeRunnable()
                           : type.isSubtype(Compilation.typeRunnable));
@@ -328,8 +328,7 @@ public class require extends Syntax
           {
             String iname = tname.replace('.', '$') + "$instance";
             decl = new Declaration(SimpleSymbol.valueOf(iname), type);
-            if (! immediate)
-              decl.setPrivate(true);
+            decl.setPrivate(true);
             decl.setFlag(Declaration.IS_CONSTANT
                          |Declaration.MODULE_REFERENCE);
             defs.addDeclaration(decl);
@@ -409,7 +408,7 @@ public class require extends Syntax
         if (! isImportedInstance)
           {
             fref.setDontDereference(true);
-            if (! immediate)
+            if (! sharedModule)
               adecl.setPrivate(true);
           } 
         // Imported variables should be read-only.
