@@ -1,10 +1,11 @@
-(test-init "macros" 101)
+(test-init "macros" 102)
 
 (test 'ok 'letxx (let ((xx #f)) (cond (#t xx 'ok))))
 
 (test 'ok 'let=> (let ((=> #f)) (cond (#t => 'ok))))
 
-(load (string-append src-prefix "mac1.scm"))
+(begin-for-syntax
+ (load (string-append src-prefix "mac1.scm")))
 (test '(1 2) 'something (something 1 2))
 
 (test '(2 3) 'something (something 2 3))
@@ -280,9 +281,9 @@
     ((def-and-use-of-x val)
      ; x below originates from this macro:
      (begin (define x val) x))))
-(define x 1)
+(define x1 1)
 (test 2 'mzscheme-lang-12.3.5-1 (def-and-use-of-x 2))
-(test 1 'mzscheme-lang-12.3.5-2 x)
+(test 1 'mzscheme-lang-12.3.5-2 x1)
 
 ;; From FLT MzScheme Manual section 12.3.5 Macro-Gnerated Top-Level
 (define-syntax def-and-use
@@ -386,7 +387,8 @@
   ((incx2) <void>
    (set! (slot (this) x) (+ 1 (slot (this) x)))))
 (define xinstance (make <xclass>))
-(invoke xinstance 'incx)
+(with-compile-options warn-invoke-unknown-method: #f
+		      (invoke xinstance 'incx))
 (test 1 'xclass (slot xinstance x))
 
 (define x (list "X1" "X2"))
@@ -436,13 +438,15 @@
 			    (quasisyntax (list x ,x))))))
 	  (m))))
 
-(set! fail-expected "define-for-syntax for now is just a synonym for define")
-(define x-72-x3 1)
-(define-for-syntax x-72-x3 2)
-(test '(1 2) 'srfi-72-example-3
-       (let-syntax ((m (lambda (form)
-                    (quasisyntax (list x-72-x3 ,x-72-x3)))))
-	 (m)))
+(begin
+  ;; Note we need to compile define and define-for-syntax
+  ;; in the same comilation unit for it to make sense.
+  (define x-72-x3 1)
+  (define-for-syntax x-72-x3 2)
+  (test '(1 2) 'srfi-72-example-4
+	(let-syntax ((m (lambda (form)
+			  (quasisyntax (list x-72-x3 ,x-72-x3)))))
+	  (m))))
 
 ;; Based on Savannah bug #17984 Chris Wegrzyn <chris.wegrzyn@gmail.com>
 ;; Compile time error in expansion of hygienic macros ending in literals
@@ -480,7 +484,7 @@
   (test 3 'savannah-bug-18105 ((crashing-syntax (arg1 arg2) 3) 1 2)))
 
 ;; Luis Casillas <luis@casillas.org> posted to Kawa list 2007-02-02:
-(define (alter-syntax-datum proc stx)
+(define-for-syntax (alter-syntax-datum proc stx)
   ;; must use define-syntax-datum in PLT
   (datum->syntax-object stx (proc (syntax-object->datum stx))))
 (define-syntax define-symbol-altering-macro
@@ -571,3 +575,24 @@
    (syntax-rules ()
      ((foo-26993 "foo") 'ok)))
 (test 'ok 'test-savannah-26993 (foo-26993 "foo"))
+
+;; Savannah bug #27042: Bad interaction between syntax-rules and call-with-values
+;; (Though the was actually in the hygiene handling of lambda,
+;; and had nothing to do specifically with call-with-values.)
+(test '(0 10 0) 'test-savannah-27042
+      (let-syntax ((dlet
+		    (syntax-rules ()
+		      ((dlet (var val) body)
+		       (let ((saved var))
+			 (set! var val)
+			 (call-with-values (lambda () body)
+			   (lambda (result)
+			     (set! var saved)
+			     result)))))))
+	(let* ((x 0)
+	       (x0 x)
+	       (x1 
+		(dlet (x (+ x 10))
+		      x))
+	       (x2 x))
+	  (list x0 x1 x2))))
