@@ -1,8 +1,9 @@
 ;; The next line is just to test the case that we don't extend ModuleBody.
 (module-extends <pair>)
+(module-implements java.io.Closeable)
 (module-export list-length-1 list-length-3 classify list1234 <A8711>
 	       length-diff1 length-diff2 length-diff3 make-literal make-pair
-	       my-array-7 mod2-v5)
+	       my-array-7 mod2-v5 close)
 (require <module1>)
 (require <module1a>)
 (define (list-length-1 x) :: <integer>
@@ -13,6 +14,7 @@
 (define list1234 (deldup (call-to-first '((1 1 2 3 4 4)))))
 
 ;; Caused VerifyError
+(define-variable internal-node-name)
 (define name internal-node-name)
 (define (classify)
   (let node-loop ((classes '()))
@@ -72,3 +74,5 @@
   (let ((loc (location mod1-v5)))
     (+ 10 (loc))))
 
+(define (close) :: void
+  (set! my-array-7 #f))
